@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -62,7 +62,7 @@ static void updateDeviceHostNameInfo(HostAddr addr, char* symbolic, int actualDe
 	  symbolic[MAX_LEN_SYM_HOST_NAME-2] = '\0';
 
 	for(i=0; i<strlen(symbolic); i++)
-	  if(isupper(symbolic[i])) tolower(symbolic[i]);
+	  if(isupper(symbolic[i])) symbolic[i] = tolower(symbolic[i]);
 
 	setResolvedName(el, symbolic, type);
       }
@@ -153,7 +153,7 @@ static void resolveAddress(HostAddr *hostAddr, short keepAddressNumeric) {
   /* First check whether the address we search for is cached... */
   if((data_data.dptr != NULL) &&
      (data_data.dsize == (sizeof(StoredAddress))) &&
-     (myGlobals.actTime - ((StoredAddress*)data_data.dptr)->recordCreationTime < CONST_DNSCACHE_LIFETIME) ) {
+     (myGlobals.actTime - ((StoredAddress*)data_data.dptr)->recordCreationTime < CONST_DNSCACHE_LIFETIME)) {
     StoredAddress *retrievedAddress;
 
     retrievedAddress = (StoredAddress*)data_data.dptr;
@@ -389,10 +389,10 @@ static void resolveAddress(HostAddr *hostAddr, short keepAddressNumeric) {
              * Treat as NOT FOUND, but use reportedFreaky to put out one message per run.
              * Known to happen under Linux, other OSes uncertain...
              */
-	if (reportedFreaky == FALSE) {
-                reportedFreaky = TRUE;
-		traceEvent(CONST_TRACE_NOISY, "gethost... call returned NULL/NETDB_SUCCESS - "
-			   "this is odd, but apparently normal");
+	if((reportedFreaky == FALSE) && (hp != NULL)) {
+	  reportedFreaky = TRUE;
+	  traceEvent(CONST_TRACE_NOISY, "gethost... call returned NULL/NETDB_SUCCESS - "
+		     "this is odd, but apparently normal");
 	}
 	myGlobals.numDNSErrorHostNotFound++;
 	break;
@@ -519,14 +519,14 @@ static void queueAddress(HostAddr elem, int forceResolution) {
 
   if((!forceResolution)
      && myGlobals.runningPref.trackOnlyLocalHosts
-     && (!_pseudoLocalAddress(&elem)))
+     && (!_pseudoLocalAddress(&elem, NULL, NULL)))
     return;
 
   /*
     The address queue is far too long. This is usefult for
     avoiding problems due to DOS applications
   */
-  if(myGlobals.addressQueuedCurrent > MAX_NUM_QUEUED_ADDRESSES) {
+  if(myGlobals.addressQueuedCurrent > (MAX_NUM_QUEUED_ADDRESSES*myGlobals.numDevices)) {
     static char shownMsg = 0;
 
     if(!shownMsg) {
@@ -554,6 +554,11 @@ static void queueAddress(HostAddr elem, int forceResolution) {
     key_data.dsize = 16;
   }
 #endif
+  else {
+	traceEvent(CONST_TRACE_WARNING, "Invalid address family (%d) found!",
+			elem.hostFamily);
+	return;
+  }
 
   safe_snprintf(__FILE__, __LINE__, dataBuf, sizeof(dataBuf), "%s", addrtostr(&elem));
   data_data.dptr = dataBuf;
@@ -562,9 +567,11 @@ static void queueAddress(HostAddr elem, int forceResolution) {
   rc = gdbm_store(myGlobals.addressQueueFile, key_data, data_data, GDBM_INSERT);
 
   if (rc == 0) {
+    accessMutex(&myGlobals.queueAddressMutex, "dequeueAddress");
     myGlobals.addressQueuedCurrent++, myGlobals.addressQueuedCount++;
     if (myGlobals.addressQueuedCurrent > myGlobals.addressQueuedMax)
       myGlobals.addressQueuedMax = myGlobals.addressQueuedCurrent;
+    releaseMutex(&myGlobals.queueAddressMutex);
 
 #ifdef DNS_DEBUG
     traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Queued address '%s' [addr queue=%d/max=%d]",
@@ -573,11 +580,20 @@ static void queueAddress(HostAddr elem, int forceResolution) {
   } else {
     /* rc = 1 is duplicate key, which is fine.  Other codes are problems... */
     if (rc != 1) {
-      traceEvent(CONST_TRACE_ERROR, "Queue of address '%s' failed, code %d [addr queue=%d/max=%d]",
-		 dataBuf, rc, myGlobals.addressQueuedCurrent, myGlobals.addressQueuedMax);
+      traceEvent(CONST_TRACE_ERROR, "Queue of address '%s' failed (%s) [addr queue=%d/max=%d]",
+		 dataBuf, 
+#if defined(WIN32) && defined(__GNUC__)
+                 "no additional information available",
+#else
+                 gdbm_strerror(gdbm_errno),
+#endif
+                 myGlobals.addressQueuedCurrent,
+                 myGlobals.addressQueuedMax);
       traceEvent(CONST_TRACE_INFO, "ntop processing continues, address will not be resolved");
     } else {
+      accessMutex(&myGlobals.queueAddressMutex, "dequeueAddress");
       myGlobals.addressQueuedDup++;
+      releaseMutex(&myGlobals.queueAddressMutex);
 #ifdef DNS_DEBUG
       traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Duplicate queue of address '%s' ignored",
 		 dataBuf);
@@ -596,15 +612,42 @@ void cleanupAddressQueue(void) {
 
 /* ************************************ */
 
+/*
+  Remove from queue an address that was waiting to be
+  resolved
+*/
+void purgeQueuedV4HostAddress(u_int32_t addr) {
+  datum key_data;
+  struct in_addr addrv4;
+
+  addrv4.s_addr = addr;
+
+  /*
+    traceEvent(CONST_TRACE_INFO, "purgeQueuedV4HostAddress(%s)", 
+                                 _intoa(addrv4, buf, sizeof(buf))); 
+  */
+
+  key_data.dptr = (void*)&addr;
+  key_data.dsize = 4;
+  if(gdbm_delete(myGlobals.addressQueueFile, key_data)) {
+    accessMutex(&myGlobals.queueAddressMutex, "purgeQueuedV4HostAddress");
+    if(myGlobals.addressQueuedCurrent > 0) myGlobals.addressQueuedCurrent--;
+    releaseMutex(&myGlobals.queueAddressMutex);
+  }
+}
+
+/* ************************************ */
+
 void* dequeueAddress(void *_i) {
-  int dqaIndex = (int)_i;
+  int dqaIndex = (int)((long)_i);
 
   HostAddr addr;
   datum key_data, data_data;
 
-  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: DNSAR(%d): Address resolution thread running [p%d]",
-             pthread_self(), dqaIndex+1, getpid());
-
+  traceEvent(CONST_TRACE_INFO, 
+	     "THREADMGMT[t%lu]: DNSAR(%d): Address resolution thread running",
+             pthread_self(), dqaIndex+1);
+  
   while(myGlobals.ntopRunState <= FLAG_NTOPSTATE_RUN) {
 #ifdef DEBUG
     traceEvent(CONST_TRACE_INFO, "DEBUG: Waiting for address to resolve...");
@@ -650,7 +693,10 @@ void* dequeueAddress(void *_i) {
       traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Resolved address %s", addrtostr(&addr));
 #endif
 
-      myGlobals.addressQueuedCurrent--;
+      accessMutex(&myGlobals.queueAddressMutex, "dequeueAddress");
+      if(myGlobals.addressQueuedCurrent > 0) myGlobals.addressQueuedCurrent--;
+      releaseMutex(&myGlobals.queueAddressMutex);
+
       gdbm_delete(myGlobals.addressQueueFile, data_data);
       key_data = data_data;
       data_data = gdbm_nextkey(myGlobals.addressQueueFile, key_data);
@@ -1270,7 +1316,7 @@ static void msdns_filter_name(char *msg) {
 	if(id == 128)
 	  msg[j++] = '\'';
 	else if(id < 128) {
-	  snprintf(tmpStr2, sizeof(tmpStr2), "%c", id);
+	  safe_snprintf(__FILE__, __LINE__, tmpStr2, sizeof(tmpStr2), "%c", id);
 	  msg[j++] = tmpStr2[0];
 	}
 
@@ -1364,6 +1410,8 @@ static void setHostName(HostTraffic *srcHost, char *name) {
 
 /* ************************************ */
 
+//#define MDNS_DEBUG
+
 static void handleMdnsName(HostTraffic *srcHost, u_short sport, u_char *mdns_name) {
   char *mdnsStrtokState, *name = NULL, *appl = NULL, *proto = NULL, *domain = NULL;
   char *tmpStr = strdup((char*)mdns_name);
@@ -1415,6 +1463,8 @@ static void handleMdnsName(HostTraffic *srcHost, u_short sport, u_char *mdns_nam
 	/* Digital Audio Access Protocol (daap.sourceforge.net) */
 	updateHostUsers(name, BITFLAG_DAAP_USER, srcHost);
       }
+    } else if(name && appl && (!strcmp(appl, "local"))) {
+      setHostName(srcHost, name);
     }
 
      free(tmpStr);
@@ -1658,7 +1708,7 @@ u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
     {
       u_int32_t     padding;
 
-      padding=((u_int32_t)bp) % sizeof(u_int32_t);
+      padding=((u_int32_t)((long)bp)) % sizeof(u_int32_t);
       bp += padding;
       buflen -= padding;
     }
@@ -1755,7 +1805,6 @@ u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
   return(transactionId);
 }
 
-
 /* **************************************** */
 
 void checkSpoofing(HostTraffic *hostToCheck, int actualDeviceId) {
@@ -1783,3 +1832,17 @@ void checkSpoofing(HostTraffic *hostToCheck, int actualDeviceId) {
 
 }
 
+/* **************************************** */
+
+char* host2networkName(HostTraffic *el, char *buf, u_short buf_len) {
+  struct in_addr addr = el->hostIpAddress.Ip4Address;
+  char buf1[64];
+      
+  addr.s_addr = addr.s_addr & (0xFFFFFFFF << (32-el->network_mask));
+
+  safe_snprintf(__FILE__, __LINE__, buf, buf_len, "%s/%d",
+		_intoa(addr, buf1, sizeof(buf1)), 
+		el->network_mask);
+  
+  return(buf);
+}
