@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -108,7 +108,7 @@ void initGdbm(char *prefDirectory,  /* Directory with persistent files */
   initSingleGdbm(&myGlobals.addressQueueFile, "addressQueue.db", spoolDirectory, TRUE,  NULL);
   initSingleGdbm(&myGlobals.dnsCacheFile,     "dnsCache.db",     spoolDirectory, -1,    NULL);
   initSingleGdbm(&myGlobals.macPrefixFile,    "macPrefix.db",    spoolDirectory, FALSE,  &statbuf);
-  initSingleGdbm(&myGlobals.fingerprintFile,  "fingerprint.db",    spoolDirectory, FALSE,  &statbuf);
+  initSingleGdbm(&myGlobals.fingerprintFile,  "fingerprint.db",  spoolDirectory, FALSE,  &statbuf);
   createVendorTable(&statbuf);
 }
 
@@ -222,9 +222,7 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
 
   myGlobals.numThreads = 0;            /* # of running threads */
 
-  myGlobals.numDequeueThreads = 1;
-
-  for (i = 0; i < MAX_NUM_DEQUEUE_THREADS; i ++)
+  for (i = 0; i < MAX_NUM_DEQUEUE_ADDRESS_THREADS; i ++)
     myGlobals.dequeueAddressThreadId[i] = (pthread_t)-1;
 
 #ifdef HAVE_OPENSSL
@@ -292,7 +290,8 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   myGlobals.ipPortMapper.theMapper = NULL;
   myGlobals.ipPortMapper.numSlots = 0;
   myGlobals.numHandledSIGPIPEerrors = 0;
-  for (i=0; i<=1; i++) {
+
+  for(i=0; i<=1; i++) {
     myGlobals.numHandledRequests[i] = 0;
     myGlobals.numHandledBadrequests[i] = 0;
     myGlobals.numSuccessfulRequests[i] = 0;
@@ -304,6 +303,7 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
     myGlobals.numUnsuccessfulDenied[i] = 0;
     myGlobals.numUnsuccessfulForbidden[i] = 0;
   }
+
   myGlobals.numSSIRequests = 0;
   myGlobals.numBadSSIRequests = 0;
   myGlobals.numHandledSSIRequests = 0;
@@ -332,8 +332,8 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   /*
    * Create two variables (semaphores) used by functions in pbuf.c to queue packets
    */
-  createCondvar(&myGlobals.queueCondvar);
   createCondvar(&myGlobals.queueAddressCondvar);
+  createMutex(&myGlobals.queueAddressMutex);
 
   createMutex(&myGlobals.gdbmMutex);        /* data to synchronize thread access to db files */
   createMutex(&myGlobals.tcpSessionsMutex); /* data to synchronize TCP sessions access */
@@ -341,27 +341,17 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   createMutex(&myGlobals.purgePortsMutex);  /* data to synchronize port purge access */
   createMutex(&myGlobals.purgeMutex);       /* synchronize purging */
   createMutex(&myGlobals.securityItemsMutex);
-  createMutex(&myGlobals.packetProcessMutex);
-  createMutex(&myGlobals.packetQueueMutex);
   createMutex(&myGlobals.hostsHashLockMutex);
 
   for(i=0; i<CONST_HASH_INITIAL_SIZE; i++) {
     createMutex(&myGlobals.hostsHashMutex[i]);
     myGlobals.hostsHashMutexNumLocks[i] = 0;
   }
-
-  /* Packet Capture */
-  for (i = 0; i <= CONST_PACKET_QUEUE_LENGTH; i ++)
-    memset(&myGlobals.packetQueue[i], 0, sizeof(PacketInformation));
-
-  myGlobals.receivedPackets = 0;
+  
+  myGlobals.receivedPackets          = 0;
   myGlobals.receivedPacketsProcessed = 0;
-  myGlobals.receivedPacketsQueued = 0;
-  myGlobals.packetQueueLen = 0;
-  myGlobals.maxPacketQueueLen = 0;
-  myGlobals.receivedPacketsLostQ = 0;
-  myGlobals.packetQueueHead = 0;
-  myGlobals.packetQueueTail = 0;
+  myGlobals.receivedPacketsQueued    = 0;
+  myGlobals.receivedPacketsLostQ     = 0;
 
   /* NB: Log View is allocated in main.c so it's available for the very 1st traceEvent() */
 
@@ -460,7 +450,6 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
 
   myGlobals.mtuSize        = _mtuSize;
   myGlobals.headerSize     = _headerSize;
-  myGlobals.numDequeueThreads = MAX_NUM_DEQUEUE_THREADS;
 
   /* ********************************** */
 
@@ -534,13 +523,22 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
 void initNtop(char *devices) {
   char value[32];
 
-  initIPServices();
+  revertSlashIfWIN32(myGlobals.dbPath, 0);
+  revertSlashIfWIN32(myGlobals.spoolPath, 0);
 
+  initIPServices();
   handleProtocols();
 
   if(myGlobals.numIpProtosToMonitor == 0)
     addDefaultProtocols();
 
+#ifdef HAVE_LIBPCRE
+  if(myGlobals.runningPref.enableL7) 
+    initl7();
+  else
+    traceEvent(CONST_TRACE_INFO, "No patterns to load: protocol guessing disabled.");
+#endif
+
   /*
    * initialize memory and data.
    */
@@ -554,7 +552,37 @@ void initNtop(char *devices) {
   initGdbm(myGlobals.dbPath, myGlobals.spoolPath, 0);
 
   if(myGlobals.runningPref.daemonMode) {
-    daemonizeUnderUnix();
+    /*
+      Before bacoming a daemon we need o make sure that
+      ntop has been installed properly and that all the 
+      html files are on the right place
+    */
+    
+    int idx, found = 0;
+
+    for(idx=0; (!found) && (myGlobals.dataFileDirs[idx] != NULL); idx++) {
+      char tmpStr[256];
+      struct stat statbuf;
+
+      if(strcmp(myGlobals.dataFileDirs[idx], ".") /* ignore local paths */ ) {
+	safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr),
+		      "%s/html/%s",
+		      myGlobals.dataFileDirs[idx],
+		      "ntop.gif" /* This file must always exist */);
+	
+	if(stat(tmpStr, &statbuf) == 0) {
+	  found = 1;
+	  break;
+	}
+      }
+    }
+
+    
+    if(!found) {
+      traceEvent(CONST_TRACE_WARNING, "ntop will not become a daemon as it has not been");
+      traceEvent(CONST_TRACE_WARNING, "installed properly (did you do 'make install')");
+    } else
+      daemonizeUnderUnix();
   }
 
   /* Handle local addresses (if any) */
@@ -580,6 +608,7 @@ void initNtop(char *devices) {
 
   createPortHash();
   initCounters();
+  initDB();
   initApps();
   initThreads();
 
