@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -49,28 +49,26 @@ extern struct in6_addr _in6addr_linklocal_allnodes;
 #define releaseAddrResMutex()
 #endif
 
-
-
 #ifdef HAVE_LIBWRAP
 extern int allow_severity, deny_severity;
 #endif
 
 /* version.c */
 extern char *version, *osName, *author, *buildDate, *configureDate,
-            *configure_parameters,
-            *host_system_type,
-            *target_system_type,
-            *compiler_cppflags,
-            *compiler_cflags,
-            *include_path,
-            *system_libs,
-            *install_path,
+  *configure_parameters,
+  *host_system_type,
+  *target_system_type,
+  *compiler_cppflags,
+  *compiler_cflags,
+  *include_path,
+  *system_libs,
+  *install_path,
 #ifdef MAKE_WITH_I18N
-            *locale_dir,
+  *locale_dir,
 #endif
-            *distro,
-            *release,
-            *force_runtime;
+  *distro,
+  *release,
+  *force_runtime;
 #ifdef MEMORY_DEBUG
 extern char *memoryDebug;
 #endif
@@ -111,7 +109,7 @@ extern void revertDoubleColumnIfWIN32(char *str);
 extern void checkUserIdentity(int userSpecified);
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
-    Work-arounds.  Instead of cluttering ntop with a bunch of #ifdef logic,
+   Work-arounds.  Instead of cluttering ntop with a bunch of #ifdef logic,
    * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 #ifndef HAVE_PCAP_OPEN_DEAD
 extern pcap_t *pcap_open_dead(int linktype, int snaplen);
@@ -121,19 +119,8 @@ extern pcap_t *pcap_open_dead(int linktype, int snaplen);
 extern char *ntop_strsignal(int sig);
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
-    Dummies.  Instead of cluttering ntop with a bunch of #ifdef logic,
+   Dummies.  Instead of cluttering ntop with a bunch of #ifdef logic,
    * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-#ifndef HAVE_PCAP_FREECODE
-extern void pcap_freecode(struct bpf_program *pgm);
-#endif
-
-#ifndef HAVE_PCAP_FINDALLDEVS
-extern int pcap_findalldevs(pcap_if_t **alldevsp, char *errbuf);
-#endif
-
-#ifndef HAVE_PCAP_FREEALLDEVS
-extern pcap_freealldevs(pcap_if_t *alldevs);
-#endif
 
 #ifndef WIN32
 /* utildl.c */
@@ -151,6 +138,7 @@ extern void initNtop(char *devices);
 extern int printable(int ch);
 extern void cleanupAddressQueue(void);
 extern void* dequeueAddress(void* notUsed);
+extern void purgeQueuedV4HostAddress(u_int32_t addr);
 #ifdef INET6
 extern char* _intop(struct in6_addr *addr,char *buf, u_short buflen);
 extern char* intop(struct in6_addr *addr);
@@ -171,7 +159,8 @@ extern u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
                                  DNSHostInfo *hostPtr, short length,
                                  short *isRequest, short *positiveReply);
 extern void checkSpoofing(HostTraffic *el, int actualDeviceId);
-extern void cleanupHostEntries();
+extern void cleanupHostEntries(void);
+extern char* host2networkName(HostTraffic *el, char *buf, u_short buf_len);
 
 /* admin.c */
 extern void showUsers(void);
@@ -210,17 +199,23 @@ extern HostTraffic* allocFcScsiCounters(HostTraffic *host);
 extern u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
 		      short* useIPAddressForSearching, HostTraffic **el, int actualDeviceId);
 extern u_int hashFcHost(FcAddress *fcAddress, u_short vsanId,
-                         HostTraffic **el, int actualDeviceId);
+			HostTraffic **el, int actualDeviceId);
 extern void freeHostInfo(HostTraffic *host, int actualDeviceId);
 extern void freeHostInstances(int actualDeviceId);
+extern int is_host_ready_to_purge(int actDevice, HostTraffic *el, time_t now);
 extern int purgeIdleHosts(int devId);
 extern void setHostSerial(HostTraffic *el);
-extern HostTraffic * _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlanId,
-				u_char checkForMultihoming, u_char forceUsingIPaddress, int actualDeviceId,
-                                char *file, int line);
+extern HostTraffic * _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, 
+				 u_int16_t vlanId, u_char checkForMultihoming, 
+				 u_char forceUsingIPaddress, int actualDeviceId,
+				 char *file, int line);
 #define lookupHost(a, b, c, d, e, f) _lookupHost(a, b, c, d, e, f, __FILE__, __LINE__)
 extern HostTraffic * lookupFcHost(FcAddress *fcAddress, u_short vsanId,
 				  int actualDeviceId);
+extern void add_valid_ptr(void* ptr);
+extern void remove_valid_ptr(void* ptr);
+extern int is_valid_ptr(void* ptr);
+
 /* initialize.c */
 extern void initIPServices(void);
 extern void resetDevice(int devIdx, short fullReset);
@@ -229,6 +224,7 @@ extern void initCounters(void);
 extern void resetStats(int);
 extern void reinitMutexes(void);
 extern void initThreads(void);
+extern void initDeviceSemaphores(int deviceId);
 extern void initApps(void);
 extern int validInterface(char *name);
 extern void initDevices(char* devices);
@@ -239,8 +235,8 @@ extern void startSniffer(void);
 
 
 extern int safe_snprintf(char* file, int line,
-                          char* buf, size_t sizeofbuf,
-                          char* format, ...);
+			 char* buf, size_t sizeofbuf,
+			 char* format, ...);
 
 #define safe_strncat(a, b, c) _safe_strncat(__FILE__, __LINE__, a, b, c)
 extern int _safe_strncat(char* file, int line,
@@ -251,6 +247,7 @@ extern void initSingleGdbm(GDBM_FILE *database, char *dbName, char *directory,
 			   int doUnlink, struct stat *statbuf);
 extern void initGdbm(char *prefDirectory, char *spoolDirectory, int initPrefsOnly);
 extern void addDevice(char* deviceName, char* deviceDescr);
+extern void allocDeviceMemory(int deviceId);
 
 /* leaks.c */
 
@@ -260,7 +257,8 @@ extern void addDevice(char* deviceName, char* deviceDescr);
 #ifdef MAKE_WITH_SAFER_ROUTINES
 
 /* Fix to the free prototype courtesy of Tanner Lovelace <lovelace@opennms.org> */
-#define free(a)       ntop_safefree((void**)&(a), __FILE__, __LINE__)
+/* Fix free() courtesy of Benoit Dolez <bdolez@exosec.fr> */
+#define free(a)       { void *__t = (a); ntop_safefree((void**)&(__t), __FILE__, __LINE__); a = __t; }
 extern void           ntop_safefree(void **ptr, char* file, int line);
 #define malloc(sz)    ntop_safemalloc(sz, __FILE__, __LINE__)
 extern void*          ntop_safemalloc(unsigned int sz, char* file, int line);
@@ -274,34 +272,34 @@ extern char* ntop_safestrdup(char *ptr, char* file, int line);
 
 #elif defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 1)
 
-  /* mtrace()/muntrace() - use existing routines */
+/* mtrace()/muntrace() - use existing routines */
 
 #elif defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 2)
 
-  /* ElectricFence - use existing routines */
+/* ElectricFence - use existing routines */
 
 #elif defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 3)
 
-#define _memorycheck(a,b,c,d) { \
-  enum mcheck_status _status = mprobe((void *)a); \
-  switch(_status) { \
-    case MCHECK_DISABLED: \
-    case MCHECK_OK: \
-      break; \
-    case MCHECK_HEAD: \
+#define _memorycheck(a,b,c,d) {						\
+    enum mcheck_status _status = mprobe((void *)a);			\
+    switch(_status) {							\
+    case MCHECK_DISABLED:						\
+    case MCHECK_OK:							\
+      break;								\
+    case MCHECK_HEAD:							\
       traceEvent(CONST_TRACE_WARNING, "MCHECK(%s): %p MCHECK_HEAD, modified before block [%s@%d]", b, a, c, d); \
-      break; \
-    case MCHECK_TAIL: \
+      break;								\
+    case MCHECK_TAIL:							\
       traceEvent(CONST_TRACE_WARNING, "MCHECK(%s): %p MCHECK_TAIL, modified after block [%s@%d]", b, a, c, d); \
-      break; \
-    case MCHECK_FREE: \
+      break;								\
+    case MCHECK_FREE:							\
       traceEvent(CONST_TRACE_WARNING, "MCHECK(%s): %p MCHECK_FREE, already freed [%s@%d]", b, a, c, d); \
-      break; \
-  } \
-}
+      break;								\
+    }									\
+  }
 #define memorycheck(a,b) _memorycheck(a, b, __FILE__, __LINE__)
 
-  /* ntop custom monitor */
+/* ntop custom monitor */
 
 extern void initLeaks(void);
 extern void termLeaks(void);
@@ -318,14 +316,14 @@ extern void resetLeaks(void);
    Berthold Gunreben <bg@suse.de>
    
 */
-#define free(a) do {   \
-       union __x {     \
-       void **voidptr; \
-       __typeof__(&(a)) ptr;      \
-       } __x;  \
-       __x.ptr = &(a);  \
-       ntop_free(__x.voidptr, __FILE__, __LINE__); \
-} while (0)
+#define free(a) do {				\
+    union __x {					\
+      void **voidptr;				\
+      __typeof__(&(a)) ptr;			\
+    } __x;					\
+    __x.ptr = &(a);				\
+    ntop_free(__x.voidptr, __FILE__, __LINE__); \
+  } while (0)
 
 extern void*          ntop_malloc(unsigned int sz, char* file, int line);
 extern void*          ntop_calloc(unsigned int c, unsigned int sz, char* file, int line);
@@ -335,7 +333,7 @@ extern void           ntop_free(void **ptr, char* file, int line);
 
 #elif defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 4)
 
-  /* mcheck() - use existing routines */
+/* mcheck() - use existing routines */
 
 #elif defined(MEMORY_DEBUG) 
 #else
@@ -360,13 +358,10 @@ extern void  ntop_gdbm_close(GDBM_FILE g, char* theFile, int theLine);
 /* ntop.c */
 #ifndef WIN32
 extern void handleSigHup(int signalId);
-#endif
-extern void *pcapDispatch(void *_i);
-#ifndef WIN32
-#ifdef HANDLE_DIED_CHILD
 extern RETSIGTYPE handleDiedChild(int);
 #endif
-#endif
+
+extern void *pcapDispatch(void *_i);
 extern void daemonizeUnderUnix(void);
 extern void detachFromTerminalUnderUnix(int);
 extern void createPortHash(void);
@@ -436,6 +431,7 @@ extern void term_ssl(void);
 #endif
 
 /* main.c */
+extern char static_ntop;
 extern void usage(FILE * fp);
 extern void welcome(FILE * fp);
 
@@ -454,14 +450,19 @@ extern void updateDbHostsTraffic(int deviceToUpdate);
 extern int isInitialHttpData(char* packetData);
 extern int isInitialSshData(char* packetData);
 extern int isInitialFtpData(char* packetData);
+extern char* findHostCommunity(u_int32_t host_ip, char *buf, u_short buf_len);
+extern void setHostCommunity(HostTraffic *el);
 extern void updateDeviceThpt(int deviceToUpdate, int quickUpdate);
 
 /* prefs.c */
 extern void loadPrefs(int argc, char* argv[]);
 extern int parseOptions(int argc, char* argv[]);
 extern int fetchPrefsValue(char *key, char *value, int valueLen);
+extern int fetchPwValue(char *key, char *value, int valueLen);
 extern void storePrefsValue(char *key, char *value);
+extern void storePwValue(char *key, char *value);
 extern void delPrefsValue(char *key);
+extern void delPwValue(char *key);
 extern void processStrPref(char *key, char *value, char **globalVar, bool savePref);
 extern void processIntPref(char *key, char *value, int *globalVar, bool savePref);
 extern void processBoolPref(char *key, bool value, bool *globalVar, bool savePref);
@@ -470,7 +471,7 @@ extern void initUserPrefs(UserPref *pref);
 
 /* util.c */
 extern void setEmptySerial(HostSerial *a);
-extern void handleAddressLists(char* addresses, u_int32_t theNetworks[MAX_NUM_NETWORKS][3],
+extern void handleAddressLists(char* addresses, u_int32_t theNetworks[MAX_NUM_NETWORKS][4],
 			       u_short *numNetworks, char *localAddresses,
 			       int localAddressesLen, int flagWhat);
 extern void handleFlowsSpecs(void);
@@ -498,42 +499,91 @@ extern int in6_isglobal(struct in6_addr *addr);
 extern unsigned short prefixlookup(struct in6_addr *addr, NtopIfaceAddr *addrs, int size);
 extern unsigned short addrlookup(struct in6_addr *addr,  NtopIfaceAddr *addrs);
 extern NtopIfaceAddr *getLocalHostAddressv6(NtopIfaceAddr *addrs, char* device);
-extern unsigned short isLinkLocalAddress(struct in6_addr *addr);
-extern unsigned short in6_isMulticastAddress(struct in6_addr *addr);
-extern unsigned short in6_isLocalAddress(struct in6_addr *addr, u_int deviceId);
-extern unsigned short in6_pseudoLocalAddress(struct in6_addr *addr);
-extern unsigned short in6_deviceLocalAddress(struct in6_addr *addr, u_int deviceId);
-extern unsigned short in6_isPseudoLocalAddress(struct in6_addr *addr, u_int deviceId);
-extern unsigned short in6_isPrivateAddress(struct in6_addr *addr);
+extern unsigned short isLinkLocalAddress(struct in6_addr *addr,
+					 u_int32_t *the_local_network, 
+					 u_int32_t *the_local_network_mask);
+extern unsigned short in6_isMulticastAddress(struct in6_addr *addr,
+					     u_int32_t *the_local_network, 
+					     u_int32_t *the_local_network_mask);
+extern unsigned short in6_isLocalAddress(struct in6_addr *addr, u_int deviceId,
+					 u_int32_t *the_local_network, 
+					 u_int32_t *the_local_network_mask);
+extern unsigned short in6_pseudoLocalAddress(struct in6_addr *addr,
+					     u_int32_t *the_local_network, 
+					     u_int32_t *the_local_network_mask);
+extern unsigned short in6_deviceLocalAddress(struct in6_addr *addr, u_int deviceId,
+					     u_int32_t *the_local_network, 
+					     u_int32_t *the_local_network_mask);
+extern unsigned short in6_isPseudoLocalAddress(struct in6_addr *addr, u_int deviceId,
+					       u_int32_t *the_local_network, 
+					       u_int32_t *the_local_network_mask);
+extern unsigned short in6_isPrivateAddress(struct in6_addr *addr,
+					   u_int32_t *the_local_network, 
+					   u_int32_t *the_local_network_mask);
 #endif
 extern unsigned short computeIdx(HostAddr *srcAddr, HostAddr *dstAddr, int sport, int dport);
 extern u_int16_t computeTransId(HostAddr *srcAddr, HostAddr *dstAddr, int sport, int dport);
-extern unsigned short in_isBroadcastAddress(struct in_addr *addr);
-extern unsigned short in_isMulticastAddress(struct in_addr *addr);
-extern unsigned short in_isLocalAddress(struct in_addr *addr, u_int deviceId);
-extern unsigned short in_isPrivateAddress(struct in_addr *addr);
-extern unsigned short in_deviceLocalAddress(struct in_addr *addr, u_int deviceId);
-extern unsigned short in_pseudoLocalAddress(struct in_addr *addr);
-extern unsigned short in_isPseudoLocalAddress(struct in_addr *addr, u_int deviceId);
-extern unsigned short in_isPseudoBroadcastAddress(struct in_addr *addr);
+extern unsigned short in_isBroadcastAddress(struct in_addr *addr,
+					    u_int32_t *the_local_network, 
+					    u_int32_t *the_local_network_mask);
+extern unsigned short in_isMulticastAddress(struct in_addr *addr,
+					    u_int32_t *the_local_network, 
+					    u_int32_t *the_local_network_mask);
+extern u_int8_t num_network_bits(u_int32_t addr);
+extern unsigned short in_isLocalAddress(struct in_addr *addr, u_int deviceId,
+					u_int32_t *the_local_network, 
+					u_int32_t *the_local_network_mask);
+extern unsigned short in_isPrivateAddress(struct in_addr *addr,
+					  u_int32_t *the_local_network, 
+					  u_int32_t *the_local_network_mask);
+extern unsigned short in_deviceLocalAddress(struct in_addr *addr, u_int deviceId,
+					    u_int32_t *the_local_network, 
+					    u_int32_t *the_local_network_mask);
+extern unsigned short in_pseudoLocalAddress(struct in_addr *addr,
+					    u_int32_t *the_local_network, 
+					    u_int32_t *the_local_network_mask);
+extern unsigned short in_isPseudoLocalAddress(struct in_addr *addr, u_int deviceId,
+					      u_int32_t *the_local_network, 
+					      u_int32_t *the_local_network_mask);
+extern unsigned short in_isPseudoBroadcastAddress(struct in_addr *addr,
+						  u_int32_t *the_local_network, 
+						  u_int32_t *the_local_network_mask);
 extern char* copy_argv(register char **argv);
-extern unsigned short isPrivateAddress(HostAddr *addr);
-extern unsigned short isBroadcastAddress(HostAddr *addr);
-extern unsigned short isMulticastAddress(HostAddr *addr);
-extern unsigned short isLocalAddress(HostAddr *addr, u_int actualDeviceId);
+extern unsigned short isPrivateAddress(HostAddr *addr,
+				       u_int32_t *the_local_network, 
+				       u_int32_t *the_local_network_mask);
+extern unsigned short isBroadcastAddress(HostAddr *addr,
+					 u_int32_t *the_local_network, 
+					 u_int32_t *the_local_network_mask);
+extern unsigned short isMulticastAddress(HostAddr *addr,
+					 u_int32_t *the_local_network, 
+					 u_int32_t *the_local_network_mask);
+extern unsigned short isLocalAddress(HostAddr *addr, u_int actualDeviceId,
+				     u_int32_t *the_local_network, 
+				     u_int32_t *the_local_network_mask);
 extern int dotted2bits(char *mask);
 extern void handleLocalAddresses(char* addresses);
-extern unsigned short isPseudoLocalAddress(HostAddr *addr, u_int actualDeviceId);
-extern unsigned short _pseudoLocalAddress(HostAddr *addr);
+extern unsigned short isPseudoLocalAddress(HostAddr *addr, u_int actualDeviceId,
+					   u_int32_t *the_local_network, 
+					   u_int32_t *the_local_network_mask);
+extern unsigned short _pseudoLocalAddress(HostAddr *addr,
+					  u_int32_t *the_local_network, 
+					  u_int32_t *the_local_network_mask);
 extern unsigned short __pseudoLocalAddress(struct in_addr *addr,
-					   u_int32_t theNetworks[MAX_NUM_NETWORKS][3],
-					   u_short numNetworks);
-extern unsigned short deviceLocalAddress(HostAddr *addr, u_int deviceId);
-extern unsigned short isPseudoBroadcastAddress(HostAddr *addr);
+					   u_int32_t theNetworks[MAX_NUM_NETWORKS][4],
+					   u_short numNetworks,
+					   u_int32_t *the_local_network, 
+					   u_int32_t *the_local_network_mask);
+extern unsigned short deviceLocalAddress(HostAddr *addr, u_int deviceId,
+					   u_int32_t *the_local_network, 
+					   u_int32_t *the_local_network_mask);
+extern unsigned short isPseudoBroadcastAddress(HostAddr *addr,
+					       u_int32_t *the_local_network, 
+					       u_int32_t *the_local_network_mask);
 extern void printLogTime(void);
 extern int32_t gmt2local(time_t t);
 extern char *dotToSlash(char *name);
-extern int getLocalHostAddress(struct in_addr *hostIpAddress, char* device);
+extern int getLocalHostAddress(struct in_addr *hostIpAddress, u_int8_t *netmask_v6, char* device);
 extern NtopIfaceAddr * getLocalHostAddressv6(NtopIfaceAddr *addrs, char* device);
 extern void fillDomainName(HostTraffic *el);
 extern int createThread(pthread_t *threadId, void *(*__start_routine) (void *), char* userParm);
@@ -564,14 +614,14 @@ extern int _lockHostsHashMutex(HostTraffic *host, char *where, char *file, int l
 #define unlockHostsHashMutex(a) _unlockHostsHashMutex(a, __FILE__, __LINE__)
 extern int _unlockHostsHashMutex(HostTraffic *host, char* file, int line);
 
-#define setHolder(a) { \
-                       if(fileName != NULL) { strncpy(a.file, fileName, sizeof(a.file)-1); a.file[sizeof(a.file)-1]='\0'; } else \
-                                            { memset(&(a), 0, sizeof(a)); } \
-                       a.line = fileLine; \
-                       a.pid = getpid(); \
-                       a.thread = pthread_self(); \
-                       gettimeofday(&(a.time), NULL) ; \
-                     }
+#define setHolder(a) {							\
+    if(fileName != NULL) { strncpy(a.file, fileName, sizeof(a.file)-1); a.file[sizeof(a.file)-1]='\0'; } else \
+      { memset(&(a), 0, sizeof(a)); }					\
+    a.line = fileLine;							\
+    a.pid = getpid();							\
+    a.thread = pthread_self();						\
+    gettimeofday(&(a.time), NULL) ;					\
+  }
 
 extern int createCondvar(ConditionalVariable *condvarId);
 extern void deleteCondvar(ConditionalVariable *condvarId);
@@ -585,6 +635,7 @@ extern void stringSanityCheck(char* string, char* parm);
 extern void uriSanityCheck(char* string, char* parm, int allowParms);
 extern void pathSanityCheck(char* string, char* parm);
 extern int fileSanityCheck(char* string, char* parm, int nonFatal);
+extern void sanitize_rrd_string(char* name);
 extern int ipSanityCheck(char* string, char* parm, int nonFatal);
 extern void deviceSanityCheck(char* string);
 extern int checkCommand(char* commandName);
@@ -606,8 +657,8 @@ extern time_t getTimeMapping(u_int16_t transactionId,
                              struct timeval theTime);
 extern void traceEvent(int eventTraceLevel, char* file,
                        int line, char * format, ...)
-     __attribute__ ((format (printf, 4, 5)));
-     extern char *_strncpy(char *dest, const char *src, size_t n);
+  __attribute__ ((format (printf, 4, 5)));
+extern char *_strncpy(char *dest, const char *src, size_t n);
 #ifndef HAVE_LOCALTIME_R
 extern struct tm *localtime_r(const time_t *t, struct tm *tp);
 #endif
@@ -620,8 +671,8 @@ extern void addPassiveSessionInfo(HostAddr *theHost, u_short thePort, char *note
 extern int isPassiveSession(HostAddr *theHost, u_short thePort, char **notes);
 extern void addVoIPSessionInfo(HostAddr *theHost, u_short thePort, char *notes);
 extern int isVoIPSession(HostAddr *theHost, u_short thePort, char **notes);
-extern void initPassiveSessions();
-extern void termPassiveSessions();
+extern void initPassiveSessions(void);
+extern void termPassiveSessions(void);
 extern int getPortByName(ServiceEntry **theSvc, char* portName);
 extern char *getPortByNumber(ServiceEntry **theSvc, int port);
 extern char *getPortByNum(int port, int type);
@@ -697,16 +748,16 @@ extern int snprintf(char *str, size_t n, const char *fmt, ...);
   Courtesy of http://ettercap.sourceforge.net/
 */
 #ifndef CFG_LITTLE_ENDIAN
-#define ptohs(x) ( (u_int16_t)                       \
-                      ((u_int16_t)*((u_int8_t *)x+1)<<8|  \
-                      (u_int16_t)*((u_int8_t *)x+0)<<0)   \
-                    )
+#define ptohs(x) ( (u_int16_t)				\
+		   ((u_int16_t)*((u_int8_t *)x+1)<<8|	\
+		    (u_int16_t)*((u_int8_t *)x+0)<<0)   \
+		   )
 
-#define ptohl(x) ( (u_int32)*((u_int8_t *)x+3)<<24|  \
-                      (u_int32)*((u_int8_t *)x+2)<<16|  \
-                      (u_int32)*((u_int8_t *)x+1)<<8|   \
-                      (u_int32)*((u_int8_t *)x+0)<<0    \
-                    )
+#define ptohl(x) ( (u_int32)*((u_int8_t *)x+3)<<24|	\
+		   (u_int32)*((u_int8_t *)x+2)<<16|	\
+		   (u_int32)*((u_int8_t *)x+1)<<8|	\
+		   (u_int32)*((u_int8_t *)x+0)<<0	\
+		   )
 #else
 #define ptohs(x) *(u_int16_t *)(x)
 #define ptohl(x) *(u_int32 *)(x)
@@ -716,8 +767,8 @@ extern int snprintf(char *str, size_t n, const char *fmt, ...);
 
 #ifdef HAVE_GETOPT_H
 extern int getopt_long(int ___argc, char *const *___argv,
-                        const char *__shortopts,
-                        const struct option *__longopts, int *__longind);
+		       const char *__shortopts,
+		       const struct option *__longopts, int *__longind);
 extern int getopt_long_only();
 #endif /* HAVE_GETOPT_H */
 
@@ -727,17 +778,17 @@ extern char **buildargv(const char *argv);
 #ifndef HAVE_FREEARGV
 extern void freeargv(char **argv);
 #endif
-extern void handleWhiteBlackListAddresses(char* addresses, u_int32_t theNetworks[MAX_NUM_NETWORKS][3],
+extern void handleWhiteBlackListAddresses(char* addresses, u_int32_t theNetworks[MAX_NUM_NETWORKS][4],
 					  u_short *numNets, char* outAddresses,
 					  int outAddressesLen);
 extern unsigned short isOKtoSave(u_int32_t addr,
-				 u_int32_t whiteNetworks[MAX_NUM_NETWORKS][3],
-				 u_int32_t blackNetworks[MAX_NUM_NETWORKS][3],
+				 u_int32_t whiteNetworks[MAX_NUM_NETWORKS][4],
+				 u_int32_t blackNetworks[MAX_NUM_NETWORKS][4],
 				 u_short numWhiteNets, u_short numBlackNets);
 extern float timeval_subtract(struct timeval x, struct timeval y);
 extern void freePortsUsage(HostTraffic *el);
 extern PortUsage* getPortsUsage(HostTraffic *el, u_int portIdx, int createIfNecessary);
-extern char* vlan2name(int vlanId, char *buf, int buf_len);
+extern char* vlan2name(u_int16_t vlanId, char *buf, int buf_len);
 extern void mkdir_p(char *tag, char *path, int permission);
 
 /* Formatting for %.2f ... */
@@ -750,12 +801,18 @@ extern char* getSAPInfo(u_int16_t sapInfo, short encodeString);
 extern char* getSpecialMacInfo(HostTraffic* el, short encodeString);
 extern void createVendorTable(struct stat *statbuf);
 
+/* l7.c */
+extern void initl7(void);
+extern void l7SessionProtoDetection(IPSession *theSession, 
+				    u_int packetDataLength, 
+				    u_char* packetData);
+
 /* sessions.c */
 #define checkSessionIdx(a) _checkSessionIdx(a, actualDeviceId, __FILE__, __LINE__)
 extern u_int _checkSessionIdx(u_int idx, int actualDeviceId, char* file, int line);
 extern void freeSession(IPSession *sessionToPurge, int actualDeviceId, u_char allocateMemoryIfNeeded, u_char lockMutex);
 extern void freeFcSession(FCSession *sessionToPurge, int actualDeviceId,
-                           u_char allocateMemoryIfNeeded, u_char lockMutex);
+			  u_char allocateMemoryIfNeeded, u_char lockMutex);
 extern void scanTimedoutTCPSessions(int actualDeviceId);
 extern void updateUsedPorts(HostTraffic *srcHost, HostTraffic *dstHost,
 			    u_short sport, u_short dport, u_int length);
@@ -766,17 +823,18 @@ extern IPSession* handleSession(const struct pcap_pkthdr *h,
                                 HostTraffic *dstHost, u_short dport,
                                 u_int length, struct tcphdr *tp,
                                 u_int tcpDataLength, u_char* packetData,
-                                int actualDeviceId, u_short *newSession);
+                                int actualDeviceId, u_short *newSession,
+				u_char real_session /* vs. faked/netflow-session */);
 extern void updateHostUsers(char *userName, int userType, HostTraffic *theHost);
 extern void handlePluginSessionTermination(IPSession *sessionToPurge, int actualDeviceId);
 
 extern FCSession* handleFcSession(const struct pcap_pkthdr *h,
-                                   u_short fragmentedData,
-                                   HostTraffic *srcHost, HostTraffic *dstHost,
-                                   u_int length, u_int payload_len, u_short oxid,
-                                   u_short rxid, u_short protocol, u_char rCtl,
-                                   u_char isXchgOrig, const u_char *bp,
-                                   int actualDeviceId);
+				  u_short fragmentedData,
+				  HostTraffic *srcHost, HostTraffic *dstHost,
+				  u_int length, u_int payload_len, u_short oxid,
+				  u_short rxid, u_short protocol, u_char rCtl,
+				  u_char isXchgOrig, const u_char *bp,
+				  int actualDeviceId);
 
 /* fcUtils.c */
 extern int isFlogiAcc(FcAddress *fcAddress, u_int8_t r_ctl, u_int8_t type, u_int8_t cmd);
@@ -785,18 +843,30 @@ extern int isPlogi(u_int8_t r_ctl, u_int8_t type, u_int8_t cmd);
 extern int isLogout(u_int8_t r_ctl, u_int8_t type, u_int8_t cmd);
 extern int isRscn(u_int8_t r_ctl, u_int8_t type, u_int8_t cmd);
 extern int fillFcpInfo(const u_char *bp, HostTraffic *srcHost,
-                        HostTraffic *dstHost);
+		       HostTraffic *dstHost);
 extern FcFabricElementHash *getFcFabricElementHash(u_short vsanId,
-                                             int actualDeviceId);
+						   int actualDeviceId);
 extern int isValidFcNxPort(FcAddress *fcAddress);
 extern int updateFcFabricElementHash(FcFabricElementHash **theHash, u_short vsanId,
-                                      const u_char *bp, FcAddress *srcAddr,
-                                      FcAddress *dstAddr,
-                                      u_short protocol, u_char r_ctl,
-                                      u_int32_t pktlen);
+				     const u_char *bp, FcAddress *srcAddr,
+				     FcAddress *dstAddr,
+				     u_short protocol, u_char r_ctl,
+				     u_int32_t pktlen);
 extern void processFcNSCacheFile(char *filename);
 
-
+extern u_int32_t num_db_insert, num_db_insert_failed;
+extern int is_db_enabled(void);
+extern void initDB(void);
+extern void termDB(void);
+extern int dump_session_to_db(IPSession *sess);
+extern int insert_flow_record(u_int16_t probeId,
+			      u_int32_t srcAddr, u_int32_t dstAddr,
+			      u_int16_t input, u_int16_t output,
+			      u_int32_t sentPkts, u_int32_t sentOctets,
+			      u_int32_t rcvdPkts, u_int32_t rcvdOctets,
+			      u_int32_t first, u_int32_t last,
+			      u_int16_t srcPort, u_int16_t dstPort, u_int8_t tcpFlags,
+			      u_int8_t proto, u_int8_t tos, u_int16_t vlanId);
 #ifdef HAVE_NETDB_H
 extern int h_errno; /* netdb.h */
 #endif
@@ -824,7 +894,7 @@ extern int h_errno; /* netdb.h */
 
 #ifdef WIN32
 #ifndef __MINGW32__
-#define strncasecmp(a, b, c) strnicmp(a, b, c)
+#define strncasecmp(a, b, c) _strnicmp(a, b, c)
 #endif
 #define sleep(a /* sec */) waitForNextEvent(1000*a /* ms */)
 #else
@@ -834,25 +904,25 @@ extern int h_errno; /* netdb.h */
 #define NOW ((time_t) time ((time_t *) 0))
 
 #if defined(LBL_ALIGN)
-#define EXTRACT_16BITS(p) \
-	((u_short)*((u_char *)(p) + 0) << 8 | \
-	(u_short)*((u_char *)(p) + 1))
-#define EXTRACT_32BITS(p) \
-	((u_int32_t)*((u_char *)(p) + 0) << 24 | \
-	(u_int32_t)*((u_char *)(p) + 1) << 16 | \
-	(u_int32_t)*((u_char *)(p) + 2) << 8 | \
-	(u_int32_t)*((u_char *)(p) + 3))
+#define EXTRACT_16BITS(p)			\
+  ((u_short)*((u_char *)(p) + 0) << 8 |		\
+   (u_short)*((u_char *)(p) + 1))
+#define EXTRACT_32BITS(p)			\
+  ((u_int32_t)*((u_char *)(p) + 0) << 24 |	\
+   (u_int32_t)*((u_char *)(p) + 1) << 16 |	\
+   (u_int32_t)*((u_char *)(p) + 2) << 8 |	\
+   (u_int32_t)*((u_char *)(p) + 3))
 #else
-#define EXTRACT_16BITS(p) \
-	((u_short)ntohs(*(u_short *)(p)))
-#define EXTRACT_32BITS(p) \
-	((u_int32_t)ntohl(*(u_int32_t *)(p)))
+#define EXTRACT_16BITS(p)			\
+  ((u_short)ntohs(*(u_short *)(p)))
+#define EXTRACT_32BITS(p)			\
+  ((u_int32_t)ntohl(*(u_int32_t *)(p)))
 #endif
 
-#define EXTRACT_24BITS(p) \
-	((u_int32_t)*((u_char *)(p) + 0) << 16 | \
-	(u_int32_t)*((u_char *)(p) + 1) << 8 | \
-	(u_int32_t)*((u_char *)(p) + 2))
+#define EXTRACT_24BITS(p)			\
+  ((u_int32_t)*((u_char *)(p) + 0) << 16 |	\
+   (u_int32_t)*((u_char *)(p) + 1) << 8 |	\
+   (u_int32_t)*((u_char *)(p) + 2))
 
 #define incrementUsageCounter(a, b, c) _incrementUsageCounter(a, b, c, __FILE__, __LINE__)
 
@@ -872,18 +942,18 @@ extern int h_errno; /* netdb.h */
 
 /* *************************************
 
-   Code "inherited" from nslookup
+Code "inherited" from nslookup
 
-   ************************************* */
+************************************* */
 
 #ifndef NS_GET16
-#define NS_GET16(s, cp) { \
-        u_char *t_cp = (u_char *)(cp); \
-        (s) = ((u_int16_t)t_cp[0] << 8) \
-            | ((u_int16_t)t_cp[1]) \
-            ; \
-        (cp) += NS_INT16SZ; \
-}
+#define NS_GET16(s, cp) {			\
+    u_char *t_cp = (u_char *)(cp);		\
+    (s) = ((u_int16_t)t_cp[0] << 8)		\
+      | ((u_int16_t)t_cp[1])			\
+      ;						\
+    (cp) += NS_INT16SZ;				\
+  }
 #endif
 
 /* Bit test macros */
@@ -936,38 +1006,38 @@ extern int h_errno; /* netdb.h */
 /* #define getSerial(a) myGlobals.device[deviceToUpdate].hash_hostTraffic[a]->hostSerial */
 
 #ifdef SSLWATCHDOG_DEBUG
-#define sslwatchdogDebug(text, bpcFlag, note) { \
-          traceEvent(CONST_TRACE_INFO, "SSLWDDEBUG: %1d %-10s %-15s %-15s %s", \
-                                 myGlobals.sslwatchdogCondvar.predicate, \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""), \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""), \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""), \
-                                 note); \
-}
-#define sslwatchdogDebugN(text, bpcFlag, note) { \
-          traceEvent(CONST_TRACE_INFO, "SSLWDDEBUG: %1d %-10s %-15s %-15s %d", \
-                                 myGlobals.sslwatchdogCondvar.predicate, \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""), \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""), \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""), \
-                                 note); \
-}
-#define sslwatchdogError(text, bpcFlag, note) { \
-          traceEvent(CONST_TRACE_INFO, "SSLWDERROR: %1d %-10s %-15s %-15s %s", \
-                                 myGlobals.sslwatchdogCondvar.predicate, \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""), \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""), \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""), \
-                                 note); \
-}
-#define sslwatchdogErrorN(text, bpcFlag, note) { \
-          traceEvent(CONST_TRACE_INFO, "SSLWDERROR: %1d %-10s %-15s %-15s %d", \
-                                 myGlobals.sslwatchdogCondvar.predicate, \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""), \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""), \
-                                 ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""), \
-                                 note); \
-}
+#define sslwatchdogDebug(text, bpcFlag, note) {				\
+    traceEvent(CONST_TRACE_INFO, "SSLWDDEBUG: %1d %-10s %-15s %-15s %s", \
+	       myGlobals.sslwatchdogCondvar.predicate,			\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""),	\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""),	\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""),	\
+	       note);							\
+  }
+#define sslwatchdogDebugN(text, bpcFlag, note) {			\
+    traceEvent(CONST_TRACE_INFO, "SSLWDDEBUG: %1d %-10s %-15s %-15s %d", \
+	       myGlobals.sslwatchdogCondvar.predicate,			\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""),	\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""),	\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""),	\
+	       note);							\
+  }
+#define sslwatchdogError(text, bpcFlag, note) {				\
+    traceEvent(CONST_TRACE_INFO, "SSLWDERROR: %1d %-10s %-15s %-15s %s", \
+	       myGlobals.sslwatchdogCondvar.predicate,			\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""),	\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""),	\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""),	\
+	       note);							\
+  }
+#define sslwatchdogErrorN(text, bpcFlag, note) {			\
+    traceEvent(CONST_TRACE_INFO, "SSLWDERROR: %1d %-10s %-15s %-15s %d", \
+	       myGlobals.sslwatchdogCondvar.predicate,			\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""),	\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""),	\
+	       ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""),	\
+	       note);							\
+  }
 #else
 #define sslwatchdogDebug(text, bpcFlag, note) {}
 #define sslwatchdogDebugN(text, bpcFlag, note) {}
@@ -986,23 +1056,23 @@ extern int h_errno; /* netdb.h */
 #endif
 
 #ifndef IN6_IS_ADDR_LINKLOCAL
-#define IN6_IS_ADDR_LINKLOCAL(a) \
-        ((((uint32_t *) (a))[0] & htonl (0xffc00000))                 \
-         == htonl (0xfe800000))
+#define IN6_IS_ADDR_LINKLOCAL(a)		\
+  ((((uint32_t *) (a))[0] & htonl (0xffc00000))	\
+   == htonl (0xfe800000))
 #endif
 
 /* **********************************************************
    Fixup for gdbm (which doesn't require zero termed strings
    ********************************************************** */
-#define zeroPadMallocString(sz, ptr) { \
-    if(ptr[sz-1] != '\0') { \
-      char *_zeropadmallocstringtemp = ptr; \
-      ptr = malloc(sz + 1); \
-      strncpy(ptr, _zeropadmallocstringtemp, sz); \
-      ptr[sz] = '\0'; \
-      free(_zeropadmallocstringtemp); \
-    } \
-}
+#define zeroPadMallocString(sz, ptr) {			\
+    if(ptr[sz-1] != '\0') {				\
+      char *_zeropadmallocstringtemp = ptr;		\
+      ptr = malloc(sz + 1);				\
+      strncpy(ptr, _zeropadmallocstringtemp, sz);	\
+      ptr[sz] = '\0';					\
+      free(_zeropadmallocstringtemp);			\
+    }							\
+  }
 
 /* **********************************************************
    Used in all the prints flowing from printNtopConfigInfo...
