@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  *		 	    http://www.ntop.org/
  *
@@ -158,6 +158,7 @@ extern void printPluginTrailer(char *left, char *middle);
 /* http.c */
 extern char* encodeString(char* in, char* out, u_int out_len);
 extern void printHTMLtrailer(void);
+extern void returnHTTPpageBadCommunity(void);
 extern void returnHTTPredirect(char* destination);
 void returnHTTPpageNotFound(char* additionalText);
 
@@ -178,16 +179,18 @@ extern void printHTMLheader(char *title, char *htmlTitle, int headerFlags);
 #ifdef HAVE_OPENSSL
 extern char* printSSLError(int errorId);
 #endif /* HAVE_OPENSSL */
+extern int isAllowedCommunity(char *community_name);
 extern void sendHTTPHeader(int mimeType, int headerFlags, int useCompressionIfAvailable);
 extern int readHTTPpostData(int len, char *buf, int buflen);
 
 /* report.c */
 extern void printBandwidthFooter(void);
 extern void initReports(void);
+extern void checkReportDevice(void);
 extern int reportValues(time_t *lastTime);
 extern void addPageIndicator(char *url, u_int beginIdx,
 			     u_int numEntries, u_int linesPerPage,
-			     int revertOrder, int numCol);
+			     int revertOrder, int numCol, int netmode);
 extern void printTrafficSummary(int revertOrder);
 extern void printTrafficStatistics(int revertOrder);
 extern int combineReportTypeLocality(int reportTypeReq, LocalityDisplayPolicy showLocalityMode);
@@ -201,7 +204,8 @@ extern void printFcHostsTraffic(int reportType,
 extern void printMulticastStats(int sortedColumn /* ignored so far */,
                                 int revertOrder, int pageNum);
 extern void printVSANList(unsigned int deviceId);
-extern void printHostsInfo(int sortedColumn, int revertOrder, int pageNum, int showBytes, int vlanId);
+extern void printHostsInfo(int sortedColumn, int revertOrder, int pageNum, 
+			   int showBytes, int vlanId, int ifId);
 extern void makeDot(void);
 extern void printFcHostsInfo(int sortedColumn, int revertOrder, int pageNum, int showBytes, int vsanId);
 extern void printAllSessionsHTML (char* host, int actualDeviceId, int sortedColumn,
@@ -247,19 +251,20 @@ extern void printIpTrafficMatrix(void);
 extern void printFcTrafficMatrix (u_short vsanId, u_char sent);
 extern void printThptStatsMatrix(int sortedColumn);
 extern void printThptStats(int sortedColumn);
-extern void printDomainStats(char* domainName, int clusterMode, 
+extern void printDomainStats(char* domain_domain_name, int network_mode, int clusterMode, 
 			     int sortedColumn, int revertOrder, int pageNum);
 extern void printNoDataYet(void);
 extern void printNotAvailable(char* flagName);
 extern void listNetFlows(void);
 extern void printHostHourlyTraffic(HostTraffic *el);
-extern void printASList(unsigned int deviceId);
 extern void printVLANList(unsigned int deviceId);
 extern void showPortTraffic(u_short portNr);
 extern void drawVsanStatsGraph (unsigned int deviceId);
 extern void printVSANList(unsigned int deviceId);
 extern void handleNtopConfig (char *url, UserPrefDisplayPage page, int postLen);
 extern void purgeHost(char *serialized_serial);
+extern void printInterfaceStats(void);
+extern void findHost(char *key);
 
 /* webInterface.c */
 extern int execCGI(char* cgiName);
@@ -327,12 +332,10 @@ extern char *makeHostLink(HostTraffic *el, short mode,
 extern char *makeFcHostLink (HostTraffic *el, short mode, short cutName,
                              short compactWWN, char *buf, int buflen);
 extern char *makeVsanLink (u_short vsanId, short mode, char *buf, int buflen);
-
+extern void  buildMapLink(HostTraffic *el, char *buf, int buf_len);
 extern char *getHostName(HostTraffic *el, short cutName, char *buf, int bufLen);
 
 /* graph.c */
-extern char* gdVersionGuess(void);
-extern void sendGraphFile(char* fileName, int doNotUnlink);
 extern void hostTrafficDistrib(HostTraffic *theHost, short dataSent);
 extern void hostFcTrafficDistrib(HostTraffic *theHost, short dataSent);
 extern void hostIPTrafficDistrib(HostTraffic *theHost, short dataSent);
@@ -352,18 +355,10 @@ extern void drawGlobalProtoDistribution(void);
 extern int  drawHostsDistanceGraph(int);
 extern void drawGlobalIpProtoDistribution(void);
 extern void drawGlobalFcProtoDistribution(void);
-extern void drawLunStatsBytesDistribution (HostTraffic *el);
-extern void drawLunStatsPktsDistribution (HostTraffic *el);
-extern void drawVsanStatsBytesDistribution (int deviceId);
-extern void drawVsanStatsPktsDistribution (int deviceId);
-extern void drawVsanDomainTrafficDistribution (u_short vsanId, u_char dataSent);
-extern void drawVsanSwilsProtoDistribution (u_short vsanId);
-extern void drawBar(short width, short height, FILE* filepointer,
-		    int   num_points, char  *labels[], float data[]);
-extern void drawArea(short width, short height, FILE* filepointer,
-		     int   num_points, char  *labels[], float data[],
-		     char *xtitle, char *ytitle, u_short formatYlabels);
-extern void drawPie(short width, short height, FILE* filepointer,
-		    int   num_points, char  *labels[], float data[],
-		    int sorted);
+extern void drawLunStatsBytesDistribution(HostTraffic *el);
+extern void drawLunStatsPktsDistribution(HostTraffic *el);
+extern void drawVsanStatsBytesDistribution(int deviceId);
+extern void drawVsanStatsPktsDistribution(int deviceId);
+extern void drawVsanDomainTrafficDistribution(u_short vsanId, u_char dataSent);
+extern void drawVsanSwilsProtoDistribution(u_short vsanId);
 
