@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -66,7 +66,7 @@ typedef u_int   uint32_t;
 
 #if !defined(HAVE_U_INT64_T)
 #if defined(WIN32)
-typedef _int64 u_int64_t;
+/* typedef _int64 u_int64_t; */
 #else
 #if defined(HAVE_UINT64_T)
 #define u_int64_t uint64_t
@@ -214,12 +214,12 @@ typedef struct hostAddr {
 
 typedef struct _ethSerial {
   u_char  ethAddress[LEN_ETHERNET_ADDRESS];
-  u_short vlanId;
+  u_int16_t vlanId;
 } EthSerial;
 
 typedef struct _ipSerial {
   HostAddr ipAddress;
-  u_short  vlanId;
+  u_int16_t  vlanId;
 } IpSerial;
 
 typedef struct hostSerial {
@@ -775,6 +775,7 @@ typedef struct fcScsiCounters {
 
 /* Host Traffic */
 typedef struct hostTraffic {
+  u_int8_t         to_be_deleted; /* 1 = the host will be deleted in the next purge loop */
   u_short          magic;
   u_short          l2Family;    /* 0 = Ethernet, 1 = Fibre Channel (FC) */
   u_int            hostTrafficBucket; /* Index in the **hash_hostTraffic list */
@@ -782,14 +783,16 @@ typedef struct hostTraffic {
   u_short          refCount;         /* Reference counter */
   HostSerial       hostSerial;
   HostAddr         hostIpAddress;
-  short            vlanId;           /* VLAN Id (-1 if not set) */
+  u_int16_t        vlanId;           /* VLAN Id (-1 if not set) */
+  u_int16_t        ifId;            /* Interface Id [e.g. for NetFlow] (-1 if not set) */
   u_int16_t        hostAS;           /* AS to which the host belongs to */
   time_t           firstSeen, lastSeen; /* time when this host has sent/rcvd some data  */
   u_char           ethAddress[LEN_ETHERNET_ADDRESS];
   u_char           lastEthAddress[LEN_ETHERNET_ADDRESS]; /* used for remote addresses */
   char             ethAddressString[LEN_ETHERNET_ADDRESS_DISPLAY];
   char             hostNumIpAddress[20] /* xxx.xxx.xxx.xxx */, *dnsDomainValue, *dnsTLDValue;
-  char             *hwModel, *description;
+  u_char           network_mask; /* IPv6 notation e.g. /24 */
+  char             *hwModel, *description, *community;
   char             *ip2ccValue, *fingerprint;
   char             hostResolvedName[MAX_LEN_SYM_HOST_NAME];
   short            hostResolvedNameType;
@@ -906,10 +909,10 @@ typedef struct ipSession {
   u_short magic;
   u_char isP2P;                     /* Set to 1 if this is a P2P session          */
   HostTraffic* initiator;           /* initiator address                          */
-  HostAddr initiatorRealIp;   /* Real IP address (if masqueraded and known) */
+  HostAddr initiatorRealIp;         /* Real IP address (if masqueraded and known) */
   u_short sport;                    /* initiator address (port)                   */
   HostTraffic *remotePeer;          /* remote peer address                        */
-  HostAddr remotePeerRealIp;  /* Real IP address (if masqueraded and known) */
+  HostAddr remotePeerRealIp;        /* Real IP address (if masqueraded and known) */
   char *virtualPeerName;            /* Name of a virtual host (e.g. HTTP virtual host) */
   u_short dport;                    /* remote peer address (port)                       */
   time_t firstSeen;                 /* time when the session has been initiated         */
@@ -940,7 +943,8 @@ typedef struct ipSession {
   u_char sessionState;              /* actual session state                     */
   u_char  passiveFtpSession;        /* checked if this is a passive FTP session */
   u_char  voipSession;              /* checked if this is a VoIP session */
-  char *session_info;               /* Info about this sessio (if any) */
+  char *session_info;               /* Info about this session (if any) */
+  char *guessed_protocol;
   struct ipSession *next;
 } IPSession;
 
@@ -1066,12 +1070,12 @@ struct flow_ver1_rec {
   u_int16_t output;     /* Output interface index */
   u_int32_t dPkts;      /* Packets sent in Duration */
   u_int32_t dOctets;    /* Octets sent in Duration */
-  u_int32_t First;      /* SysUptime at start of flow */
-  u_int32_t Last;       /* and of last packet of the flow */
+  u_int32_t first;      /* SysUptime at start of flow */
+  u_int32_t last;       /* and of last packet of the flow */
   u_int16_t srcport;    /* TCP/UDP source port number (.e.g, FTP, Telnet, etc.,or equivalent) */
   u_int16_t dstport;    /* TCP/UDP destination port number (.e.g, FTP, Telnet, etc.,or equivalent) */
   u_int16_t pad;        /* pad to word boundary */
-  u_int8_t  prot;       /* IP protocol, e.g., 6=TCP, 17=UDP, etc... */
+  u_int8_t  proto;      /* IP protocol, e.g., 6=TCP, 17=UDP, etc... */
   u_int8_t  tos;        /* IP Type-of-Service */
   u_int8_t  pad2[7];    /* pad to word boundary */
 };
@@ -1104,18 +1108,18 @@ struct flow_ver5_rec {
 			   & last packet in this flow)*/
   u_int32_t dOctets;    /* Octets sent in Duration (milliseconds between 1st
 			   & last packet in  this flow)*/
-  u_int32_t First;      /* SysUptime at start of flow */
-  u_int32_t Last;       /* and of last packet of the flow */
+  u_int32_t first;      /* SysUptime at start of flow */
+  u_int32_t last;       /* and of last packet of the flow */
   u_int16_t srcport;    /* TCP/UDP source port number (.e.g, FTP, Telnet, etc.,or equivalent) */
   u_int16_t dstport;    /* TCP/UDP destination port number (.e.g, FTP, Telnet, etc.,or equivalent) */
   u_int8_t  pad1;       /* pad to word boundary */
   u_int8_t  tcp_flags;  /* Cumulative OR of tcp flags */
-  u_int8_t  prot;       /* IP protocol, e.g., 6=TCP, 17=UDP, etc... */
+  u_int8_t  proto;      /* IP protocol, e.g., 6=TCP, 17=UDP, etc... */
   u_int8_t  tos;        /* IP Type-of-Service */
   u_int16_t src_as;     /* source peer/origin Autonomous System */
   u_int16_t dst_as;     /* dst peer/origin Autonomous System */
   u_int8_t  src_mask;   /* source route's mask bits */
-  u_int8_t  dst_mask;   /* destination route's mask bits */  
+  u_int8_t  dst_mask;   /* destination route's mask bits */
   u_int16_t pad2;       /* pad to word boundary */
 };
 
@@ -1144,13 +1148,13 @@ struct flow_ver7_rec {
   u_int16_t output;     /* Output interface index */
   u_int32_t dPkts;      /* Packets sent in Duration */
   u_int32_t dOctets;    /* Octets sent in Duration */
-  u_int32_t First;      /* SysUptime at start of flow */
-  u_int32_t Last;       /* and of last packet of the flow */
+  u_int32_t first;      /* SysUptime at start of flow */
+  u_int32_t last;       /* and of last packet of the flow */
   u_int16_t srcport;    /* TCP/UDP source port number (.e.g, FTP, Telnet, etc.,or equivalent) */
   u_int16_t dstport;    /* TCP/UDP destination port number (.e.g, FTP, Telnet, etc.,or equivalent) */
   u_int8_t  flags;      /* Shortcut mode(dest only,src only,full flows*/
   u_int8_t  tcp_flags;  /* Cumulative OR of tcp flags */
-  u_int8_t  prot;       /* IP protocol, e.g., 6=TCP, 17=UDP, etc... */
+  u_int8_t  proto;      /* IP protocol, e.g., 6=TCP, 17=UDP, etc... */
   u_int8_t  tos;        /* IP Type-of-Service */
   u_int16_t dst_as;     /* dst peer/origin Autonomous System */
   u_int16_t src_as;     /* source peer/origin Autonomous System */
@@ -1211,6 +1215,27 @@ typedef struct flowSetV9 {
   struct flowSetV9 *next;
 } FlowSetV9;
 
+typedef struct interfaceStats {
+  u_int32_t netflow_device_ip;
+  u_short interface_id;
+  char interface_name[32];
+  TrafficCounter inBytes, outBytes, inPkts, outPkts;
+  TrafficCounter selfBytes, selfPkts;
+  struct interfaceStats *next;
+} InterfaceStats;
+
+/* AS statistics */
+typedef struct astats {
+  u_short as_id;
+  time_t lastUpdate;
+  Counter totPktsSinceLastRRDDump;
+  TrafficCounter inBytes, outBytes, inPkts, outPkts;
+  TrafficCounter selfBytes, selfPkts;
+  struct astats *next;
+} AsStats;
+
+#define MAX_INTERFACE_STATS_QUEUE_LEN  32
+
 typedef struct netFlowGlobals {
   u_char netFlowDebug;
 
@@ -1226,7 +1251,7 @@ typedef struct netFlowGlobals {
 #ifdef HAVE_SCTP
   int netFlowInSctpSocket;
 #endif
-  u_char netFlowAssumeFTP;
+  u_char netFlowAssumeFTP, saveFlowsIntoDB, enableSessionHandling;
   u_short netFlowInPort;
   struct in_addr netFlowIfAddress, netFlowIfMask;
   char *netFlowWhiteList, *netFlowBlackList;
@@ -1244,7 +1269,8 @@ typedef struct netFlowGlobals {
 
   /* Stats */
   ProbeInfo probeList[MAX_NUM_PROBES];
-  u_int32_t whiteNetworks[MAX_NUM_NETWORKS][3], blackNetworks[MAX_NUM_NETWORKS][3];
+  InterfaceStats *ifStats;
+  u_int32_t whiteNetworks[MAX_NUM_NETWORKS][4], blackNetworks[MAX_NUM_NETWORKS][4];
   u_short numWhiteNets, numBlackNets;
   u_int32_t flowProcessed;
   Counter flowProcessedBytes;
@@ -1253,7 +1279,16 @@ typedef struct netFlowGlobals {
 
   pthread_t netFlowThread;
   int threadActive;
-  PthreadMutex whiteblackListMutex;
+  PthreadMutex whiteblackListMutex, ifStatsMutex;
+
+#ifdef HAVE_SNMP
+  pthread_t netFlowUtilsThread;
+  InterfaceStats *ifStatsQueue[MAX_INTERFACE_STATS_QUEUE_LEN];
+  u_short ifStatsQueue_len;
+  PthreadMutex ifStatsQueueMutex;
+  ConditionalVariable ifStatsQueueCondvar;
+#endif
+
 } NetFlowGlobals;
 
 /* *********************************** */
@@ -1284,6 +1319,7 @@ typedef struct ifCounters {
   u_int32_t ifOutDiscards;
   u_int32_t ifOutErrors;
   u_int32_t ifPromiscuousMode;
+  struct ifCounters *next;
 } IfCounters;
 
 
@@ -1299,7 +1335,7 @@ typedef struct sFlowGlobals {
   char *sflowWhiteList, *sflowBlackList;
   u_long numsFlowsPktsRcvd;
   u_long numsFlowsV2Rcvd, numsFlowsV4Rcvd, numsFlowsV5Rcvd, numsFlowsProcessed;
-  u_long numsFlowsRcvd, lastNumsFlowsRcvd;
+  u_long numsFlowsSamples, numsFlowCounterUpdates;
   u_long numBadsFlowsVersionsRcvd, numBadFlowReality;
   u_long numSrcsFlowsEntryFailedBlackList, numSrcsFlowsEntryFailedWhiteList,
     numSrcsFlowsEntryAccepted,
@@ -1308,7 +1344,7 @@ typedef struct sFlowGlobals {
 
   /* Stats */
   ProbeInfo probeList[MAX_NUM_PROBES];
-  u_int32_t whiteNetworks[MAX_NUM_NETWORKS][3], blackNetworks[MAX_NUM_NETWORKS][3];
+  u_int32_t whiteNetworks[MAX_NUM_NETWORKS][4], blackNetworks[MAX_NUM_NETWORKS][4];
   u_short numWhiteNets, numBlackNets;
   u_int32_t flowProcessed;
   Counter flowProcessedBytes;
@@ -1320,13 +1356,14 @@ typedef struct sFlowGlobals {
 
   u_long numSamplesReceived, initialPool, lastSample;
   u_int32_t flowSampleSeqNo, numSamplesToGo;
-  IfCounters *ifCounters[MAX_NUM_SFLOW_INTERFACES];
+  IfCounters *ifCounters;
 } SflowGlobals;
 
 /* *********************************** */
 
 typedef struct ntopInterface {
-  char *name;                    /* unique interface name */
+  char *name;                    /* Interface name (e.g. eth0) */
+  char *uniqueIfName;            /* Unique interface name used to save data on disk */
   char *humanFriendlyName;       /* Human friendly name of the interface (needed under WinNT and above) */
   int flags;                     /* the status of the interface as viewed by ntop */
 
@@ -1367,6 +1404,19 @@ typedef struct ntopInterface {
 
   int fd;                        /* unique identifier (Unix file descriptor) */
 
+  PthreadMutex asMutex;
+  AsStats *asStats;
+
+  /*
+   * NPA - Network Packet Analyzer (main thread)
+   */
+  PthreadMutex packetQueueMutex;
+  PthreadMutex packetProcessMutex;
+  PacketInformation *packetQueue; /* [CONST_PACKET_QUEUE_LENGTH+1]; */
+  u_int packetQueueLen, maxPacketQueueLen, packetQueueHead, packetQueueTail;
+  ConditionalVariable queueCondvar;
+  pthread_t dequeuePacketThreadId;
+
   /*
    * The packets section
    */
@@ -1422,7 +1472,7 @@ typedef struct ntopInterface {
   TrafficCounter fcBroadcastBytes;
   TrafficCounter class2Bytes, class3Bytes, classFBytes;
 
-  PortCounter    *ipPorts[MAX_IP_PORT];
+  PortCounter    **ipPorts; /* [MAX_IP_PORT] */
 
   TrafficCounter lastMinEthernetBytes;
   TrafficCounter lastFiveMinsEthernetBytes;
@@ -1894,7 +1944,7 @@ typedef enum {
   showPrefFCPref,
   showPrefAdvPref,
   showPrefDbgPref,
-  showPrefPluginPref,
+  showPrefDBPref
 } UserPrefDisplayPage;
 
 /* *********************************** */
@@ -1904,9 +1954,9 @@ typedef enum {
 #endif
 
 #ifdef WIN32
-#define _mkdir(a, b) mkdir(a)
+#define ntop_mkdir(a, b) _mkdir(a)
 #else
-#define _mkdir(a, b) mkdir(a, b)
+#define ntop_mkdir(a, b) mkdir(a, b)
 #endif
 
 
@@ -1938,10 +1988,15 @@ typedef struct _userPref {
   int  refreshRate;             /* -r | --refresh-time */
   bool disablePromiscuousMode;  /* -s | --no-promiscuous */
   int  traceLevel;              /* -t | --trace-level */
-
+  char *mapperURL;              /* -U | --disable-mapper */
   u_int     maxNumHashEntries;  /* -x */
   u_int     maxNumSessions;     /* -X */
 
+  /* SQL Database */
+  char    sqlDbConfig[64];
+  u_int   sqlRecDaysLifetime;
+  bool    saveRecordsIntoDb, saveSessionsIntoDb;
+
   u_int16_t defaultVsan;        /* -v | --default-vsan */
   char      *webAddr;           /* -w | --http-serveraddress[:port] */
   int       webPort;
@@ -1959,9 +2014,9 @@ typedef struct _userPref {
 #endif
 
   bool mergeInterfaces;         /* -M | --no-interface-merge */
+  bool enableL7;                /* Enable/disable l7 protocol pattern matching */
   char *pcapLogBasePath;        /* -O | --pcap-file-path */ /* Added by Ola Lundqvist <opal@debian.org>. */
   char *fcNSCacheFile;          /* -N | --wwn-map */
-  char *mapperURL;              /* -U | --mapper */
 
 #ifdef HAVE_OPENSSL
   char *sslAddr;                /* -W | --https-serveraddress[:port] */
@@ -1981,10 +2036,6 @@ typedef struct _userPref {
   char *P3Pcp;                  /* --p3p-cp '137' */
   char *P3Puri;                 /* --p3p-uri '138' */
 
-#if !defined(WIN32) && defined(HAVE_PCAP_SETNONBLOCK)
-  bool setNonBlocking;          /* --set-pcap-nonblocking '139' */
-#endif
-
   char *instance;               /* --instance '140' */
   char *logo;
 
@@ -1994,7 +2045,6 @@ typedef struct _userPref {
   bool printFcOnly;             /* --fc-only '147' */
   bool printIpOnly;             /* --no-fc '148' */
   bool noInvalidLunDisplay;     /* --no-invalid-lun '149' */
-
   bool disableMutexExtraInfo;   /* --disable-mutexextrainfo '145' */
 
   bool skipVersionCheck;        /* --skip-version-check '150' */
@@ -2027,8 +2077,10 @@ typedef struct ntopGlobals {
   UserPref savedPref;         /* this is what is saved */
   UserPref runningPref;       /* this is what is currently used */
 #ifndef WIN32
-  char      *effectiveUserName;
-  int       userId, groupId;    /* 'u' */
+  char *effectiveUserName;
+  int  userId, groupId;         /* 'u' */
+#else
+  u_char useU3;                 /* --U3 */
 #endif
   char *dbPath;                 /* 'P' */
   char *spoolPath;              /* 'Q' */
@@ -2042,8 +2094,7 @@ typedef struct ntopGlobals {
   char *securityItems[MAX_NUM_PWFILE_ENTRIES];
 
   /* Results flags - something we've learned */
-  bool haveASN,
-       haveVLANs;
+  bool haveASN, haveVLANs;
 
   /* Physical and Logical network interfaces */
 
@@ -2068,17 +2119,15 @@ typedef struct ntopGlobals {
   /* Multi-thread related */
   unsigned short numThreads;       /* # of running threads */
 
-  ConditionalVariable queueCondvar;
   ConditionalVariable queueAddressCondvar;
+  PthreadMutex queueAddressMutex;
 
   pthread_t mainThreadId;
 
   /*
-   * NPA - Network Packet Analyzer (main thread)
+   * Purge database
    */
-  PthreadMutex packetQueueMutex;
-  PthreadMutex packetProcessMutex;
-  pthread_t dequeueThreadId;
+  pthread_t purgeDbThreadId;
 
   /*
    * HTS - Hash Purge
@@ -2103,12 +2152,13 @@ typedef struct ntopGlobals {
   pthread_t scanFingerprintsThreadId;
   time_t nextFingerprintScan;
 
+
   /*
    * DNSAR - DNS Address Resolution - optional
    */
-  unsigned short numDequeueThreads;
   PthreadMutex addressResolutionMutex;
-  pthread_t dequeueAddressThreadId[MAX_NUM_DEQUEUE_THREADS];
+  u_int numDequeueAddressThreads;
+  pthread_t dequeueAddressThreadId[MAX_NUM_DEQUEUE_ADDRESS_THREADS];
 
   /*
    * Control mutexes
@@ -2141,7 +2191,7 @@ typedef struct ntopGlobals {
 #endif /* HAVE_OPENSSL */
 
   /* ntop state - see flags in globals-defines.h */
-  short ntopRunState;      
+  short ntopRunState;
 
   u_char resetHashNow;       /* used for hash reset */
 
@@ -2204,7 +2254,6 @@ typedef struct ntopGlobals {
   volatile unsigned long numHandledSIGPIPEerrors;
   u_short checkVersionStatus;
   time_t checkVersionStatusAgain;
-  char *gdVersionGuessValue;
 
   /* Purge */
   Counter numPurgedHosts, numTerminatedSessions;
@@ -2231,8 +2280,6 @@ typedef struct ntopGlobals {
   PortProtoMapperHandler ipPortMapper;
 
   /* Packet Capture */
-  PacketInformation packetQueue[CONST_PACKET_QUEUE_LENGTH+1];
-  u_int packetQueueLen, maxPacketQueueLen, packetQueueHead, packetQueueTail;
   Counter receivedPackets, receivedPacketsProcessed, receivedPacketsQueued, receivedPacketsLostQ;
 
   TransactionTime transTimeHash[CONST_NUM_TRANSACTION_ENTRIES];
@@ -2241,16 +2288,13 @@ typedef struct ntopGlobals {
   u_short *mtuSize, *headerSize;
 
   /* (Pseudo) Local Networks */
-  u_int32_t localNetworks[MAX_NUM_NETWORKS][3]; /* [0]=network, [1]=mask, [2]=broadcast */
+  u_int32_t localNetworks[MAX_NUM_NETWORKS][4]; /* [0]=network, [1]=mask, [2]=broadcast, [3]=mask_v6 */
   u_short numLocalNetworks;
 
 #if defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 3)
   size_t allocatedMemory;
 #endif
 
-#ifdef MAKE_WITH_MALLINFO
-  u_int baseMemoryUsage;
-#endif
   u_int ipTrafficMatrixMemoryUsage;
   u_int fcTrafficMatrixMemoryUsage;
   u_char webInterfaceDisabled;
