@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  * 			    http://www.ntop.org/
  *
@@ -51,8 +51,8 @@ u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
 
   if(((*useIPAddressForSearching) == 1) || ((ether_addr == NULL) && (hostIpAddress != NULL))) {
     if(myGlobals.runningPref.trackOnlyLocalHosts
-       && (!isLocalAddress(hostIpAddress, actualDeviceId))
-       && (!_pseudoLocalAddress(hostIpAddress))) {
+       && (!isLocalAddress(hostIpAddress, actualDeviceId, NULL, NULL))
+       && (!_pseudoLocalAddress(hostIpAddress, NULL, NULL))) {
       *el = myGlobals.otherHostEntry;
       return(OTHER_HOSTS_ENTRY);
     } else {
@@ -73,16 +73,16 @@ u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
   } else if(hostIpAddress == NULL) {
     memcpy(&idx, &ether_addr[LEN_ETHERNET_ADDRESS-sizeof(u_int)], sizeof(u_int));
     (*useIPAddressForSearching) = 0;
-  } else if(isBroadcastAddress(hostIpAddress)) {
+  } else if(isBroadcastAddress(hostIpAddress, NULL, NULL)) {
     *el = myGlobals.broadcastEntry;
     return(BROADCAST_HOSTS_ENTRY);
-  } else if(isPseudoLocalAddress(hostIpAddress, actualDeviceId)) {
+  } else if(isPseudoLocalAddress(hostIpAddress, actualDeviceId, NULL, NULL)) {
     memcpy(&idx, &ether_addr[LEN_ETHERNET_ADDRESS-sizeof(u_int)], sizeof(u_int));
     (*useIPAddressForSearching) = 0;
   } else {
     if(hostIpAddress != NULL) {
       if(myGlobals.runningPref.trackOnlyLocalHosts
-	 && (!isPseudoLocalAddress(hostIpAddress, actualDeviceId))) {
+	 && (!isPseudoLocalAddress(hostIpAddress, actualDeviceId, NULL, NULL))) {
 	*el = myGlobals.otherHostEntry;
 	return(OTHER_HOSTS_ENTRY);
       } else {
@@ -113,8 +113,8 @@ u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
 
 /* ************************************ */
 
-u_int hashFcHost (FcAddress *fcaddr, u_short vsanId, HostTraffic **el,
-                  int actualDeviceId)
+u_int hashFcHost(FcAddress *fcaddr, u_short vsanId, HostTraffic **el,
+		 int actualDeviceId)
 {
   u_int idx = 0;
 
@@ -277,7 +277,7 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
     return;
   }
 
-  if(host->magic != CONST_MAGIC_NUMBER) {
+  if((host->magic != CONST_MAGIC_NUMBER) && (host->magic != CONST_UNMAGIC_NUMBER)) {
     traceEvent(CONST_TRACE_ERROR, "Bad magic number (expected=%d/real=%d) freeHostInfo()",
 	       CONST_MAGIC_NUMBER, host->magic);
     return;
@@ -313,7 +313,7 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
     else
       key_data.dsize = 0;
 
-    if(key_data.dsize) {
+    if(key_data.dsize && (key_data.dptr != NULL)) {
       gdbm_delete(myGlobals.addressQueueFile, key_data);
 
 #ifdef DNS_DEBUG
@@ -473,6 +473,7 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
   host->dnsTLDValue = NULL;
   if(host->description != NULL) free(host->description);
   if(host->hwModel != NULL) free(host->hwModel);
+  if(host->community != NULL) free(host->community);
   if(host->ip2ccValue != NULL) free(host->ip2ccValue);
   host->ip2ccValue = NULL;
 
@@ -483,26 +484,8 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
     #endif
   */
 
-  /*
-    Do not free the host pointer but add it to
-    a list of 'ready to use' pointers.
-
-    Memory Recycle
-  */
-
-#if RECYCLE_MEMORY
-  if(myGlobals.hostsCacheLen < (MAX_HOSTS_CACHE_LEN-1)) {
-    host->magic = 0;
-    myGlobals.hostsCache[myGlobals.hostsCacheLen++] = host;
-    if(myGlobals.hostsCacheLen > myGlobals.hostsCacheLenMax)
-      myGlobals.hostsCacheLenMax = myGlobals.hostsCacheLen;
-  } else
-#endif
-    {
-      /* No room left: it's time to free the bucket */
-      memset(host, 0, sizeof(HostTraffic)); /* Debug code */
-      free(host);
-    }
+  memset(host, 0, sizeof(HostTraffic)); /* Debug code */
+  free(host);
 
   myGlobals.numPurgedHosts++;
 
@@ -541,6 +524,8 @@ void freeHostInstances(int actualDeviceId) {
     for(idx=FIRST_HOSTS_ENTRY; idx<myGlobals.device[actualDeviceId].actualHashSize; idx++) {
       HostTraffic *el = myGlobals.device[actualDeviceId].hash_hostTraffic[idx];
 
+    if(myGlobals.ntopRunState >= FLAG_NTOPSTATE_SHUTDOWN) break;
+
       while(el != NULL) {
 	HostTraffic *nextEl = el->next;
 	el->next = NULL;
@@ -563,9 +548,35 @@ void freeHostInstances(int actualDeviceId) {
 
 /* ************************************ */
 
+int is_host_ready_to_purge(int actDevice, HostTraffic *el, time_t now) {
+  /* Time used to decide whether a host need to be purged */
+  time_t noSessionPurgeTime   = now-PARM_HOST_PURGE_MINIMUM_IDLE_NOACTVSES;
+  time_t withSessionPurgeTime = now-PARM_HOST_PURGE_MINIMUM_IDLE_ACTVSES;
+  
+  if(el->to_be_deleted
+     || (
+	 (el->refCount == 0)
+	 && ((!myGlobals.runningPref.rFileName) 
+	     &&  (((el->numHostSessions == 0) && (el->lastSeen < noSessionPurgeTime))
+		  || ((el->numHostSessions > 0)  && (el->lastSeen < withSessionPurgeTime))))
+	 && (!broadcastHost(el)) && (el != myGlobals.otherHostEntry)
+	 && (myGlobals.device[actDevice].virtualDevice /* e.g. sFlow/NetFlow */
+	     || (!myGlobals.runningPref.stickyHosts)
+	     || ((el->l2Family == FLAG_HOST_TRAFFIC_AF_ETH) &&
+		 ((el->hostNumIpAddress[0] == '\0') /* Purge MAC addresses too */
+		  || (!subnetPseudoLocalHost(el)))) /* Purge remote hosts only */
+	     || ((el->l2Family == FLAG_HOST_TRAFFIC_AF_FC) &&
+		 (el->fcCounters->hostNumFcAddress[0] == '\0')))))
+    return(1);
+  else
+    return(0);
+}
+
+/* ************************************ */
+
 int purgeIdleHosts(int actDevice) {
   u_int idx, numFreedBuckets=0, numHosts = 0;
-  time_t startTime = time(NULL), noSessionPurgeTime, withSessionPurgeTime;
+  time_t now = time(NULL);
   static time_t lastPurgeTime[MAX_NUM_DEVICES];
   static char firstRun = 1;
   HostTraffic **theFlaggedHosts = NULL;
@@ -573,7 +584,7 @@ int purgeIdleHosts(int actDevice) {
   float hiresDeltaTime;
   struct timeval hiresTimeStart, hiresTimeEnd;
   HostTraffic *el, *prev, *next;
-
+  
   /* if(myGlobals.runningPref.rFileName != NULL) return; */
 
 #ifdef IDLE_PURGE_DEBUG
@@ -587,24 +598,15 @@ int purgeIdleHosts(int actDevice) {
 
   gettimeofday(&hiresTimeStart, NULL);
 
-  if(startTime < (lastPurgeTime[actDevice]+PARM_HOST_PURGE_INTERVAL))
+  if(now < (lastPurgeTime[actDevice]+PARM_HOST_PURGE_INTERVAL))
     return(0); /* Too short */
   else
-    lastPurgeTime[actDevice] = startTime;
+    lastPurgeTime[actDevice] = now;
 
   maxHosts = myGlobals.device[actDevice].hostsno; /* save it as it can change */
   myGlobals.piMem = maxHosts*sizeof(HostTraffic*);
   theFlaggedHosts = (HostTraffic**)calloc(1, myGlobals.piMem);
 
-  /* Time used to decide whether a host need to be purged */
-  noSessionPurgeTime   = startTime-PARM_HOST_PURGE_MINIMUM_IDLE_NOACTVSES;
-  withSessionPurgeTime = startTime-PARM_HOST_PURGE_MINIMUM_IDLE_ACTVSES;
-
-#ifdef IDLE_PURGE_DEBUG
-  traceEvent(CONST_TRACE_INFO, "IDLE_PURGE_DEBUG: Beginning noS < %d, wS < %d",
-	     noSessionPurgeTime, withSessionPurgeTime);
-#endif
-
   purgeOldFragmentEntries(actDevice); /* let's do this too */
 
 #ifdef IDLE_PURGE_DEBUG
@@ -622,23 +624,25 @@ int purgeIdleHosts(int actDevice) {
   accessMutex(&myGlobals.hostsHashLockMutex, "scanIdleLoop");
 
   for(idx=0; idx<myGlobals.device[actDevice].actualHashSize; idx++) {
+    if(myGlobals.ntopRunState >= FLAG_NTOPSTATE_SHUTDOWN) break;
+
     if((el = myGlobals.device[actDevice].hash_hostTraffic[idx]) != NULL) {
-      prev = NULL;
+      prev = NULL;     
 
       while(el) {
-	if((el->refCount == 0)
-	   && (   ((el->numHostSessions == 0) && (el->lastSeen < noSessionPurgeTime))
-	       || ((el->numHostSessions > 0)  && (el->lastSeen < withSessionPurgeTime)))
-	   && (!broadcastHost(el)) && (el != myGlobals.otherHostEntry)
-	   && (myGlobals.device[actDevice].virtualDevice /* e.g. sFlow/NetFlow */
-	       || (!myGlobals.runningPref.stickyHosts)
-	       || ((el->l2Family == FLAG_HOST_TRAFFIC_AF_ETH) &&
-                   ((el->hostNumIpAddress[0] == '\0') /* Purge MAC addresses too */
-                    || (!subnetPseudoLocalHost(el)))) /* Purge remote hosts only */
-               || ((el->l2Family == FLAG_HOST_TRAFFIC_AF_FC) &&
-                   (el->fcCounters->hostNumFcAddress[0] == '\0')))) {
+	if(is_host_ready_to_purge(actDevice, el, now)) {
+	  if(!el->to_be_deleted) {
+	    el->to_be_deleted = 1; /* Delete it at the next run */
+
+	    /* Skip it and move to next host */
+	    prev = el;
+	    el = el->next;	    
+	  } else {
 	  /* Host selected for deletion */
 	  theFlaggedHosts[numHosts++] = el;
+	  el->magic = CONST_UNMAGIC_NUMBER;
+	  purgeQueuedV4HostAddress(el->hostIpAddress.Ip4Address.s_addr);
+	  remove_valid_ptr(el);
 	  next = el->next;
 
 	  if(prev != NULL)
@@ -648,6 +652,7 @@ int purgeIdleHosts(int actDevice) {
 
           el->next = NULL;
 	  el = next;
+	  }
 	} else {
 	  /* Move to next host */
 	  prev = el;
@@ -681,7 +686,8 @@ int purgeIdleHosts(int actDevice) {
   traceEvent(CONST_TRACE_INFO, "IDLE_PURGE_DEBUG: releaseMutex(purgeMutex)...released");
 #endif
 
-  traceEvent(CONST_TRACE_NOISY, "IDLE_PURGE: Device %d [%s] FINISHED selection, %d [out of %d] hosts selected",
+  traceEvent(CONST_TRACE_NOISY, "IDLE_PURGE: Device %d [%s] FINISHED selection, "
+	     "%d [out of %d] hosts selected",
 	     actDevice, myGlobals.device[actDevice].name, numHosts, scannedHosts);
 
   /* Now free the entries */
@@ -757,21 +763,22 @@ void setHostSerial(HostTraffic *el) {
   Searches a host and returns it. If the host is not
   present in the hash a new bucket is created
 */
-HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlanId,
-			u_char checkForMultihoming, u_char forceUsingIPaddress,
-			int actualDeviceId, char *file, int line) {
+HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t vlanId,
+			 u_char checkForMultihoming, u_char forceUsingIPaddress,
+			 int actualDeviceId, char *file, int line) {
   u_int idx, isMultihomed = 0;
   HostTraffic *el=NULL;
   char buf[MAX_LEN_SYM_HOST_NAME_HTML];
   short useIPAddressForSearching = forceUsingIPaddress;
   char* symEthName = NULL, *ethAddr;
-  u_char setSpoofingFlag = 0;
+  u_char setSpoofingFlag = 0, locked_mutex = 0;
   u_short numRuns=0;
   u_int hostFound = 0;
   u_int updateIPinfo = 0;
+  u_int32_t the_local_network, the_local_network_mask;
 
   if((hostIpAddress == NULL) && (ether_addr == NULL)) {
-    traceEvent(CONST_TRACE_WARNING, "Both Ethernet and IP addresses are NULL in lookupHost()[%s/%d]", file, line);
+    traceEvent(CONST_TRACE_WARNING, "Both Ethernet and IP addresses are NULL in lookupHost()[%s:%d]", file, line);
     return(NULL);
   }
 
@@ -799,13 +806,20 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
     return(el); /* Found */
   else if(idx == FLAG_NO_PEER)
     return(NULL);
-  else
+  else {
     el = myGlobals.device[actualDeviceId].hash_hostTraffic[idx];
+    if(el) {
+      lockHostsHashMutex(el, "_lookupHost");
+      accessMutex(&myGlobals.hostsHashLockMutex, "lookupHost");
+      el = myGlobals.device[actualDeviceId].hash_hostTraffic[idx];
+      locked_mutex = 1;
+    }
+  }
 
   while (el != NULL) {
     if(el->magic != CONST_MAGIC_NUMBER) {
       traceEvent(CONST_TRACE_ERROR,
-                 "Bad magic number (expected=%d/real=%d) [deviceId=%d] lookupHost()[%s/%d]",
+                 "Bad magic number (expected=%d/real=%d) [deviceId=%d] lookupHost()[%s:%d]",
 		 CONST_MAGIC_NUMBER, el->magic, actualDeviceId,
                  file, line);
       break; /* Can't trust this chain ... */
@@ -813,77 +827,80 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
 
     if(el->hostTrafficBucket != idx) {
       traceEvent(CONST_TRACE_WARNING,
-                 "Error: wrong bucketIdx %s/%s (expected=%d/real=%d) [deviceId=%d] lookupHost()[%s/%d]",
+                 "Error: wrong bucketIdx %s/%s (expected=%d/real=%d) [deviceId=%d] lookupHost()[%s:%d]",
 		 el->ethAddressString, el->hostNumIpAddress,
 		 idx, el->hostTrafficBucket, actualDeviceId,
                  file, line);
     }
 
-    if(useIPAddressForSearching == 0) {
-      /* compare with the ethernet-address then the IP address */
-      if(memcmp(el->ethAddress, ether_addr, LEN_ETHERNET_ADDRESS) == 0) {
-	if((hostIpAddress != NULL) &&
-	   (hostIpAddress->hostFamily == el->hostIpAddress.hostFamily)) {
-	  if((!isMultihomed) && checkForMultihoming) {
-	    /* This is a local address hence this is a potential multihomed host. */
+    if(!is_host_ready_to_purge(actualDeviceId, el, myGlobals.actTime)) {
+      if(useIPAddressForSearching == 0) {
+	/* compare with the ethernet-address then the IP address */
+	if(memcmp(el->ethAddress, ether_addr, LEN_ETHERNET_ADDRESS) == 0) {
+	  if((hostIpAddress != NULL) &&
+	     (hostIpAddress->hostFamily == el->hostIpAddress.hostFamily)) {
+	    if((!isMultihomed) && checkForMultihoming) {
+	      /* This is a local address hence this is a potential multihomed host. */
 
-	    if(!(addrnull(&el->hostIpAddress)) &&
-		(addrcmp(&el->hostIpAddress,hostIpAddress) != 0)) {
-	      isMultihomed = 1;
-	      FD_SET(FLAG_HOST_TYPE_MULTIHOMED, &el->flags);
-	    } else {
-	      updateIPinfo = 1;
+	      if(!(addrnull(&el->hostIpAddress)) &&
+		 (addrcmp(&el->hostIpAddress,hostIpAddress) != 0)) {
+		isMultihomed = 1;
+		FD_SET(FLAG_HOST_TYPE_MULTIHOMED, &el->flags);
+	      } else {
+		updateIPinfo = 1;
+	      }
+	    }
+	    hostFound = 1;
+	    break;
+	  } else if(hostIpAddress == NULL){  /* Only Mac Addresses */
+	    hostFound = 1;
+	    break;
+	  } else { /* MAC match found and we have the IP - need to update... */
+	    updateIPinfo = 1;
+	    hostFound = 1;
+	    break;
+	  }
+	} else if((hostIpAddress != NULL) &&
+		  (addrcmp(&el->hostIpAddress, hostIpAddress) == 0)) {
+	  /* Spoofing or duplicated MAC address:
+	     two hosts with the same IP address and different MAC
+	     addresses
+	  */
+
+	  if(!hasDuplicatedMac(el)) {
+	    FD_SET(FLAG_HOST_DUPLICATED_MAC, &el->flags);
+
+	    if(myGlobals.runningPref.enableSuspiciousPacketDump) {
+	      char etherbuf[LEN_ETHERNET_ADDRESS_DISPLAY];
+
+	      traceEvent(CONST_TRACE_WARNING,
+			 "Two MAC addresses found for the same IP address "
+			 "%s: [%s/%s] (spoofing detected?)",
+			 el->hostNumIpAddress,
+			 etheraddr_string(ether_addr, etherbuf), el->ethAddressString);
+	      dumpSuspiciousPacket(actualDeviceId);
 	    }
 	  }
-	  hostFound = 1;
-	  break;
-	} else if(hostIpAddress == NULL){  /* Only Mac Addresses */
-	  hostFound = 1;
-	  break;
-	} else { /* MAC match found and we have the IP - need to update... */
-	  updateIPinfo = 1;
+
+	  setSpoofingFlag = 1;
 	  hostFound = 1;
 	  break;
 	}
-      } else if((hostIpAddress != NULL) &&
-		(addrcmp(&el->hostIpAddress, hostIpAddress) == 0)) {
-	/* Spoofing or duplicated MAC address:
-	   two hosts with the same IP address and different MAC
-	   addresses
-	*/
-
-	if(!hasDuplicatedMac(el)) {
-	  FD_SET(FLAG_HOST_DUPLICATED_MAC, &el->flags);
-
-	  if(myGlobals.runningPref.enableSuspiciousPacketDump) {
-	    char etherbuf[LEN_ETHERNET_ADDRESS_DISPLAY];
-
-	    traceEvent(CONST_TRACE_WARNING,
-		       "Two MAC addresses found for the same IP address "
-		       "%s: [%s/%s] (spoofing detected?)",
-		       el->hostNumIpAddress,
-		       etheraddr_string(ether_addr, etherbuf), el->ethAddressString);
-	    dumpSuspiciousPacket(actualDeviceId);
-	  }
+      } else {
+	/* -o | --no-mac (or NetFlow, which doesn't have MACs) - compare with only the IP address */
+	if(addrcmp(&el->hostIpAddress, hostIpAddress) == 0) {
+	  hostFound = 1;
+	  break;
 	}
-
-	setSpoofingFlag = 1;
-	hostFound = 1;
-	break;
-      }
-    } else {
-      /* -o | --no-mac (or NetFlow, which doesn't have MACs) - compare with only the IP address */
-      if(addrcmp(&el->hostIpAddress, hostIpAddress) == 0) {
-        hostFound = 1;
-        break;
       }
     }
-
     el = el->next;
     numRuns++;
   } /* while */
 
-  if((hostFound == 1) && (vlanId > 0) && (vlanId != el->vlanId) && (!isMultivlaned(el))) {
+  if(locked_mutex) releaseMutex(&myGlobals.hostsHashLockMutex);
+
+  if((hostFound == 1) && (vlanId != NO_VLAN) && (vlanId != el->vlanId) && (!isMultivlaned(el))) {
     FD_SET(FLAG_HOST_TYPE_MULTIVLANED, &el->flags);
     if(myGlobals.multipleVLANedHostCount == 0) {
       traceEvent(CONST_TRACE_ERROR, "mVLAN: Host (identical IP/MAC) found on multiple VLANs");
@@ -921,7 +938,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
       if(myGlobals.runningPref.numericFlag == 0)
         ipaddr2str(el->hostIpAddress, 1);
 
-      if(isBroadcastAddress(&el->hostIpAddress))
+      if(isBroadcastAddress(&el->hostIpAddress, NULL, NULL))
         FD_SET(FLAG_BROADCAST_HOST, &el->flags);
     }
   } else {
@@ -937,23 +954,15 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
 		   myGlobals.runningPref.maxNumHashEntries);
       }
 
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
       return(NULL);
     }
-
-#if RECYCLE_MEMORY
-    if(myGlobals.hostsCacheLen > 0) {
-      el = myGlobals.hostsCache[--myGlobals.hostsCacheLen];
-      myGlobals.hostsCacheReused++;
-      /*
-	traceEvent(CONST_TRACE_INFO, "Fetched host from pointers cache (len=%d)",
-	(int)myGlobals.hostsCacheLen);
-      */
-    } else
-#endif
-      {   
-        if((el = (HostTraffic*)malloc(sizeof(HostTraffic))) == NULL)
-          return(NULL);
-      }
+        
+    if((el = (HostTraffic*)malloc(sizeof(HostTraffic))) == NULL) {
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      return(NULL);
+    }
+     
     memset(el, 0, sizeof(HostTraffic));
     el->firstSeen = myGlobals.actTime;
 
@@ -967,11 +976,17 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
       freePortsUsage(el);
 
     len = (size_t)myGlobals.numIpProtosList*sizeof(ShortProtoTrafficInfo**);
-    if((el->ipProtosList = (ShortProtoTrafficInfo**)malloc(len)) == NULL) return(NULL);
+    if((el->ipProtosList = (ShortProtoTrafficInfo**)malloc(len)) == NULL) {
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      return(NULL);
+    }
     memset(el->ipProtosList, 0, len);
 
     len = (size_t)myGlobals.numIpProtosToMonitor*sizeof(ProtoTrafficInfo**);
-    if((el->protoIPTrafficInfos = (ProtoTrafficInfo**)malloc(len)) == NULL) return(NULL);
+    if((el->protoIPTrafficInfos = (ProtoTrafficInfo**)malloc(len)) == NULL) {
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      return(NULL);
+    }
     memset(el->protoIPTrafficInfos, 0, len);
 
     el->magic = CONST_MAGIC_NUMBER;
@@ -985,11 +1000,17 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
     myGlobals.device[actualDeviceId].hash_hostTraffic[el->hostTrafficBucket] = el;  /* Insert a new entry */
     myGlobals.device[actualDeviceId].hostsno++;
 
+    if(0) 
+      traceEvent(CONST_TRACE_INFO, "-> Allocated(%d) [tot=%d]", 
+		 actualDeviceId, myGlobals.device[actualDeviceId].hostsno);
+
+    the_local_network = 0, the_local_network_mask = 0;
+
     if(ether_addr != NULL) {
       if((hostIpAddress == NULL)
 	 || ((hostIpAddress != NULL)
-	     && isPseudoLocalAddress(hostIpAddress, actualDeviceId)
-	     /* && (!isBroadcastAddress(hostIpAddress))*/
+	     && isPseudoLocalAddress(hostIpAddress, actualDeviceId, &the_local_network, &the_local_network_mask)
+	     /* && (!isBroadcastAddress(hostIpAddress, &the_local_network, &the_local_network_mask))*/
 	     )) {
 	char etherbuf[LEN_ETHERNET_ADDRESS_DISPLAY];
 
@@ -1002,6 +1023,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
 	symEthName = getSpecialMacInfo(el, (short)(!myGlobals.separator[0]));
 	FD_SET(FLAG_SUBNET_LOCALHOST, &el->flags);
 	FD_SET(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
+	/* traceEvent(CONST_TRACE_WARNING, "-> %u/%u", the_local_network, the_local_network_mask); */
       } else if(hostIpAddress != NULL) {
 	/* This is packet that's being routed or belonging to a
 	   remote network that uses the same physical wire (or forged)*/
@@ -1017,10 +1039,10 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
 
 	  FD_CLR(FLAG_SUBNET_LOCALHOST, &el->flags);
 
-	  if(isPrivateAddress(hostIpAddress)) FD_SET(FLAG_PRIVATE_IP_ADDRESS, &el->flags);
+	  if(isPrivateAddress(hostIpAddress, &the_local_network, &the_local_network_mask)) FD_SET(FLAG_PRIVATE_IP_ADDRESS, &el->flags);
 
-	  if(!isBroadcastAddress(hostIpAddress)) {
-	    if(isPseudoLocalAddress(hostIpAddress, actualDeviceId))
+	  if(!isBroadcastAddress(hostIpAddress, &the_local_network, &the_local_network_mask)) {
+	    if(isPseudoLocalAddress(hostIpAddress, actualDeviceId, &the_local_network, &the_local_network_mask))
 	      FD_SET(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
 	    else
 	      FD_CLR(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
@@ -1047,7 +1069,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
 	}
 
 	FD_SET(FLAG_BROADCAST_HOST, &el->flags);
-	if(isMulticastAddress(&el->hostIpAddress))
+	if(isMulticastAddress(&el->hostIpAddress, &the_local_network, &the_local_network_mask))
 	  FD_SET(FLAG_MULTICAST_HOST, &el->flags);
 	strncpy(el->hostNumIpAddress,
 		_addrtostr(&el->hostIpAddress, buf, sizeof(buf)),
@@ -1056,7 +1078,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
 
 	if((!addrnull(&el->hostIpAddress)) /* 0.0.0.0 */
 	   && (!addrfull(&el->hostIpAddress)) /* 255.255.255.255 */
-	   && isBroadcastAddress(&el->hostIpAddress)) {
+	   && isBroadcastAddress(&el->hostIpAddress, &the_local_network, &the_local_network_mask)) {
 	  /*
 	    The sender of this packet has obviously a wrong netmask because:
 	    - it is a local host
@@ -1098,10 +1120,10 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
       strncpy(el->hostNumIpAddress,
 	      _addrtostr(hostIpAddress, buf, sizeof(buf)),
 	      sizeof(el->hostNumIpAddress));
-      if(isBroadcastAddress(&el->hostIpAddress)) FD_SET(FLAG_BROADCAST_HOST, &el->flags);
-      if(isMulticastAddress(&el->hostIpAddress)) FD_SET(FLAG_MULTICAST_HOST, &el->flags);
-      if(isPrivateAddress(hostIpAddress))        FD_SET(FLAG_PRIVATE_IP_ADDRESS,  &el->flags);
-      if((ether_addr == NULL) && (isPseudoLocalAddress(hostIpAddress, actualDeviceId)))
+      if(isBroadcastAddress(&el->hostIpAddress, &the_local_network, &the_local_network_mask)) FD_SET(FLAG_BROADCAST_HOST, &el->flags);
+      if(isMulticastAddress(&el->hostIpAddress, &the_local_network, &the_local_network_mask)) FD_SET(FLAG_MULTICAST_HOST, &el->flags);
+      if(isPrivateAddress(hostIpAddress, &the_local_network, &the_local_network_mask))        FD_SET(FLAG_PRIVATE_IP_ADDRESS,  &el->flags);
+      if((ether_addr == NULL) && (isPseudoLocalAddress(hostIpAddress, actualDeviceId, &the_local_network, &the_local_network_mask)))
 	FD_SET(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
 
       setResolvedName(el, el->hostNumIpAddress, FLAG_HOST_SYM_ADDR_TYPE_IP);
@@ -1110,11 +1132,13 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
       if(myGlobals.runningPref.numericFlag == 0)
 	ipaddr2str(el->hostIpAddress, 1);
 
+      getHostAS(el);
     } else {
       /* This is a new entry and hostIpAddress was NOT set.  Fill in MAC address, if we have it */
       if(symEthName[0] != '\0') {
         /* This is a local address so we have the MAC address */
-	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s%s", symEthName, &el->ethAddressString[8]);
+	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s%s", 
+		      symEthName, &el->ethAddressString[8]);
 
 	buf[MAX_LEN_SYM_HOST_NAME-1] = '\0';
         setResolvedName(el, buf, FLAG_HOST_SYM_ADDR_TYPE_MAC);
@@ -1122,16 +1146,17 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
     }
 
 #ifdef HASH_DEBUG
-      traceEvent(CONST_TRACE_INFO, "HASH_DEBUG: Adding %s/%s [idx=%d][device=%d][actualHashSize=%d][#hosts=%d]",
+      traceEvent(CONST_TRACE_INFO, "HASH_DEBUG: Adding %s/%s [idx=%d][device=%d]"
+		 "[actualHashSize=%d][#hosts=%d]",
 		 el->ethAddressString, el->hostNumIpAddress, idx, actualDeviceId,
-		 myGlobals.device[actualDeviceId].actualHashSize, myGlobals.device[actualDeviceId].hostsno);
+		 myGlobals.device[actualDeviceId].actualHashSize, 
+		 myGlobals.device[actualDeviceId].hostsno);
 #endif
 
     setHostSerial(el);
     handlePluginHostCreationDeletion(el, (u_short)actualDeviceId, 1 /* host creation */);
   }
 
-
   if(el != NULL) {
     el->lastSeen = myGlobals.actTime;
 
@@ -1161,6 +1186,8 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, short vlan
   hashSanityCheck();
 #endif
 
+  if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+  
   return(el);
 }
 
@@ -1171,6 +1198,7 @@ HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
   u_int idx;
   HostTraffic *el=NULL;
   FcNameServerCacheEntry *fcnsEntry;
+  u_char locked_mutex = 0;
   u_short numRuns=0;
   u_int hostFound = 0;
 
@@ -1188,9 +1216,14 @@ HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
   }
   else if(idx == FLAG_NO_PEER)
     return(NULL);
-  else
+  else {
     el = myGlobals.device[actualDeviceId].hash_hostTraffic[idx];
-
+    if(el) {
+      lockHostsHashMutex(el, "lookupFcHost");
+      el = myGlobals.device[actualDeviceId].hash_hostTraffic[idx];
+      locked_mutex = 1;
+    }
+  }
 
   hostFound = 0;  /* This is the same type as the one of HashList */
 
@@ -1233,30 +1266,24 @@ HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
 		   myGlobals.runningPref.maxNumHashEntries);
       }
 
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
       return(NULL);
     }
 
-#if RECYCLE_MEMORY
-    if(myGlobals.hostsCacheLen > 0) {
-      el = myGlobals.hostsCache[--myGlobals.hostsCacheLen];
-      myGlobals.hostsCacheReused++;
-      /*
-	traceEvent(CONST_TRACE_INFO, "Fetched host from pointers cache (len=%d)",
-	(int)myGlobals.hostsCacheLen);
-      */
-    } else
-#endif
-      {
-	if((el = (HostTraffic*)malloc(sizeof(HostTraffic))) == NULL)
-	  return(NULL);
-      }
+    if((el = (HostTraffic*)malloc(sizeof(HostTraffic))) == NULL) {
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      return(NULL);
+    }
 
     memset(el, 0, sizeof(HostTraffic));
     el->firstSeen = myGlobals.actTime;
 
     resetHostsVariables(el);
 
-    if(allocFcScsiCounters(el) == NULL) return(NULL);
+    if(allocFcScsiCounters(el) == NULL) {
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      return(NULL);
+    }
     el->l2Family = FLAG_HOST_TRAFFIC_AF_FC;
     el->fcCounters->devType = SCSI_DEV_UNINIT;
     el->magic = CONST_MAGIC_NUMBER;
@@ -1309,6 +1336,7 @@ HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
   hashSanityCheck();
 #endif
 
+  if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
   return(el);
 }
 
@@ -1369,3 +1397,65 @@ static void hostHashSanityCheck(HostTraffic *host) {
 
 #endif /* HASH_DEBUG */
 
+/* ****************************** 
+   
+   Utility functions used by the remote plugin
+   
+   ****************************** */
+
+#define MAX_NUM_VALID_PTRS   8
+static void* valid_ptrs[MAX_NUM_VALID_PTRS] = { NULL };
+
+void add_valid_ptr(void* ptr) {
+  int i;
+  
+  traceEvent(CONST_TRACE_INFO, "add_valid_ptr(%p)", ptr);
+
+  for(i=0; i<MAX_NUM_VALID_PTRS; i++) {
+    if(valid_ptrs[i] == NULL) {
+      valid_ptrs[i] = ptr;
+      break;
+    }
+  }
+
+  valid_ptrs[MAX_NUM_VALID_PTRS-1] = ptr;
+}
+
+/* ****************************** */
+
+void remove_valid_ptr(void* ptr) {
+  int i;  
+
+  for(i=0; i<MAX_NUM_VALID_PTRS; i++) {
+    if(valid_ptrs[i] == ptr) {
+      valid_ptrs[i] = NULL;
+      return;
+    }
+  } 
+
+  /* traceEvent(CONST_TRACE_ERROR, "remove_valid_ptr(%p) failed", ptr); */
+}
+
+/* ****************************** */
+
+int is_valid_ptr(void* ptr) {
+  int i;
+
+  for(i=0; i<MAX_NUM_VALID_PTRS; i++) {
+    if(valid_ptrs[i] == ptr) {
+      if(i > 0) {
+	/* Move towards the top */
+	void *swap = valid_ptrs[i-1];
+	valid_ptrs[i-1] = valid_ptrs[i];
+	valid_ptrs[i] = swap;
+      }
+      
+      traceEvent(CONST_TRACE_INFO, "is_valid_ptr(%p): 1", ptr);
+      return(1);
+    }
+  }
+
+  traceEvent(CONST_TRACE_INFO, "is_valid_ptr(%p): 0", ptr);
+  return(0);
+}
+
