@@ -725,3 +725,130 @@ int iface_getalladdr(int type, int *size, char **addr){
 #endif
 
 #endif /* INET6 */
+
+/* ********************************************************************** */
+
+void sanitizeIfName(char *deviceDescr) {
+#ifdef WIN32
+	int i;
+
+	for(i=0; i<strlen(deviceDescr); i++)
+    switch(deviceDescr[i]) {
+    case ':':
+    case '/':
+    case '\\':
+	case '.':
+	case ' ':
+      deviceDescr[i] = '_';
+    }
+#endif
+}
+
+/* ********************************************************************** */
+
+void calculateUniqueInterfaceName(int deviceId) {
+#ifdef WIN32
+  int i;
+#endif
+
+  if(myGlobals.device[deviceId].uniqueIfName) 
+    free(myGlobals.device[deviceId].uniqueIfName);
+
+  myGlobals.device[deviceId].uniqueIfName = strdup(myGlobals.device[deviceId].humanFriendlyName);
+  sanitizeIfName(myGlobals.device[deviceId].uniqueIfName);
+}
+
+/* ********************************************************************** */
+
+#ifdef HAVE_SNMP
+
+#include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-includes.h>
+#include <string.h>
+
+#ifndef min
+#define min(a,b) (a<b ? a : b)
+#endif
+
+char* getIfName(char *hostname, char *community, int ifIdx,
+		char *ifName_buf, u_short ifName_buflen) {
+  struct snmp_session session, *ss;
+  struct snmp_pdu *pdu;
+  struct snmp_pdu *response;
+  char buf[64];
+  oid anOID[MAX_OID_LEN];
+  size_t anOID_len = MAX_OID_LEN;
+
+  struct variable_list *vars;
+  int status;
+  int count=1;
+
+  ifName_buf[0] = '\0';
+
+  /*
+   * Initialize the SNMP library
+   */
+  init_snmp("ntop");
+
+  /*
+   * Initialize a "session" that defines who we're going to talk to
+   */
+  snmp_sess_init( &session );                   /* set up defaults */
+  session.peername = strdup(hostname);
+
+  /* set up the authentication parameters for talking to the server */
+
+  /* set the SNMP version number */
+  session.version = SNMP_VERSION_1;
+
+  /* set the SNMPv1 community name used for authentication */
+  session.community = (u_char*)community;
+  session.community_len = strlen(community);
+
+  /*
+   * Open the session
+   */
+  ss = snmp_open(&session); /* establish the session */
+
+  if (!ss)
+    return(ifName_buf);  
+    
+  pdu = snmp_pdu_create(SNMP_MSG_GET);
+
+  snprintf(buf, sizeof(buf), ".1.3.6.1.2.1.31.1.1.1.1.%d", ifIdx);
+  read_objid(buf, anOID, &anOID_len); snmp_add_null_var(pdu, anOID, anOID_len);
+
+  traceEvent(CONST_TRACE_NOISY, 
+	     "Reading SNMP interface name: [host=%s][community=%s][%s]",
+	     hostname, community, buf);
+  /*
+   * Send the Request out.
+   */
+  status = snmp_synch_response(ss, pdu, &response);
+
+  /*
+   * Process the response.
+   */
+  if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
+    /* manipuate the information ourselves */
+    for(vars = response->variables; vars; vars = vars->next_variable) {
+      if (vars->type == ASN_OCTET_STR) {
+	int len = min(vars->val_len, ifName_buflen-1);
+	memcpy(ifName_buf, vars->val.string, len);
+	ifName_buf[len] = '\0';
+      }
+    }
+  }
+
+  /*
+   * Clean up:
+   *  1) free the response.
+   *  2) close the session.
+   */
+  if (response) snmp_free_pdu(response);
+  snmp_close(ss);
+
+  return (ifName_buf);
+}
+
+#endif
