@@ -443,3 +443,11 @@ void *iface_addr_getinfo(struct iface_addr *ia, void *infobuf);
 
 #endif /* _IFACE_H_ */
 #endif /* INET6 */
+
+extern void calculateUniqueInterfaceName(int deviceId);
+extern void sanitizeIfName(char *deviceDescr);
+
+#ifdef HAVE_SNMP
+extern char* getIfName(char *hostname, char *community, int ifIdx,
+		       char *ifName_buf, u_short ifName_buflen);
+#endif
