@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -96,10 +96,11 @@ static void setDomainName(void) {
   while((len > 0) && (myGlobals.runningPref.domainName[len] != '.'))
     len--;
 
-  if(len == 0)
-    myGlobals.shortDomainName = strdup(myGlobals.runningPref.domainName);
-  else
+  if((len > 0) 
+     && ((len+1) <  strlen(myGlobals.runningPref.domainName)))
     myGlobals.shortDomainName = strdup(&myGlobals.runningPref.domainName[len+1]);
+  else
+    myGlobals.shortDomainName = strdup(myGlobals.runningPref.domainName);
 }
 
 /* ------------------------------------------------------------ */
@@ -118,7 +119,7 @@ void initIPServices(void) {
   /* Let's count the entries first */
   numSlots = 0;
   for(idx=0; myGlobals.configFileDirs[idx] != NULL; idx++) {
-    char tmpStr[64];
+    char tmpStr[256];
 
     safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr), "%s/services", myGlobals.configFileDirs[idx]);
     fd = fopen(tmpStr, "r");
@@ -226,11 +227,11 @@ static void initIPCountryTable(void) {
   myGlobals.countryFlagHead->b[1]=NULL;
 
   if (!myGlobals.runningPref.printFcOnly) {
-      fd=checkForInputFile("IP2CC",
-                           "IP address <-> Country Code mapping",
-                           CONST_P2C_FILE,
-                           NULL,
-                           &compressedFormat);
+    fd=checkForInputFile("IP2CC",
+			 "IP address <-> Country Code mapping",
+			 CONST_P2C_FILE,
+			 NULL,
+			 &compressedFormat);
       if(fd != NULL) {
           char *strtokState, *cc, *ip, *prefix;
           int numRead=0;
@@ -272,7 +273,6 @@ void createDeviceIpProtosList(int devIdx) {
   size_t len = (size_t)myGlobals.numIpProtosList*sizeof(TrafficCounter);
 
   if(len > 0) {
-
     if(myGlobals.device[devIdx].ipProtosList != NULL)
       free(myGlobals.device[devIdx].ipProtosList);
     if((myGlobals.device[devIdx].ipProtosList = (TrafficCounter*)malloc(len)) == NULL)
@@ -576,7 +576,7 @@ void initCounters(void) {
               myGlobals.asCount++;
           }
           traceEvent(CONST_TRACE_INFO, "ASN: ....Used %d KB of memory (%d per entry)",
-                     ((myGlobals.asMem+512)/1024), sizeof(IPNode));
+                     ((myGlobals.asMem+512)/1024), (int)sizeof(IPNode));
           myGlobals.haveASN = TRUE;
       } else
           traceEvent(CONST_TRACE_NOISY, "ASN: ntop continues ok, but without ASN information.");
@@ -750,17 +750,6 @@ void initCounters(void) {
 #endif /* MAKE_WITH_I18N */
 
   initIPCountryTable();
-
-  /*
-   * Guess at the version of gd.  Since there's no gdVersion() function, we do
-   * this in an ugly way - we load the .so library and look for the entry point
-   * of functions that have been added along the way.
-   */
-  traceEvent(CONST_TRACE_INFO, "GDVERCHK: Guessing at libgd version");
-  myGlobals.gdVersionGuessValue = strdup(gdVersionGuess());
-  if(myGlobals.gdVersionGuessValue != NULL)
-    traceEvent(CONST_TRACE_INFO, "GDVERCHK: ... as %s", myGlobals.gdVersionGuessValue);
-
 }
 
 
@@ -850,7 +839,8 @@ void resetStats(int deviceId) {
 
 /* ******************************* */
 
-void initSingleGdbm(GDBM_FILE *database, char *dbName, char *directory,
+void initSingleGdbm(GDBM_FILE *database,
+		    char *dbName, char *directory,
 		    int doUnlink, struct stat *statbuf) {
   char tmpBuf[200], theDate[48];
   time_t        st_time, now;
@@ -858,15 +848,28 @@ void initSingleGdbm(GDBM_FILE *database, char *dbName, char *directory,
   int d;
 
   /* Courtesy of Andreas Pfaller <apfaller@yahoo.com.au>. */
-  /* directory is used by intop to specify where to open the files.
-     If called with NULL, use the myGlobals.dbPath value instead
-     (Minor fix for intop - Burton Strauss (BStrauss@acm.org) - Apr2002)
-  */
-
   memset(&tmpBuf, 0, sizeof(tmpBuf));
+ 
+#ifdef WIN32
+  {
+    unsigned long driveSerial;
+  
+    get_serial(&driveSerial);
+
+    safe_snprintf(__FILE__, __LINE__, tmpBuf, sizeof(tmpBuf), "%s/%u",
+		  directory != NULL ? directory : myGlobals.dbPath, driveSerial);
+
+    mkdir_p("DB", tmpBuf, 0x777);
+
+    safe_snprintf(__FILE__, __LINE__, tmpBuf, sizeof(tmpBuf), "%s/%u/%s",
+		  directory != NULL ? directory : myGlobals.dbPath, driveSerial,
+		  dbName);
+  }
+#else
   safe_snprintf(__FILE__, __LINE__, tmpBuf, sizeof(tmpBuf), "%s/%s",
 	      directory != NULL ? directory : myGlobals.dbPath,
 	      dbName);
+#endif
 
   if(statbuf) {
     if(stat(tmpBuf, statbuf) == 0) {
@@ -912,7 +915,7 @@ void initSingleGdbm(GDBM_FILE *database, char *dbName, char *directory,
   traceEvent(CONST_TRACE_NOISY, "%s database '%s'",
 	     doUnlink == TRUE ? "Creating" : "Opening",
 	     tmpBuf);
-  *database = gdbm_open (tmpBuf, 0, GDBM_WRCREAT, 00664, NULL);
+  *database = gdbm_open (tmpBuf, 0, GDBM_WRCREAT, 00640, NULL);
 
   if(*database == NULL) {
     traceEvent(CONST_TRACE_ERROR, "....open of %s failed: %s",
@@ -962,9 +965,6 @@ void reinitMutexes (void) {
                                              * access */
   createMutex(&myGlobals.fcSessionsMutex);
   createMutex(&myGlobals.purgePortsMutex);  /* data to synchronize port purge access */
-  createMutex(&myGlobals.packetQueueMutex);
-  createMutex(&myGlobals.packetProcessMutex);
-
   createMutex(&myGlobals.purgePortsMutex);  /* data to synchronize port purge access */
 
   for(i=0; i<CONST_HASH_INITIAL_SIZE; i++) {
@@ -973,6 +973,7 @@ void reinitMutexes (void) {
   }
 
   createMutex(&myGlobals.securityItemsMutex);
+  createMutex(&myGlobals.hostsHashLockMutex);
 
   if(myGlobals.runningPref.numericFlag == 0) {
     createMutex(&myGlobals.addressResolutionMutex);
@@ -990,9 +991,12 @@ void reinitMutexes (void) {
 void initThreads(void) {
   int i;
 
-  createThread(&myGlobals.dequeueThreadId, dequeuePacket, NULL);
-  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPA: Started thread for network packet analyzer",
-             (long)myGlobals.dequeueThreadId);
+  for(i=0; i<myGlobals.numDevices; i++) {
+    createThread(&myGlobals.device[i].dequeuePacketThreadId, dequeuePacket, (void*)((long)i));
+    traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPA: Started thread for network packet analyzer (%s)",
+	       (long)myGlobals.device[i].dequeuePacketThreadId,
+	       myGlobals.device[i].humanFriendlyName);
+  }
 
   /*
    * Create the thread (3) - SFP - Scan Fingerprints
@@ -1011,11 +1015,13 @@ void initThreads(void) {
   if(myGlobals.runningPref.numericFlag == 0) {
     createMutex(&myGlobals.addressResolutionMutex);
 
+    myGlobals. numDequeueAddressThreads = MAX_NUM_DEQUEUE_ADDRESS_THREADS;
+
     /*
      * Create the thread (6) - DNSAR - DNS Address Resolution - optional
      */
-    for(i=0; i<myGlobals.numDequeueThreads; i++) {
-      createThread(&myGlobals.dequeueAddressThreadId[i], dequeueAddress, (char*)i);
+    for(i=0; i<myGlobals.numDequeueAddressThreads; i++) {
+      createThread(&myGlobals.dequeueAddressThreadId[i], dequeueAddress, (char*)((long)i));
       traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: DNSAR(%d): Started thread for DNS address resolution",
 		 (long)myGlobals.dequeueAddressThreadId[i], i+1);
     }
@@ -1033,7 +1039,6 @@ void initThreads(void) {
 #endif
 }
 
-
 /*
  * Initialize helper applications
  */
@@ -1042,6 +1047,37 @@ void initApps(void) {
   /* Nothing to do at the moment */
 }
 
+/* ******************************* */
+
+void initDeviceSemaphores(int deviceId) {
+  traceEvent(CONST_TRACE_INFO, "Initializing device %s (%d)",
+	     myGlobals.device[deviceId].name, deviceId);
+    
+  createMutex(&myGlobals.device[deviceId].asMutex);
+  createMutex(&myGlobals.device[deviceId].packetProcessMutex);
+  createMutex(&myGlobals.device[deviceId].packetQueueMutex);
+  memset(&myGlobals.device[deviceId].packetQueue, 0, sizeof(PacketInformation));
+  myGlobals.device[deviceId].packetQueueLen           = 0;
+  myGlobals.device[deviceId].maxPacketQueueLen        = 0;
+  myGlobals.device[deviceId].packetQueueHead          = 0;
+  myGlobals.device[deviceId].packetQueueTail          = 0;
+
+  createCondvar(&myGlobals.device[deviceId].queueCondvar);
+}
+
+/* ******************************* */
+
+void allocDeviceMemory(int deviceId) {
+  if(!myGlobals.device[deviceId].ipPorts)
+    myGlobals.device[deviceId].ipPorts = 
+      (PortCounter**)calloc(sizeof(PortCounter*), MAX_IP_PORT);
+  
+  if(!myGlobals.device[deviceId].packetQueue)
+    myGlobals.device[deviceId].packetQueue = 
+      (PacketInformation*)calloc(sizeof(PacketInformation), (CONST_PACKET_QUEUE_LENGTH+1));
+}
+
+/* ******************************* */
 
 /*
  * Initialize the table of NICs enabled for packet sniffing
@@ -1061,9 +1097,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
   int i, deviceId, mallocLen, memlen;
   NtopInterface *tmpDevice, *oldDevice;
   char *workDevices = NULL;
-  char myName[80], *column = NULL;
-  char ebuf[CONST_SIZE_PCAP_ERR_BUF];
-
+  char myName[80], *column = NULL, ebuf[CONST_SIZE_PCAP_ERR_BUF], tmpStr[64];
 
   ebuf[0] = '\0', myName[0] = '\0';
 
@@ -1073,13 +1107,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
   }
 
   /* Remove unwanted characters */
-  for(i=0; i<strlen(deviceDescr); i++)
-    switch(deviceDescr[i]) {
-    case ':':
-    case '/':
-    case '\\':
-      deviceDescr[i] = '_';
-    }
+ sanitizeIfName(deviceName);
 
   traceEvent(CONST_TRACE_NOISY, "Adding network device %s", deviceName);
 
@@ -1095,9 +1123,18 @@ void addDevice(char* deviceName, char* deviceDescr) {
     myGlobals.device = tmpDevice;
     if (oldDevice != NULL) free(oldDevice);
     deviceId = myGlobals.numDevices;
-    myGlobals.device[deviceId].humanFriendlyName = strdup(deviceDescr);
-    myGlobals.device[deviceId].name = strdup(deviceName);
+
+    safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr), "device.name.%s", deviceName);
+    if(fetchPrefsValue(tmpStr, ebuf, sizeof(ebuf)) != -1)
+      myGlobals.device[deviceId].humanFriendlyName = strdup(ebuf);
+    else
+      myGlobals.device[deviceId].humanFriendlyName = strdup(deviceDescr);
+
+    allocDeviceMemory(deviceId);
+
+    myGlobals.device[deviceId].name         = strdup(deviceName);
     myGlobals.device[deviceId].samplingRate =  myGlobals.runningPref.samplingRate;
+    calculateUniqueInterfaceName(deviceId);
     myGlobals.numDevices++;
 
     if(myGlobals.numDevices >= MAX_NUM_DEVICES) {
@@ -1185,29 +1222,15 @@ void addDevice(char* deviceName, char* deviceDescr) {
         exit(10); /* Just in case */
       }
 
-#if !defined(WIN32) && defined(HAVE_PCAP_SETNONBLOCK)
-      if(myGlobals.runningPref.setNonBlocking == TRUE) {
-       char errBuf[100];
-       memset(&errBuf, 0, sizeof(errBuf));
-       if(pcap_setnonblock(myGlobals.device[deviceId].pcapPtr, TRUE, errBuf) != 0) {
-         traceEvent(CONST_TRACE_ERROR, "NONBLOCK: pcap_setnonblock(), device %d, failed (%s)", deviceId, errBuf); 
-         traceEvent(CONST_TRACE_WARNING, "NONBLOCK: web server may appear to hang"); 
-       } else {
-         traceEvent(CONST_TRACE_INFO, "NONBLOCK: pcap_setnonblock(), device %d, succeeded", deviceId);
-         traceEvent(CONST_TRACE_WARNING, "NONBLOCK: cpu usage may be high - see man page and FAQ");
-       }
-      }
-#endif
-
       if(myGlobals.runningPref.pcapLog != NULL) {
 	if(strlen(myGlobals.runningPref.pcapLog) > 64)
 	  myGlobals.runningPref.pcapLog[64] = '\0';
         safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s%c%s.%s.pcap",
-                myGlobals.runningPref.pcapLogBasePath, /* Added by Ola Lundqvist <opal@debian.org> */
-                CONST_PATH_SEP, myGlobals.runningPref.pcapLog,
-                myGlobals.device[deviceId].humanFriendlyName != NULL ?
-                    myGlobals.device[deviceId].humanFriendlyName :
-                    myGlobals.device[deviceId].name);
+		      myGlobals.runningPref.pcapLogBasePath, /* Added by Ola Lundqvist <opal@debian.org> */
+		      CONST_PATH_SEP, myGlobals.runningPref.pcapLog,
+		      myGlobals.device[deviceId].uniqueIfName != NULL ?
+		      myGlobals.device[deviceId].uniqueIfName :
+		      myGlobals.device[deviceId].name);
 
 	myGlobals.device[deviceId].pcapDumper = pcap_dump_open(myGlobals.device[deviceId].pcapPtr, myName);
 
@@ -1224,8 +1247,8 @@ void addDevice(char* deviceName, char* deviceDescr) {
 	  safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s%cntop-suspicious-pkts.dev%s.pcap",
 			myGlobals.runningPref.pcapLogBasePath, /* Added by Ola Lundqvist <opal@debian.org> */
 			CONST_PATH_SEP,
-			myGlobals.device[deviceId].humanFriendlyName != NULL ?
-			myGlobals.device[deviceId].humanFriendlyName :
+			myGlobals.device[deviceId].uniqueIfName != NULL ?
+			myGlobals.device[deviceId].uniqueIfName :
 			myGlobals.device[deviceId].name);
 	else
 	  safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s%cntop-suspicious-pkts.pcap",
@@ -1244,11 +1267,11 @@ void addDevice(char* deviceName, char* deviceDescr) {
 
       if(myGlobals.runningPref.enableOtherPacketDump) {
         safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s%cntop-other-pkts.%s.pcap",
-                myGlobals.runningPref.pcapLogBasePath,
-                CONST_PATH_SEP,
-                myGlobals.device[deviceId].humanFriendlyName != NULL ?
-                    myGlobals.device[deviceId].humanFriendlyName :
-                    myGlobals.device[deviceId].name);
+		      myGlobals.runningPref.pcapLogBasePath,
+		      CONST_PATH_SEP,
+		      myGlobals.device[deviceId].uniqueIfName != NULL ?
+		      myGlobals.device[deviceId].uniqueIfName :
+		      myGlobals.device[deviceId].name);
 
 	myGlobals.device[deviceId].pcapOtherDumper = pcap_dump_open(myGlobals.device[deviceId].pcapPtr, myName);
 
@@ -1286,7 +1309,8 @@ void addDevice(char* deviceName, char* deviceDescr) {
     }
 
     if((myGlobals.device[deviceId].network.s_addr == 0) &&
-       (myGlobals.device[deviceId].netmask.s_addr == 0xFFFFFFFF) ) { /* Unnumbered interface... */
+       (myGlobals.device[deviceId].netmask.s_addr == 0xFFFFFFFF) ) {
+      /* Unnumbered interface... */
       myGlobals.device[deviceId].numHosts = MAX_SUBNET_HOSTS;
     } else {
       myGlobals.device[deviceId].numHosts = 0xFFFFFFFF - myGlobals.device[deviceId].netmask.s_addr + 1;
@@ -1358,7 +1382,9 @@ void addDevice(char* deviceName, char* deviceDescr) {
   /* ********************************************* */
 
   if(!(myGlobals.device[deviceId].dummyDevice || myGlobals.device[deviceId].virtualDevice)){
-    getLocalHostAddress(&myGlobals.device[deviceId].ifAddr, myGlobals.device[deviceId].name);
+    u_int8_t netmask_v6;
+    
+    getLocalHostAddress(&myGlobals.device[deviceId].ifAddr, &netmask_v6, myGlobals.device[deviceId].name);
 #ifdef INET6
     myGlobals.device[deviceId].v6Addrs = getLocalHostAddressv6(myGlobals.device[deviceId].v6Addrs, myGlobals.device[deviceId].name);
 #endif
@@ -1409,13 +1435,16 @@ void addDevice(char* deviceName, char* deviceDescr) {
     struct in_addr myLocalHostAddress;
 
 
-    if(myGlobals.numDevices < MAX_NUM_DEVICES) {
+    if((myGlobals.numDevices < MAX_NUM_DEVICES)
+       && strcmp(myGlobals.device[deviceId].name, "none")) {
       traceEvent(CONST_TRACE_INFO, "Checking %s for additional devices", myGlobals.device[deviceId].name);
       for(k=0; k<=MAX_NUM_DEVICES_VIRTUAL; k++) {
+	u_int8_t netmask_v6;
+	
 	safe_snprintf(__FILE__, __LINE__, tmpDeviceName, sizeof(tmpDeviceName), "%s:%d", myGlobals.device[deviceId].name, k);
 
 	traceEvent(CONST_TRACE_NOISY, "Checking %s", tmpDeviceName);
-	if(getLocalHostAddress(&myLocalHostAddress, tmpDeviceName) == 0) {
+	if(getLocalHostAddress(&myLocalHostAddress, &netmask_v6, tmpDeviceName) == 0) {
 	  /* The virtual interface exists */
 
 	  mallocLen = sizeof(NtopInterface)*(myGlobals.numDevices+1);
@@ -1431,12 +1460,14 @@ void addDevice(char* deviceName, char* deviceDescr) {
 	 myGlobals.device[myGlobals.numDevices].virtualDevice = 1;
 	 myGlobals.device[myGlobals.numDevices].activeDevice = 1;
 	 myGlobals.device[myGlobals.numDevices].humanFriendlyName = strdup(tmpDeviceName);
-	  myGlobals.device[myGlobals.numDevices++].name = strdup(tmpDeviceName);
-	  traceEvent(CONST_TRACE_INFO, "Added virtual interface: '%s'", tmpDeviceName);
-          if(myGlobals.numDevices >= MAX_NUM_DEVICES) {
-	    traceEvent(CONST_TRACE_WARNING, "Stopping scan - no room for additional (virtual) interfaces");
-	    break;
-          }
+	 myGlobals.device[myGlobals.numDevices].name = strdup(tmpDeviceName);
+	 calculateUniqueInterfaceName(myGlobals.numDevices);
+	 myGlobals.numDevices++;
+	 traceEvent(CONST_TRACE_INFO, "Added virtual interface: '%s'", tmpDeviceName);
+	 if(myGlobals.numDevices >= MAX_NUM_DEVICES) {
+	   traceEvent(CONST_TRACE_WARNING, "Stopping scan - no room for additional (virtual) interfaces");
+	   break;
+	 }
 	}
       }
     }
@@ -1448,7 +1479,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
 
   if((myGlobals.actualReportDeviceId == 0) && myGlobals.device[0].dummyDevice)
     myGlobals.actualReportDeviceId = deviceId;
-
+  
 }
 
 /* ******************************* */
@@ -1493,10 +1524,11 @@ void initDevices(char* devices) {
     if(myGlobals.device[0].pcapPtr == NULL) {
       traceEvent(CONST_TRACE_ERROR, "pcap_open_offline(%s): '%s'",
 		 myGlobals.runningPref.rFileName, ebuf);
-      return;
+      exit(-1);
     } else {
       if(myGlobals.device[0].humanFriendlyName != NULL) free(myGlobals.device[0].humanFriendlyName);
       myGlobals.device[0].humanFriendlyName = strdup(myGlobals.runningPref.rFileName);
+      calculateUniqueInterfaceName(0);
     }
 
     resetStats(0);
@@ -1507,8 +1539,8 @@ void initDevices(char* devices) {
 	safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s%cntop-suspicious-pkts.%s.pcap",
 		      myGlobals.runningPref.pcapLogBasePath, /* Added by Ola Lundqvist <opal@debian.org> */
 		      CONST_PATH_SEP,
-		      myGlobals.device[0].humanFriendlyName != NULL ?
-		      myGlobals.device[0].humanFriendlyName :
+		      myGlobals.device[0].uniqueIfName != NULL ?
+		      myGlobals.device[0].uniqueIfName :
 		      myGlobals.device[0].name);
       else
 	safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s.ntop-suspicious-pkts.pcap",
@@ -1597,7 +1629,10 @@ void initDevices(char* devices) {
 
       traceEvent(CONST_TRACE_NOISY, "Checking requested device '%s'", tmpDev);
 
-      if((nwInterface = strchr(tmpDev, ':')) != NULL) {
+      if(((nwInterface = strchr(tmpDev, ':')) != NULL) 
+	 && (!strstr(tmpDev, "dag")) /* Endace DAG cards are valid (e.g. dag0:0) */
+	 )
+	{
  	/* This is a virtual nwInterface */
         char *requestedDev;
 
@@ -1689,6 +1724,7 @@ void initDevices(char* devices) {
 void initDeviceDatalink(int deviceId) {
   if(myGlobals.device[deviceId].dummyDevice) return;
   myGlobals.device[deviceId].activeDevice = 1;
+  initDeviceSemaphores(deviceId);
   if(myGlobals.device[deviceId].virtualDevice) return;
 
   /* get datalink type */
@@ -1744,7 +1780,6 @@ void initDeviceDatalink(int deviceId) {
 	       myGlobals.device[deviceId].datalink,
 	       myGlobals.device[deviceId].mtuSize,
 	       myGlobals.device[deviceId].headerSize);
-
 }
 
 /* ******************************* */
@@ -1787,10 +1822,6 @@ static void ignoreThisSignal(int signalId) {
 
 /* ******************************* */
 
-#ifdef HANDLE_DIED_CHILD
-extern RETSIGTYPE handleDiedChild(int sig _UNUSED_); /*FreeBSD hack: to remove */
-#endif
-
 void initSignals(void) {
   /*
     The handler below has been restored due
@@ -1798,11 +1829,7 @@ void initSignals(void) {
     Courtesy of Martin Lucina <mato@kotelna.sk>
   */
 #ifndef WIN32
-#ifdef HANDLE_DIED_CHILD
   signal(SIGCHLD, handleDiedChild);
-#else
-  signal(SIGCHLD, SIG_IGN);
-#endif
 #endif
 
 #ifndef WIN32
@@ -1842,8 +1869,8 @@ void startSniffer(void) {
       /*
        * (8) - NPS - Network Packet Sniffer (main thread)
        */
-      createThread(&myGlobals.device[i].pcapDispatchThreadId, pcapDispatch, (char*)i);
-      traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%d): Started thread for network packet sniffing",
+      createThread(&myGlobals.device[i].pcapDispatchThreadId, pcapDispatch, (char*)((long)i));
+      traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%d): Started thread for network packet sniffing [%s]",
 		 (long)myGlobals.device[i].pcapDispatchThreadId, i+1, myGlobals.device[i].name);
     }
 }
@@ -1865,12 +1892,14 @@ u_int createDummyInterface(char *ifName) {
   memset(tmpDevice, 0, mallocLen);
 
   if(myGlobals.numDevices > 0) {
+    NtopInterface *old_ptr = myGlobals.device;
     memcpy(tmpDevice, myGlobals.device,
 	   sizeof(NtopInterface)*myGlobals.numDevices);
-    free(myGlobals.device);
-  }
+    myGlobals.device = tmpDevice;
+    free(old_ptr);
+  } else
+    myGlobals.device = tmpDevice;
 
-  myGlobals.device = tmpDevice;
   myGlobals.numDevices++;
   memset(&myGlobals.device[deviceId], 0, sizeof(NtopInterface));
 
@@ -1887,6 +1916,7 @@ u_int createDummyInterface(char *ifName) {
   myGlobals.device[deviceId].virtualDevice = 0;
   myGlobals.device[deviceId].activeDevice  = 0;
   myGlobals.device[deviceId].samplingRate =  myGlobals.runningPref.samplingRate;
+  calculateUniqueInterfaceName(deviceId);
 
   if(myGlobals.otherHostEntry != NULL) {
     myGlobals.device[deviceId].hash_hostTraffic[OTHER_HOSTS_ENTRY] = myGlobals.otherHostEntry;
