@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -46,21 +46,28 @@
 
 #ifdef MAKE_WITH_SAFER_ROUTINES
 
-static int glib23xMessageWritten = 0;
-
 static void stopcap(void) {
+  int i;
+
   traceEvent(CONST_TRACE_WARNING, "ntop packet capture STOPPED");
   traceEvent(CONST_TRACE_INFO, "NOTE: ntop web server remains up");
   traceEvent(CONST_TRACE_INFO, "NOTE: Shutdown gracefully and restart with more memory");
   setRunState(FLAG_NTOPSTATE_STOPCAP);
-  signalCondvar(&myGlobals.queueCondvar);
+
+  for(i=0; i<myGlobals.numDevices; i++) signalCondvar(&myGlobals.device[i].queueCondvar);
+
   signalCondvar(&myGlobals.queueAddressCondvar);
 }
 
 #undef malloc /* just to be safe */
 
+/* #define COUNT_MALLOCS 1 */
+
 void* ntop_safemalloc(unsigned int sz, char* file, int line) {
   void *thePtr;
+#ifdef COUNT_MALLOCS
+  static u_int num_allocs = 0, tot_allocs = 0;
+#endif
 
 #ifdef DEBUG
   if((sz == 0) || (sz > 32768)) {
@@ -77,6 +84,10 @@ void* ntop_safemalloc(unsigned int sz, char* file, int line) {
   thePtr = GC_malloc_atomic(sz);
 #endif
 
+#if COUNT_MALLOCS
+  tot_allocs += sz, num_allocs++; traceEvent(CONST_TRACE_ERROR, "[num_allocs=%u][size=%u][total=%u]", num_allocs, sz, tot_allocs);
+#endif
+
   if(thePtr == NULL) {
     traceEvent(CONST_TRACE_ERROR, "malloc(%u) @ %s:%d returned NULL [no more memory?]",
 	       sz, file, line);
@@ -169,22 +180,6 @@ void ntop_safefree(void **ptr, char* file, int line) {
     traceEvent(CONST_TRACE_WARNING, "free of NULL pointer @ %s:%d", 
 	       file, line);
   } else {
-#if (0)
-   /* **********DIAGNOSTIC**********
-      * Enable this code to look   *
-      * for potential glibc 2.3.x  *
-      * problems with free() in    *
-      * fork()ed child             *
-      ******************************
-      if (myGlobals.childntoppid != 0) {
-          traceEvent(CONST_TRACE_NOISY, "GLIBC23X: free in fork()ed child @ %s:%d", file, line);
-          if (glib23xMessageWritten == 0) {
-              traceEvent(CONST_TRACE_NOISY, "GLIBC23X: Please notify ntop-dev of NEW occurances");
-              glib23xMessageWritten = 1;
-          }
-      }
-      **********DIAGNOSTIC********** */
-#endif
     free(*ptr);
     *ptr = NULL;
   }
@@ -196,10 +191,14 @@ void ntop_safefree(void **ptr, char* file, int line) {
 char* ntop_safestrdup(char *ptr, char* file, int line) {  
   if(ptr == NULL) {
     traceEvent(CONST_TRACE_WARNING, "strdup of NULL pointer @ %s:%d", file, line);
-    return(strdup(""));
+#ifdef WIN32
+	return(_strdup(""));
+#else
+	return(strdup(""));
+#endif
   } else {
     char* theOut;
-    int len = strlen(ptr);
+    int len = (int)strlen(ptr);
     
 #ifndef USE_GC
     theOut = (char*)malloc((len+1)*sizeof(char));
@@ -808,6 +807,11 @@ datum ntop_gdbm_fetch(GDBM_FILE g, datum d, char* theFile, int theLine) {
 int ntop_gdbm_delete(GDBM_FILE g, datum d, char* theFile, int theLine) {
   int rc;
 
+  if((d.dptr == NULL) || (d.dsize == 0)) {
+    traceEvent(CONST_TRACE_WARNING, "Wrong data to delete passed to gdbm_delete()");
+    return(-1);
+  }
+
   if(myGlobals.gdbmMutex.isInitialized == 1) /* Mutex not yet initialized ? */
     accessMutex(&myGlobals.gdbmMutex, "ntop_gdbm_delete");
 
