@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -29,6 +29,8 @@
 #include <execinfo.h>
 #endif
 
+char static_ntop;
+
 /*
  * Hello World! This is ntop speaking...
  */
@@ -36,7 +38,7 @@ void welcome (FILE * fp) {
   fprintf (fp, "Welcome to %s v.%s\n[Configured on %s, built on %s]\n",
 	   myGlobals.program_name, version, configureDate, buildDate);
   
-  fprintf (fp, "Copyright 1998-2005 by %s.\n", author);
+  fprintf (fp, "Copyright 1998-2007 by %s.\n", author);
   fprintf (fp, "Get the freshest ntop from http://www.ntop.org/\n");
 }
 
@@ -88,7 +90,7 @@ void usage(FILE * fp) {
 #else
   fprintf(fp, "    [-i <number>    | --interface <number|name>]          %sInterface index number (or name) to monitor\n", newLine);
 #endif
-  fprintf(fp, "    [-j             | --create-other-packets]	         %sCreate file ntop-other-pkts.XXX.pcap file\n", newLine);
+  fprintf(fp, "    [-j             | --create-other-packets]             %sCreate file ntop-other-pkts.XXX.pcap file\n", newLine);
   fprintf(fp, "    [-l <path>      | --pcap-log <path>]                  %sDump packets captured to a file (debug only!)\n", newLine);
   fprintf(fp, "    [-m <addresses> | --local-subnets <addresses>]        %sLocal subnetwork(s) (see man page)\n", newLine);
   fprintf(fp, "    [-n             | --numeric-ip-addresses]             %sNumeric IP addresses - no DNS resolution\n", newLine);
@@ -127,6 +129,7 @@ void usage(FILE * fp) {
   fprintf(fp, "    [-O <path>      | --pcap-file-path <path>]            %sPath for log files in pcap format\n", newLine);
   fprintf(fp, "    [-U <URL>       | --mapper <URL>]                     %sURL (mapper.pl) for displaying host location\n", 
 	  newLine);
+
   fprintf(fp, "    [-V             | --version]                          %sOutput version information and exit\n", newLine);
   fprintf(fp, "    [-X <max num TCP sessions> ]                          %sMax num. TCP sessions ntop can handle (default %u)\n", 
 	  newLine, myGlobals.runningPref.maxNumSessions);
@@ -195,7 +198,9 @@ static void verifyOptions (void) {
 
 #ifndef WIN32    
     if ((myGlobals.runningPref.disablePromiscuousMode != 1) &&
-        getuid() /* We're not root */) {
+        getuid() /* We're not root */
+	&& myGlobals.runningPref.devices
+	&& strcmp(myGlobals.runningPref.devices, "none")) {
         char *theRootPw, *correct, *encrypted;
         struct passwd *pw = getpwuid(0);
 
@@ -314,10 +319,21 @@ int main(int argc, char *argv[]) {
   struct stat fileStat;
   int effective_argc;
   char **effective_argv;
-  char main_buf[LEN_GENERAL_WORK_BUFFER],
-       lib[LEN_GENERAL_WORK_BUFFER],
+  char main_buf[LEN_GENERAL_WORK_BUFFER];
+#ifndef WIN32
+  char lib[LEN_GENERAL_WORK_BUFFER],
        env[LEN_GENERAL_WORK_BUFFER],
        buf[LEN_GENERAL_WORK_BUFFER];
+#endif
+
+  /*
+  unsigned long driveSerial;
+  if(get_serial("C:\\", &driveSerial))
+	  printf("C:\\ Serial Number: %u\n", driveSerial);
+  else
+printf("Unable to read serial number\n");
+		exit(0);
+		*/
 
 /* Don't move this below nor above */
 #if defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 1)
@@ -329,6 +345,11 @@ int main(int argc, char *argv[]) {
   printf("MEMORY_DEBUG 4 - mcheck() - remember you need to run at the console, without -d | --daemon\n");
 #endif
 
+  if(strstr(argv[0], "ntops"))
+    static_ntop = 1;
+  else
+    static_ntop = 0;
+
   /* printf("Wait please: ntop is coming up...\n"); */
 
   /* VERY FIRST THING is to clear myGlobals, so myGlobals.ntopRunState can be used */
@@ -467,7 +488,6 @@ int main(int argc, char *argv[]) {
       }
 
       fclose(fd);
-
     }
   }
   free(readBuffer);
@@ -508,7 +528,7 @@ int main(int argc, char *argv[]) {
 
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "ntop v.%s", version);
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Configured on %s, built on %s.", configureDate, buildDate);
-  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Copyright 1998-2005 by %s", author);
+  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Copyright 1998-2007 by %s", author);
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Get the freshest ntop from http://www.ntop.org/");
  
 #ifndef WIN32
@@ -559,10 +579,10 @@ int main(int argc, char *argv[]) {
       char tmpBuf[64];
 
       safe_snprintf(__FILE__, __LINE__, tmpBuf, sizeof(tmpBuf), "%s%s", 
-                  (i>0) ? "," : "",
-                  (myGlobals.device[i].humanFriendlyName != NULL) ?
-                      myGlobals.device[i].humanFriendlyName :
-                      myGlobals.device[i].name);
+		    (i>0) ? "," : "",
+		    (myGlobals.device[i].humanFriendlyName != NULL) ?
+		    myGlobals.device[i].humanFriendlyName :
+		    myGlobals.device[i].name);
       strncat(ifStr, tmpBuf, sizeof(ifStr)-strlen(ifStr)-1)[sizeof(ifStr)-1] = '\0';
     }
   }
@@ -573,40 +593,28 @@ int main(int argc, char *argv[]) {
   } else
     traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Listening on [%s]", ifStr);
 
-  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Loading Plugins");
-  loadPlugins();
-  traceEvent(CONST_TRACE_NOISY, "Starting Plugins");
-  startPlugins();
-  traceEvent(CONST_TRACE_NOISY, "Plugins started... continuing with initialization");
+  if(!static_ntop) {
+    traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Loading Plugins");
+    loadPlugins();
+    traceEvent(CONST_TRACE_NOISY, "Starting Plugins");
+    startPlugins();
+    traceEvent(CONST_TRACE_NOISY, "Plugins started... continuing with initialization");
+  }
 
   /* ******************************* */
   
-  checkUserIdentity(userSpecified);
-
 #ifndef WIN32
   saveNtopPid();
 #endif
 
+  checkUserIdentity(userSpecified);
+
   /* ******************************* */
 
   addDefaultAdminUser();
 
   initReports();
 
-  /* If we can, set the base memory HERE */
-#ifdef MAKE_WITH_MALLINFO
-  {
-    struct mallinfo memStats;
-
-    memStats = mallinfo();
-    myGlobals.baseMemoryUsage = memStats.arena + memStats.hblkhd;
-
-    traceEvent(CONST_TRACE_NOISY, "MEMORY: Base memory load is %.2fMB (%d+%d)",
-	       xvertDOT00MB(myGlobals.baseMemoryUsage),
-	       memStats.arena,
-	       memStats.hblkhd);
-  }
-#endif
   traceEvent(CONST_TRACE_NOISY, "MEMORY: Base interface structure (no hashes loaded) is %.2fMB each",
 	     xvertDOT00MB(sizeof(NtopInterface)));
   traceEvent(CONST_TRACE_NOISY, "MEMORY:     or %.2fMB for %d interfaces",
