@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -30,15 +30,9 @@ extern char** backtrace_symbols(void* thearray[], int thearraylen);
 
 static int *servicesMapper = NULL; /* temporary value */
 
-#if defined(FREEBSD) && defined(__FreeBSD_cc_version) && (__FreeBSD_cc_version < 500000) && defined(HAVE_PCAP_SETNONBLOCK)
-#warning special code for freeBSD 4.x
-static unsigned long long setNonBlockingSleepCount;
-#endif
-
 /* *************************** */
 
 static void printMutexInfo(PthreadMutex *mutexId, char *mutexName) {
-
   traceEvent(CONST_TRACE_INFO, "%s is %s (last lock %s:%d) [max lock time %s:%d (%.6f sec)]",
 	     mutexName,
 	     mutexId->isLocked ? "*locked*" : "unlocked",
@@ -50,16 +44,24 @@ static void printMutexInfo(PthreadMutex *mutexId, char *mutexName) {
 
 #ifndef WIN32
 void handleSigHup(int signalId _UNUSED_) {
-  traceEvent(CONST_TRACE_INFO, "========================================");
+  int i;
+
   printMutexInfo(&myGlobals.gdbmMutex, "myGlobals.gdbmMutex");
-  printMutexInfo(&myGlobals.packetProcessMutex, "myGlobals.packetProcessMutex");
-  printMutexInfo(&myGlobals.packetQueueMutex, "myGlobals.packetQueueMutex");
+
+  for(i=0; i<myGlobals.numDevices; i++) {
+    char tmp[64];
+
+    safe_snprintf(__FILE__, __LINE__, tmp, sizeof(tmp), "myGlobals.packetProcessMutex[%s]",
+		  myGlobals.device[i].name);
+    printMutexInfo(&myGlobals.device[i].packetProcessMutex, tmp);
+    safe_snprintf(__FILE__, __LINE__, tmp, sizeof(tmp), "myGlobals.packetQueueMutex[%s]",
+		  myGlobals.device[i].name);
+    printMutexInfo(&myGlobals.device[i].packetQueueMutex, tmp);
+  }
 
   if(myGlobals.runningPref.numericFlag == 0)
     printMutexInfo(&myGlobals.addressResolutionMutex, "myGlobals.addressResolutionMutex");
 
-  traceEvent(CONST_TRACE_INFO, "========================================");
-
   (void)signal(SIGHUP,  handleSigHup);
 }
 
@@ -69,15 +71,11 @@ void handleSigHup(int signalId _UNUSED_) {
 
 void* pcapDispatch(void *_i) {
   int rc;
-  int i = (int)_i;
+  int i = (int)((long)_i);
   struct pcap_stat pcapStats;
 
-#if defined(FREEBSD) && defined(__FreeBSD_cc_version) && (__FreeBSD_cc_version < 500000) && defined(HAVE_PCAP_SETNONBLOCK)
-  setNonBlockingSleepCount = 0;
-#endif
-
-  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%d,%s): pcapDispatch thread starting [p%d]",
-	     pthread_self(), i+1, myGlobals.device[i].humanFriendlyName, getpid());
+  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%s): pcapDispatch thread starting [p%d]",
+	     pthread_self(), myGlobals.device[i].humanFriendlyName, getpid());
 
   /* Reset stats before to start (needed by modern libpcap versions) */
   if(myGlobals.runningPref.rFileName == NULL) {
@@ -85,17 +83,13 @@ void* pcapDispatch(void *_i) {
     myGlobals.device[i].initialPcapDroppedPkts.value = pcapStats.ps_drop;
   }
 
-  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%d,%s): pcapDispatch thread running [p%d]",
-	     pthread_self(), i+1, myGlobals.device[i].humanFriendlyName, getpid());
+  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%s): pcapDispatch thread running [p%d]",
+	     pthread_self(), myGlobals.device[i].humanFriendlyName, getpid());
 
   /* Skip ntopSleepUntilStateRUN(), just start processing packets as soon as this starts */
 
   for(;myGlobals.ntopRunState <= FLAG_NTOPSTATE_RUN;) {
-#if defined(FREEBSD) && defined(__FreeBSD_cc_version) && (__FreeBSD_cc_version < 500000) && defined(HAVE_PCAP_SETNONBLOCK)
     rc = pcap_dispatch(myGlobals.device[i].pcapPtr, -1, queuePacket, (u_char*)_i);
-#else
-    rc = pcap_loop(myGlobals.device[i].pcapPtr, -1, queuePacket, (u_char*)_i);
-#endif
 
     if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN) break;
 
@@ -112,29 +106,13 @@ void* pcapDispatch(void *_i) {
 		   myGlobals.device[i].humanFriendlyName, rc);
 	break; /* No more packets to read */
       }
-#if defined(FREEBSD) && defined(__FreeBSD_cc_version) && (__FreeBSD_cc_version < 500000) && defined(HAVE_PCAP_SETNONBLOCK)
-      if(myGlobals.runningPref.setNonBlocking == TRUE) {
-        /* select returned no data - either a signal or setNonBlock */
-        struct timespec sleepAmount;
-        sleepAmount.tv_sec = 0; sleepAmount.tv_nsec = CONST_PCAPNONBLOCKING_SLEEP_TIME;
-        rc = nanosleep(&sleepAmount, NULL);
-        ++setNonBlockingSleepCount;
-        if(((setNonBlockingSleepCount < 1000) && (setNonBlockingSleepCount % 250 == 0))
-	   || (setNonBlockingSleepCount % 100000 == 0) ){
-          traceEvent(CONST_TRACE_NOISY, "NONBLOCK: Sleep count %s %llu",
-                     myGlobals.device[i].humanFriendlyName,
-                     setNonBlockingSleepCount);
-        }
-      }
-      continue; /* Try again */
-#endif
     }
   }
 
   myGlobals.device[i].pcapDispatchThreadId = 0;
 
-  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%d,%s): pcapDispatch thread terminated [p%d]",
-	     pthread_self(), i+1, myGlobals.device[i].humanFriendlyName, getpid());
+  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%s): pcapDispatch thread terminated [p%d]",
+	     pthread_self(), myGlobals.device[i].humanFriendlyName, getpid());
 
   return(NULL);
 }
@@ -142,7 +120,6 @@ void* pcapDispatch(void *_i) {
 /* **************************************** */
 
 #ifndef WIN32
-#ifdef HANDLE_DIED_CHILD
 RETSIGTYPE handleDiedChild(int sig _UNUSED_) {
   int status;
   pid_t pidId;
@@ -158,12 +135,9 @@ RETSIGTYPE handleDiedChild(int sig _UNUSED_) {
 #endif
   }
 
-#ifdef HANDLE_DIED_CHILD
   signal(SIGCHLD, handleDiedChild);
-#endif
 }
 #endif
-#endif
 
 /* **************************************** */
 
@@ -217,7 +191,7 @@ void detachFromTerminalUnderUnix(int doChdir) {
     myGlobals.runningPref.useSyslog = DEFAULT_SYSLOG_FACILITY;
 #endif /* MAKE_WITH_SYSLOG */
 
-  if(doChdir) chdir("/");
+  if(doChdir) (void)chdir("/");
   setsid();  /* detach from the terminal */
 
   fclose(stdin);
@@ -233,7 +207,11 @@ void detachFromTerminalUnderUnix(int doChdir) {
    * Use line buffered stdout
    */
   /* setlinebuf (stdout); */
+#if SETVBUF_REVERSED
+  setvbuf(stdout, _IOLBF, (char *)NULL, 0);
+#else
   setvbuf(stdout, (char *)NULL, _IOLBF, 0);
+#endif
 
 #endif /* WIN32 */
 
@@ -613,7 +591,8 @@ static void purgeIpPorts(int theDevice) {
   traceEvent(CONST_TRACE_INFO, "Calling purgeIpPorts(%d)", theDevice);
 #endif
 
-  if(myGlobals.device[myGlobals.actualReportDeviceId].numHosts == 0) return;
+  if(myGlobals.device[theDevice].numHosts == 0)   return;
+  if(myGlobals.device[theDevice].ipPorts == NULL) return;
 
   accessMutex(&myGlobals.purgePortsMutex, "purgeIpPorts");
 
@@ -640,7 +619,7 @@ void* scanIdleLoop(void* notUsed _UNUSED_) {
              pthread_self(), getpid());
 
   ntopSleepUntilStateRUN();
- 
+
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SIH: Idle host scan thread running [p%d]",
              pthread_self(), getpid());
 
@@ -655,7 +634,9 @@ void* scanIdleLoop(void* notUsed _UNUSED_) {
 
     for(i=0; i<myGlobals.numDevices; i++)
       if(!myGlobals.device[i].virtualDevice) {
-        if(!myGlobals.runningPref.stickyHosts) purged += purgeIdleHosts(i);
+        if((!myGlobals.runningPref.stickyHosts) 
+	   && (!myGlobals.runningPref.rFileName))
+	  purged += purgeIdleHosts(i);
 #if !defined(__FreeBSD__)
 	purgeIpPorts(i);
 #endif
@@ -698,7 +679,7 @@ void* scanFingerprintLoop(void* notUsed _UNUSED_) {
 
     ntopSleepWhileSameState(CONST_FINGERPRINT_LOOP_INTERVAL);
     if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN) break;
-    
+
     if(myGlobals.runningPref.rFileName == NULL)
       myGlobals.actTime = time(NULL);
     countCycle++;
@@ -743,29 +724,28 @@ static void cleanupThreadIs(char *buf, int sizeofbuf) {
 
   buf[0]='\0';
 
-  if(pthread_self() == myGlobals.mainThreadId)
-    strncpy(buf, "MAIN", sizeofbuf);
-  else if(pthread_self() == myGlobals.dequeueThreadId)
-    strncpy(buf, "NPA", sizeofbuf);
-  else if(pthread_self() == myGlobals.scanFingerprintsThreadId)
-    strncpy(buf, "SFP", sizeofbuf);
-  else if(pthread_self() == myGlobals.scanIdleThreadId)
-    strncpy(buf, "SIH", sizeofbuf);
-  else if(pthread_self() == myGlobals.handleWebConnectionsThreadId)
-    strncpy(buf, "WEB", sizeofbuf);
+  if(buf[0] != '\0') {
+    if(pthread_self() == myGlobals.mainThreadId)
+      strncpy(buf, "MAIN", sizeofbuf);
+    else if(pthread_self() == myGlobals.scanFingerprintsThreadId)
+      strncpy(buf, "SFP", sizeofbuf);
+    else if(pthread_self() == myGlobals.scanIdleThreadId)
+      strncpy(buf, "SIH", sizeofbuf);
+    else if(pthread_self() == myGlobals.handleWebConnectionsThreadId)
+      strncpy(buf, "WEB", sizeofbuf);
 
 #if defined(HAVE_OPENSSL) && defined(MAKE_WITH_SSLWATCHDOG)
-  else if(pthread_self() == myGlobals.sslwatchdogChildThreadId)
-    strncpy(buf, "SSL", sizeofbuf);
+    else if(pthread_self() == myGlobals.sslwatchdogChildThreadId)
+      strncpy(buf, "SSL", sizeofbuf);
 #endif
-
-  else
-    for(i=0; i<myGlobals.numDequeueThreads; i++) {
-      if(pthread_self() == myGlobals.dequeueAddressThreadId[i]) {
-        safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "DNSAR%d", i+1);
-        break;
+    else
+      for(i=0; i<myGlobals.numDequeueAddressThreads; i++) {
+	if(pthread_self() == myGlobals.dequeueAddressThreadId[i]) {
+	  safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "DNSAR%d", i+1);
+	  break;
+	}
       }
-    }
+  }
 
   if(buf[0] == '\0') {
     for(i=0; i<myGlobals.numDevices; i++) {
@@ -776,11 +756,22 @@ static void cleanupThreadIs(char *buf, int sizeofbuf) {
     }
   }
 
+  if(buf[0] == '\0') {
+    for(i=0; i<myGlobals.numDevices; i++) {
+      if(pthread_self() == myGlobals.device[i].dequeuePacketThreadId) {
+        safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "NPS(%s)",
+		      myGlobals.device[i].humanFriendlyName);
+        break;
+      }
+    }
+  }
+
   if(buf[0] == '\0') {
     for(i=0; i<myGlobals.numDevices; i++) {
       if((myGlobals.device[i].netflowGlobals != NULL) &&
          (pthread_self() == myGlobals.device[i].netflowGlobals->netFlowThread)) {
-        safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "NF%d", i);
+        safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "NF(%s)", 
+		      myGlobals.device[i].humanFriendlyName);
         break;
       }
     }
@@ -790,7 +781,8 @@ static void cleanupThreadIs(char *buf, int sizeofbuf) {
     for(i=0; i<myGlobals.numDevices; i++) {
       if((myGlobals.device[i].sflowGlobals != NULL) &&
          (pthread_self() == myGlobals.device[i].sflowGlobals->sflowThread)) {
-        safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "SF%d", i);
+        safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "SF(%s)", 
+		      myGlobals.device[i].humanFriendlyName);
         break;
       }
     }
@@ -804,22 +796,20 @@ static void cleanupThreadIs(char *buf, int sizeofbuf) {
 /* *************************** */
 
 void runningThreads(char *buf, int sizeofbuf, int do_join) {
-  char buf2[LEN_SMALL_WORK_BUFFER];
+  char buf2[LEN_MEDIUM_WORK_BUFFER];
   int i;
 
   if(!do_join) {
     memset(&buf2, 0, sizeof(buf2));
-    
+
 #if defined(HAVE_OPENSSL) && defined(MAKE_WITH_SSLWATCHDOG)
-    safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "%s%s%s%s%s", 
-		  myGlobals.dequeueThreadId != 0 ? " NPA" : "",
+    safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "%s%s%s%s",
 		  myGlobals.scanFingerprintsThreadId != 0 ? " SFP" : "",
 		  myGlobals.scanIdleThreadId != 0 ? " SIH" : "",
 		  myGlobals.handleWebConnectionsThreadId != 0 ? " WEB" : "",
 		  myGlobals.sslwatchdogChildThreadId != 0 ? " SSL" : "");
 #else
-    safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "%s%s%s%s", 
-		  myGlobals.dequeueThreadId != 0 ? " NPA" : "",
+    safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "%s%s%s",
 		  myGlobals.scanFingerprintsThreadId != 0 ? " SFP" : "",
 		  myGlobals.scanIdleThreadId != 0 ? " SIH" : "",
 		  myGlobals.handleWebConnectionsThreadId != 0 ? " WEB" : "");
@@ -827,16 +817,16 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 
   }
 
-  for(i=0; i<myGlobals.numDequeueThreads; i++) {
-    if((myGlobals.dequeueAddressThreadId[i] != 0) 
+  for(i=0; i<myGlobals.numDequeueAddressThreads; i++) {
+    if((myGlobals.dequeueAddressThreadId[i] != 0)
        && (myGlobals.dequeueAddressThreadId[i] != (pthread_t)-1)) {
       if(!do_join) {
 	safe_snprintf(__FILE__, __LINE__, buf2, sizeof(buf2), " DNSAR%d", i+1);
 	safe_strncat(buf, sizeofbuf, buf2);
       } else {
 	/* Wake up thread */
-	signalCondvar(&myGlobals.queueCondvar);
 	signalCondvar(&myGlobals.queueAddressCondvar);
+	traceEvent(CONST_TRACE_INFO, "Joining thread DNSAR%d", i+1);
 	if(joinThread(&myGlobals.dequeueAddressThreadId[i]) != 0)
 	  traceEvent(CONST_TRACE_INFO, "joinThread() returned %s", strerror(errno));
       }
@@ -850,7 +840,8 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
          (!myGlobals.device[i].dummyDevice) &&
          (myGlobals.device[i].pcapPtr != NULL)) {
 	if(!do_join) {
-	  safe_snprintf(__FILE__, __LINE__, buf2, sizeof(buf2), " NPS%d", i+1);
+	  safe_snprintf(__FILE__, __LINE__, buf2, sizeof(buf2), " NPS(%s)",
+			myGlobals.device[i].humanFriendlyName);
 	  safe_strncat(buf, sizeofbuf, buf2);
 	} else {
 	  struct pcap_stat pcapStats;
@@ -858,16 +849,21 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 	  if(pcap_stats(myGlobals.device[i].pcapPtr, &pcapStats) >= 0) {
 	    traceEvent(CONST_TRACE_INFO, "STATS: %s packets received by filter on %s",
 		       formatPkts((Counter)pcapStats.ps_recv, buf2, sizeof(buf2)), myGlobals.device[i].name);
-	    
+
 	    traceEvent(CONST_TRACE_INFO, "STATS: %s packets dropped (according to libpcap)",
 		       formatPkts((Counter)pcapStats.ps_drop, buf2, sizeof(buf2)));
 	  }
+
 	  traceEvent(CONST_TRACE_INFO, "STATS: %s packets dropped (by ntop)",
-		     formatPkts(myGlobals.device[i].droppedPkts.value, buf2, sizeof(buf2)));	
-	  
+		     formatPkts(myGlobals.device[i].droppedPkts.value, buf2, sizeof(buf2)));
+
+	  /* signalCondvar(&myGlobals.device[i].queueCondvar); */
 	  pcap_close(myGlobals.device[i].pcapPtr);
+
+	  traceEvent(CONST_TRACE_INFO, "Joining thread  NPS(%s)",
+		     myGlobals.device[i].humanFriendlyName);
 	  if(joinThread(&myGlobals.device[i].pcapDispatchThreadId) != 0)
-	    traceEvent(CONST_TRACE_INFO, "joinThread() returned %s", strerror(errno));
+	    traceEvent(CONST_TRACE_INFO, "joinThread() returned: %s", strerror(errno));
 	}
       }
     }
@@ -879,6 +875,10 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 	  safe_snprintf(__FILE__, __LINE__, buf2, sizeof(buf2), " NF%d", i);
 	  safe_strncat(buf, sizeofbuf, buf2);
 	} else {
+	  traceEvent(CONST_TRACE_INFO, "Joining thread NF%d [%u]", i,
+		     (unsigned int)myGlobals.device[i].netflowGlobals->netFlowThread);
+	  close(myGlobals.device[i].netflowGlobals->netFlowInSocket);
+
 	  if(joinThread(&myGlobals.device[i].netflowGlobals->netFlowThread) != 0)
 	    traceEvent(CONST_TRACE_INFO, "joinThread() returned %s", strerror(errno));
 	}
@@ -892,11 +892,34 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 	  safe_snprintf(__FILE__, __LINE__, buf2, sizeof(buf2), " SF%d", i);
 	  safe_strncat(buf, sizeofbuf, buf2);
 	} else {
+	  traceEvent(CONST_TRACE_INFO, "Joining thread SF%d", i);
 	  if(joinThread(&myGlobals.device[i].sflowGlobals->sflowThread) != 0)
 	    traceEvent(CONST_TRACE_INFO, "joinThread() returned %s", strerror(errno));
 	}
       }
     }
+
+    for(i=0; i<myGlobals.numDevices; i++) {
+      if(myGlobals.device[i].dequeuePacketThreadId != 0) {
+	if(!do_join) {
+	  safe_snprintf(__FILE__, __LINE__, buf2, sizeof(buf2), " NPA(%s)",
+			myGlobals.device[i].humanFriendlyName);
+	  safe_strncat(buf, sizeofbuf, buf2);
+	} else {
+	  traceEvent(CONST_TRACE_INFO, "Signaling thread NPA(%s)",
+		     myGlobals.device[i].humanFriendlyName);	 
+	  signalCondvar(&myGlobals.device[i].queueCondvar);
+	  /*
+	  traceEvent(CONST_TRACE_INFO, "Joining thread NPA(%s)",
+		     myGlobals.device[i].humanFriendlyName);	  
+	  if(joinThread(&myGlobals.device[i].dequeuePacketThreadId) != 0)
+	    traceEvent(CONST_TRACE_INFO, "joinThread() returned %s", strerror(errno));
+	  else
+	    traceEvent(CONST_TRACE_INFO, "Join completed succesfully");
+	  */
+	}
+      }
+    }
   }
 }
 
@@ -904,14 +927,26 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 
 /* Report statistics and write out the raw packet file */
 RETSIGTYPE cleanup(int signo) {
-  struct pcap_stat pcapStats;
   int i, j;
   char buf[128];
-  
+  static int cleanup_called = 0;
+
   if(myGlobals.ntopRunState <= FLAG_NTOPSTATE_SHUTDOWN) {
+    static u_char caught_signal = 0;
     traceEvent(CONST_TRACE_INFO, "CLEANUP[t%lu]: ntop caught signal %d", pthread_self(), signo);
+
+    if(caught_signal){
+      traceEvent(CONST_TRACE_INFO, "ntop is now quitting...");
+      exit(0);
+    } else
+      caught_signal = 1;
   }
 
+#ifndef WIN32
+  signal(SIGALRM, cleanup);
+  alarm(10);
+#endif
+
 #ifdef HAVE_BACKTRACE
   if(signo == SIGSEGV) {
     void *array[20];
@@ -952,6 +987,11 @@ RETSIGTYPE cleanup(int signo) {
 
   setRunState(FLAG_NTOPSTATE_SHUTDOWN);
 
+  if(cleanup_called) {
+    exit(0);
+  }
+
+  cleanup_called = 1;
 
   cleanupThreadIs(buf, sizeof(buf));
   traceEvent(CONST_TRACE_INFO, "CLEANUP[t%lu] catching thread is %s", pthread_self(), buf);
@@ -986,7 +1026,7 @@ RETSIGTYPE cleanup(int signo) {
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "CLEANUP: Locked purge mutex, continuing shutdown");
 
   runningThreads(buf, sizeof(buf), 0);
-  traceEvent(CONST_TRACE_INFO, "CLEANUP: Continues%s%s%s", 
+  traceEvent(CONST_TRACE_INFO, "CLEANUP: Continues%s%s%s",
              buf[0] == '\0' ? "" : " (still running",
              buf,
              buf[0] == '\0' ? "" : ")");
@@ -1017,24 +1057,30 @@ RETSIGTYPE cleanup(int signo) {
   endservent();
 #endif
 
-  tryLockMutex(&myGlobals.packetProcessMutex, "cleanup");
-  deleteMutex(&myGlobals.packetProcessMutex);
-  tryLockMutex(&myGlobals.packetQueueMutex, "cleanup");
-  deleteMutex(&myGlobals.packetQueueMutex);
+  for(i=0; i<myGlobals.numDevices; i++) {
+    tryLockMutex(&myGlobals.device[i].asMutex, "cleanup");
+    deleteMutex(&myGlobals.device[i].asMutex);
+    tryLockMutex(&myGlobals.device[i].packetProcessMutex, "cleanup");
+    deleteMutex(&myGlobals.device[i].packetProcessMutex);
+    tryLockMutex(&myGlobals.device[i].packetQueueMutex, "cleanup");
+    deleteMutex(&myGlobals.device[i].packetQueueMutex);
+  }
+
   if(myGlobals.runningPref.numericFlag == 0) {
     tryLockMutex(&myGlobals.addressResolutionMutex, "cleanup");
     deleteMutex(&myGlobals.addressResolutionMutex);
   }
-  
+
   for(i=0; i<CONST_HASH_INITIAL_SIZE; i++) {
     tryLockMutex(&myGlobals.hostsHashMutex[i], "cleanup");
     deleteMutex(&myGlobals.hostsHashMutex[i]);
   }
 
-  deleteCondvar(&myGlobals.queueCondvar);
   deleteCondvar(&myGlobals.queueAddressCondvar);
+  deleteMutex(&myGlobals.queueAddressMutex);
 
   termGdbm();
+  termDB();
 
   tryLockMutex(&myGlobals.gdbmMutex, "cleanup");
   deleteMutex(&myGlobals.gdbmMutex);
@@ -1042,8 +1088,8 @@ RETSIGTYPE cleanup(int signo) {
   deleteMutex(&myGlobals.purgeMutex);
 
   for(i=0; i<myGlobals.numDevices; i++) {
-
-    traceEvent(CONST_TRACE_INFO, "CLEANUP: Freeing device %s (idx=%d)", myGlobals.device[i].name, i);
+    traceEvent(CONST_TRACE_INFO, "CLEANUP: Freeing device %s",
+	       myGlobals.device[i].humanFriendlyName);
 
     if(myGlobals.device[i].ipTrafficMatrix != NULL) {
       /* Courtesy of Wies-Software <wies@wiessoft.de> */
@@ -1054,9 +1100,10 @@ RETSIGTYPE cleanup(int signo) {
       free(myGlobals.device[i].ipTrafficMatrix);
     }
 
+    /* FIX
     if(myGlobals.device[i].ipTrafficMatrixHosts != NULL)
       free(myGlobals.device[i].ipTrafficMatrixHosts);
-
+    */
     if(myGlobals.device[i].ipProtoStats != NULL)
       free(myGlobals.device[i].ipProtoStats);
 
@@ -1074,6 +1121,11 @@ RETSIGTYPE cleanup(int signo) {
 	  free(myGlobals.device[i].ipPorts[port]);
     }
 
+    if(myGlobals.device[i].ipPorts)    
+      free(myGlobals.device[i].ipPorts);
+
+    if(myGlobals.device[i].packetQueue) 
+      free(myGlobals.device[i].packetQueue);
 
     accessMutex(&myGlobals.tcpSessionsMutex, "cleanup");
     if(myGlobals.device[i].tcpSession != NULL)
@@ -1081,6 +1133,7 @@ RETSIGTYPE cleanup(int signo) {
     releaseMutex(&myGlobals.tcpSessionsMutex);
 
     free(myGlobals.device[i].humanFriendlyName);
+    free(myGlobals.device[i].uniqueIfName);
     free(myGlobals.device[i].name);
 
     if(myGlobals.device[i].pcapDumper != NULL)
@@ -1101,8 +1154,13 @@ RETSIGTYPE cleanup(int signo) {
         free(tmp);
       }
     }
-#endif
-
+#endif  
+    
+    while(myGlobals.device[i].asStats) {
+      AsStats *next = myGlobals.device[i].asStats->next;
+      free(myGlobals.device[i].asStats);
+      myGlobals.device[i].asStats = next;
+    }
   }
 
   if(myGlobals.device)
@@ -1110,7 +1168,7 @@ RETSIGTYPE cleanup(int signo) {
 
   if(myGlobals.broadcastEntry != NULL) free(myGlobals.broadcastEntry);
   if(myGlobals.otherHostEntry != NULL) {
-    if(myGlobals.otherHostEntry->portsUsage != NULL) 
+    if(myGlobals.otherHostEntry->portsUsage != NULL)
       freePortsUsage(myGlobals.otherHostEntry);
     myGlobals.otherHostEntry->portsUsage = NULL;
     free(myGlobals.otherHostEntry);
@@ -1162,10 +1220,7 @@ RETSIGTYPE cleanup(int signo) {
   if(myGlobals.spoolPath) free(myGlobals.spoolPath);
   if(myGlobals.rrdPath != NULL)
     free(myGlobals.rrdPath);
-  
-  if(myGlobals.gdVersionGuessValue != NULL)
-    free(myGlobals.gdVersionGuessValue);
-  
+
 #if defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 1)
   traceEvent(CONST_TRACE_INFO, "===================================");
   muntrace();
@@ -1187,7 +1242,7 @@ RETSIGTYPE cleanup(int signo) {
 
   memset(&buf, 0, sizeof(buf));
   runningThreads(buf, sizeof(buf), 0);
-  if(buf[0] != '\0') 
+  if(buf[0] != '\0')
     traceEvent(CONST_TRACE_INFO, "CLEANUP[t%lu]: Still running threads%s", pthread_self(), buf);
 
   traceEvent(CONST_TRACE_INFO, "===================================");
