@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *                      
  *  			    http://www.ntop.org/
  *  					
@@ -59,6 +59,9 @@ typedef int NDIS_STATUS, *PNDIS_STATUS;
 extern int getopt(int num, char *const *argv, const char *opts);
 #define getopt getopt____
 
+#define unlink(a) _unlink(a)
+#define close(a) _close(a)
+
 #if defined(__GNUC__)
 /* on mingw, struct timezone isn't defined so s/struct timezone/void/ - Scott Renfro <scott@renfro.org> */
 extern int gettimeofday(struct timeval*, void*);
@@ -77,6 +80,7 @@ extern ULONG GetHostIPAddr();
 #define MAKE_WITH_ZLIB
 #define HAVE_DIRENT_H
 #define HAVE_PCAP_FREEALLDEVS
+#define HAVE_MYSQL_H
 
 #if !defined(__GNUC__)
 #define INET6
@@ -421,11 +425,14 @@ extern void sniffSinglePacket(void(*pbuf_process)(u_char *unused,
 		  const struct pcap_pkthdr *h, 
 		  const u_char *p));
 
+extern char* strptime(const char *buf, const char *fmt, struct tm *tm);
+
 /* ********************************************* */
 
 // Max number of packets handled by the public version
 #define MAX_NUM_PACKETS   1000
 
 extern short isWinNT();
+extern void get_serial(unsigned long *driveSerial);
 
 /* #define WIN32_DEMO */
