@@ -44,7 +44,7 @@ static PluginInfo LsPluginInfo[] = {
     "each specific host.<br>A note card database is available for recording "
     "additional information.",
     "2.3a", /* version */
-    "<a href=\"mailto:&#109;&#097;&#114;&#097;&#110;&#103;&#111;&#110;&#105;&#064;&#117;&#110;&#105;&#109;&#099;&#046;&#105;&#116;\" alt=\"Mail to A. Marangoni\">A.Marangoni</a>", 
+    "<a class=mailto href=\"mailto:&#109;&#097;&#114;&#097;&#110;&#103;&#111;&#110;&#105;&#064;&#117;&#110;&#105;&#109;&#099;&#046;&#105;&#116;\" alt=\"Mail to A. Marangoni\">A.Marangoni</a>", 
     "LastSeen", /* http://<host>:<port>/plugins/Ls */
     0, /* Active by default */
     ViewOnly,
@@ -107,7 +107,7 @@ static void handleLsPacket(u_char *_deviceId,
   traceEvent(CONST_TRACE_INFO, "->%s [%x]", intoa(ip.ip_dst), ip.ip_dst.s_addr);
 #endif
 
-  rc = in_isPseudoLocalAddress(&ip.ip_src, deviceId);
+  rc = in_isPseudoLocalAddress(&ip.ip_src, deviceId, NULL, NULL);
 
   if(rc == 0) 
     return;
@@ -121,7 +121,7 @@ static void handleLsPacket(u_char *_deviceId,
   HostI.LastUpdated = myGlobals.actTime;
 
   safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr), "%u", (unsigned) ip.ip_src.s_addr);
-  key_data.dptr = tmpStr; key_data.dsize = strlen(key_data.dptr)+1;
+  key_data.dptr = tmpStr; key_data.dsize = (int)strlen(key_data.dptr)+1;
   data_data.dptr = (char *)&HostI;
   data_data.dsize = sizeof(HostI)+1;
 
@@ -230,7 +230,7 @@ static void processHTMLrequest(char* url) {
     safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr), "N_%u", (unsigned)tablehost[entry].HostIpAddress.Ip4Address.s_addr);
 
     key_data.dptr = tmpStr;
-    key_data.dsize = strlen(key_data.dptr)+1;
+    key_data.dsize = (int)strlen(key_data.dptr)+1;
 		
     content = gdbm_fetch(LsDB,key_data);
     strncpy(HostN.note, no_note, sizeof(HostN.note));	
@@ -316,7 +316,7 @@ static void addNotes(char *addr, char *PostNotes) {
   safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr), "N_%s",addr);
 
   key_data.dptr = tmpStr;
-  key_data.dsize = strlen(key_data.dptr)+1;
+  key_data.dsize = (int)strlen(key_data.dptr)+1;
   data_data.dptr = (char *)&HostN;
   data_data.dsize = sizeof(HostN)+1;
 
@@ -340,7 +340,7 @@ static void NotesURL(char *addr, char *ip_addr) {
 
   safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr), "N_%s",addr);
   key_data.dptr = tmpStr;
-  key_data.dsize = strlen(key_data.dptr)+1;
+  key_data.dsize = (int)strlen(key_data.dptr)+1;
 
   content = gdbm_fetch(LsDB,key_data);
 
@@ -374,11 +374,11 @@ static void deletelastSeenURL( char *addr ) {
   safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr), "N_%s",addr);
 
   key_data.dptr = addr;
-  key_data.dsize = strlen(key_data.dptr)+1;
+  key_data.dsize = (int)strlen(key_data.dptr)+1;
 
   gdbm_delete(LsDB,key_data);  /* Record */
   key_data.dptr = tmpStr;
-  key_data.dsize = strlen(key_data.dptr)+1;
+  key_data.dsize = (int)strlen(key_data.dptr)+1;
   gdbm_delete(LsDB,key_data);  /* Notes */
 }
 
@@ -414,7 +414,7 @@ PluginInfo* lsPluginEntryFctn(void) {
   
     /* Fix courtesy of Ralf Amandi <Ralf.Amandi@accordata.net> */
     safe_snprintf(__FILE__, __LINE__, tmpBuf, sizeof(tmpBuf), "%s/LsWatch.db", myGlobals.dbPath);
-    LsDB = gdbm_open (tmpBuf, 0, GDBM_WRCREAT, 00664, NULL);
+    LsDB = gdbm_open (tmpBuf, 0, GDBM_WRCREAT, 00640, NULL);
 
     if(LsDB == NULL) {
       traceEvent(CONST_TRACE_ERROR, 
