@@ -38,7 +38,7 @@ static PluginInfo PDAPluginInfo[] = {
     "PDA",
     "This plugin produces a minimal ntop report, suitable for display on a pda",
     "2.2a",            /* version */
-    "<a href=\"mailto:&#119;&#097;&#108;&#116;&#101;&#114;&#098;&#114;&#111;&#099;&#107;&#064;&#110;&#101;&#116;&#115;&#099;&#097;&#112;&#101;&#046;&#110;&#101;&#116;\">W. Brock</A>", 
+    "<a class=mailto href=\"mailto:&#119;&#097;&#108;&#116;&#101;&#114;&#098;&#114;&#111;&#099;&#107;&#064;&#110;&#101;&#116;&#115;&#099;&#097;&#112;&#101;&#046;&#110;&#101;&#116;\">W. Brock</A>", 
     "PDAPlugin",      /* http://<host>:<port>/plugins/PDAPlugin */
     0,                /* Active by default */
     ViewOnly,
@@ -233,7 +233,7 @@ void printPdaSummaryData(void) {
 
   safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),"<tr><td>Sampling Time</td>"
 	      "<td>%s</td></tr>\n",
-	      formatSeconds(myGlobals.actTime-myGlobals.initialSniffTime, formatBuf, sizeof(formatBuf)));
+	      formatSeconds((long)(myGlobals.actTime-myGlobals.initialSniffTime), formatBuf, sizeof(formatBuf)));
   sendString(buf);
 
   /** **/
