@@ -42,7 +42,9 @@
 #define FLAG_RRD_ACTION_LIST                2
 #define FLAG_RRD_ACTION_GRAPH_SUMMARY       3
 #define FLAG_RRD_ACTION_NF_SUMMARY          4
-#define FLAG_RRD_ACTION_ARBITRARY           5
+#define FLAG_RRD_ACTION_NF_IF_SUMMARY       5
+#define FLAG_RRD_ACTION_IF_SUMMARY          6
+#define FLAG_RRD_ACTION_ARBITRARY           7
 
 #define CONST_RRD_PERMISSIONS_PRIVATE       0
 #define CONST_RRD_PERMISSIONS_GROUP         1
@@ -70,6 +72,8 @@
 #define DEFAULT_RRD_MONTHS                  36   /* months of day by day data (default) */
 #define DEFAULT_RRD_DUMP_DELAY              10   /* ms (default) */
 
+#define AS_RRD_DUMP_PKTS_THRESHOLD          10   /* packets */
+
 /* ****************************************************** */
 
 /*
@@ -181,7 +185,7 @@ static char *rrdNames[] = {
 /* RRD_DEBUG controls debug messages in rrdPlugin.c.  See the definition in globals-defines.h,
  * where you really should set it, so the util.c calls get enabled.
  */
-/* #define RRD_DEBUG */
+#define RRD_DEBUG 3
 
 /*
  * Optional: Set a default font for the rrd generated graphs
@@ -318,6 +322,21 @@ static const char *rrd_summary_host_sentRcvd_bytes[] = {
   NULL
 };
 
+static const char *rrd_summary_nf_if_octets[] = { 
+  "ifInOctets",
+  "ifOutOctets",
+  "ifSelfOctets",
+  NULL
+};
+
+static const char *rrd_summary_nf_if_pkts[] = { 
+  "ifInPkts",
+  "ifOutPkts",
+  "ifSelfPkts",
+ NULL
+};
+
+
 #ifdef MAX_RRD_PROCESS_BUFFER
 static float rrdprocessBuffer[MAX_RRD_PROCESS_BUFFER];
 static int rrdprocessBufferInit,
