@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -56,11 +56,11 @@ static char __see__ []    =
    * ('P' option) and so these are processed separately.
    */
 #ifdef WIN32
-static char*  short_options = "4:6:a:bce:f:ghi:jl:m:nop:qr:st:w:x:zAB:C:D:F:MN:O:P:Q:S:U:VX:W:";
+static char*  short_options = "46a:bce:f:ghi:jl:m:nop:qr:st:w:x:zAB:C:D:F:MN:O:P:Q:S:U:VX:W:";
 #elif defined(MAKE_WITH_SYSLOG)
-static char*  short_options = "4:6:a:bcde:f:ghi:jl:m:nop:qr:st:u:w:x:zAB:C:D:F:IKLMN:O:P:Q:S:U:VX:W:";
+static char*  short_options = "46a:bcde:f:ghi:jl:m:nop:qr:st:u:w:x:zAB:C:D:F:IKLMN:O:P:Q:S:U:VX:W:";
 #else
-static char*  short_options = "4:6:a:bcde:f:ghi:jl:m:nop:qr:st:u:w:x:zAB:C:D:F:IKMN:O:P:Q:S:U:VX:W:";
+static char*  short_options = "46a:bcde:f:ghi:jl:m:nop:qr:st:u:w:x:zAB:C:D:F:IKMN:O:P:Q:S:U:VX:W:";
 #endif
 
 static struct option const long_options[] = {
@@ -146,8 +146,6 @@ static struct option const long_options[] = {
   { "p3p-cp",                           required_argument, NULL, 137 },
   { "p3p-uri",                          required_argument, NULL, 138 },
 
-  { "set-pcap-nonblocking",             no_argument,       NULL, 139 },
-
   { "instance",                         required_argument, NULL, 140 },
 
   { "disable-stopcap",                  no_argument,       NULL, 142 },
@@ -155,10 +153,10 @@ static struct option const long_options[] = {
   { "disable-mutexextrainfo",           no_argument,       NULL, 145 },
 
   { "fc-only",                          no_argument,       NULL, 147 },
-  { "no-fc",                            no_argument,       0, 148 },
-  { "no-invalid-lun",                   no_argument,       0, 149 },
+  { "no-fc",                            no_argument,       0, 148    },
+  { "no-invalid-lun",                   no_argument,       0, 149    },
 
-  { "skip-version-check",               no_argument,       NULL, 150 },
+  { "skip-version-check",               required_argument, NULL, 150 },
 
   {NULL, 0, NULL, 0}
 };
@@ -215,8 +213,7 @@ void loadPrefs(int argc, char* argv[]) {
 
     case 'P':
       stringSanityCheck(optarg, "-P | --db-file-path");
-      if(myGlobals.dbPath != NULL)
-	free(myGlobals.dbPath);
+      if(myGlobals.dbPath != NULL) free(myGlobals.dbPath);
 
       myGlobals.dbPath = strdup(optarg);
       break;
@@ -240,7 +237,7 @@ void loadPrefs(int argc, char* argv[]) {
     /* Handle key w/o trailing \0 so valgrind is happy */
     zeroPadMallocString(key.dsize, key.dptr);
 
-    if (fetchPrefsValue(key.dptr, buf, sizeof (buf)) == 0) {
+    if (fetchPrefsValue(key.dptr, buf, sizeof(buf)) == 0) {
       processNtopPref(key.dptr, buf, FALSE, &myGlobals.runningPref);
     }
 
@@ -249,11 +246,12 @@ void loadPrefs(int argc, char* argv[]) {
     key = nextkey;
   }
 
-  if(myGlobals.runningPref.mergeInterfaces != mergeInterfacesSave)
+  if(myGlobals.runningPref.mergeInterfaces != mergeInterfacesSave) {
     if(myGlobals.runningPref.mergeInterfaces == 0)
       traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge disabled from prefs file");
     else
       traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge enabled from prefs file");
+  }
 
   myGlobals.savedPref = myGlobals.runningPref;
 }
@@ -479,8 +477,8 @@ int parseOptions(int argc, char* argv[]) {
 
     case 'N':
       pathSanityCheck(optarg, "-N | --wwn-map");
-      if (myGlobals.runningPref.fcNSCacheFile != NULL)
-	free (myGlobals.runningPref.fcNSCacheFile);
+      if(myGlobals.runningPref.fcNSCacheFile != NULL)
+	   free(myGlobals.runningPref.fcNSCacheFile);
       myGlobals.runningPref.fcNSCacheFile = strdup (optarg);
       break;
 
@@ -496,6 +494,8 @@ int parseOptions(int argc, char* argv[]) {
       if(myGlobals.dbPath != NULL)
 	free(myGlobals.dbPath);
 
+      if(optarg[strlen(optarg)-1] == '/') optarg[strlen(optarg)-1] = '\0';
+      mkdir_p("dbPath", optarg, 0777);
       myGlobals.dbPath = strdup(optarg);
       break;
 
@@ -503,6 +503,8 @@ int parseOptions(int argc, char* argv[]) {
       pathSanityCheck(optarg, "-Q | --spool-file-path" );
       if(myGlobals.spoolPath != NULL)
 	free(myGlobals.spoolPath);
+      if(optarg[strlen(optarg)-1] == '/') optarg[strlen(optarg)-1] = '\0';
+      mkdir_p("spoolPath", optarg, 0777);
       myGlobals.spoolPath = strdup(optarg);
       break;
 
@@ -631,21 +633,9 @@ int parseOptions(int argc, char* argv[]) {
       myGlobals.runningPref.P3Puri = strdup(optarg);
       break;
 
-#ifndef WIN32
-    case 139:
- #ifdef HAVE_PCAP_SETNONBLOCK
-      printf("WARNING: --set-pcap-nonblocking requested\n");
-      myGlobals.runningPref.setNonBlocking = TRUE;
- #else
-      printf("FATAL ERROR: --set-pcap-nonblocking invalid - pcap_setnonblock() unavailable\n");
-      exit(-1);
- #endif
-      break;
-#endif
-
     case 140: /* instance */
     {
-      int idx, i, found;
+      int idx, found;
       struct stat statbuf;
       char fileName[64], tmpStr[512];
       FILE *fd;
@@ -655,8 +645,8 @@ int parseOptions(int argc, char* argv[]) {
         free(myGlobals.runningPref.instance);
       myGlobals.runningPref.instance = strdup(optarg);
 
-      memset(&tmpStr, 0, sizeof(tmpStr)); 
-      memset(&fileName, 0, sizeof(fileName)); 
+      memset(&tmpStr, 0, sizeof(tmpStr));
+      memset(&fileName, 0, sizeof(fileName));
       for(found=0, idx=0; (found != 1) && (myGlobals.dataFileDirs[idx] != NULL); idx++) {
         safe_snprintf(__FILE__, __LINE__, fileName, sizeof(fileName),
                       "%s_" CONST_NTOP_LOGO_GIF,
@@ -795,28 +785,28 @@ int parseOptions(int argc, char* argv[]) {
 
 /* ******************************** */
 
-int fetchPrefsValue(char *key, char *value, int valueLen) {
+static int fetchGdbmValue(GDBM_FILE gdbmfile, char *key, char *value, int valueLen) {
   datum key_data;
   datum data_data;
 
   if(value == NULL) return(-1);
 
 #ifdef DEBUG
-  traceEvent(CONST_TRACE_INFO, "DEBUG: Entering fetchPrefValue()");
+  traceEvent(CONST_TRACE_INFO, "DEBUG: Entering fetchGdbmValue()");
 #endif
   value[0] = '\0';
 
   key_data.dptr  = key;
   key_data.dsize = strlen(key_data.dptr)+1;
 
-  if(myGlobals.prefsFile == NULL) {
+  if(gdbmfile == NULL) {
 #ifdef DEBUG
-    traceEvent(CONST_TRACE_INFO, "DEBUG: Leaving fetchPrefValue()");
+    traceEvent(CONST_TRACE_INFO, "DEBUG: Leaving fetchGdbmValue()");
 #endif
     return(-1); /* ntop is quitting... */
   }
 
-  data_data = gdbm_fetch(myGlobals.prefsFile, key_data);
+  data_data = gdbm_fetch(gdbmfile, key_data);
 
   memset(value, 0, valueLen);
 
@@ -835,7 +825,19 @@ int fetchPrefsValue(char *key, char *value, int valueLen) {
 
 /* ******************************** */
 
-void storePrefsValue(char *key, char *value) {
+int fetchPrefsValue(char *key, char *value, int valueLen) {
+  return(fetchGdbmValue(myGlobals.prefsFile, key, value, valueLen));
+}
+
+/* ******************************** */
+
+int fetchPwValue(char *key, char *value, int valueLen) {
+  return(fetchGdbmValue(myGlobals.pwFile, key, value, valueLen));
+}
+
+/* ******************************** */
+
+static void storeGdbmValue(GDBM_FILE gdbmfile, char *key, char *value) {
   datum key_data;
   datum data_data;
 
@@ -853,14 +855,14 @@ void storePrefsValue(char *key, char *value) {
   data_data.dptr  = value;
   data_data.dsize = strlen(value)+1;
 
-  if(myGlobals.prefsFile == NULL) {
+  if(gdbmfile == NULL) {
 #ifdef DEBUG
     traceEvent(CONST_TRACE_INFO, "DEBUG: Leaving storePrefsValue()");
 #endif
     ; /* ntop is quitting... */
   }
 
-  if(gdbm_store(myGlobals.prefsFile, key_data, data_data, GDBM_REPLACE) != 0)
+  if(gdbm_store(gdbmfile, key_data, data_data, GDBM_REPLACE) != 0)
     traceEvent(CONST_TRACE_ERROR, "While adding %s=%s.", key, value);
   else {
 #ifdef DEBUG
@@ -871,7 +873,19 @@ void storePrefsValue(char *key, char *value) {
 
 /* ******************************** */
 
-void delPrefsValue (char *key) {
+void storePrefsValue(char *key, char *value) {
+  storeGdbmValue(myGlobals.prefsFile, key, value);
+}
+
+/* ******************************** */
+
+void storePwValue(char *key, char *value) {
+  storeGdbmValue(myGlobals.pwFile, key, value);
+}
+
+/* ******************************** */
+
+static void delGdbmValue(GDBM_FILE gdbmfile, char *key) {
   datum key_data;
 
   if((key == NULL) || (myGlobals.ntopRunState >= FLAG_NTOPSTATE_SHUTDOWN)) return;
@@ -884,16 +898,18 @@ void delPrefsValue (char *key) {
   key_data.dptr   = key;
   key_data.dsize  = strlen(key_data.dptr)+1;
 
-  if(myGlobals.prefsFile == NULL) {
+  if(gdbmfile == NULL) {
 #ifdef DEBUG
     traceEvent(CONST_TRACE_INFO, "DEBUG: Leaving storePrefsValue()");
 #endif
     ; /* ntop is quitting... */
   }
 
-  if(gdbm_delete (myGlobals.prefsFile, key_data) != 0)
-    traceEvent(CONST_TRACE_ERROR, "While deleting %s", key);
-  else {
+  if(gdbm_delete(gdbmfile, key_data) != 0) {
+#ifdef DEBUG
+    traceEvent(CONST_TRACE_ERROR, "While deleting %s: key not found", key);
+#endif
+  } else {
 #ifdef DEBUG
     traceEvent(CONST_TRACE_INFO, "Deleted %s", key);
 #endif
@@ -902,11 +918,23 @@ void delPrefsValue (char *key) {
 
 /* ******************************** */
 
+void delPrefsValue(char *key) {
+  delGdbmValue(myGlobals.prefsFile, key);
+}
+
+/* ******************************** */
+
+void delPwValue(char *key) {
+  delGdbmValue(myGlobals.pwFile, key);
+}
+
+/* ******************************** */
+
 void processStrPref (char *key, char *value, char **globalVar, bool savePref)
 {
   if (key == NULL) return;
 
-  if (strcmp (value, "") == 0) {
+  if (strcmp(value, "") == 0) {
     /* If a value is specified as NULL but the current value is not, delete
      * the pref. This is assumed to be the way the user will change such a
      * pref.
@@ -927,7 +955,7 @@ void processStrPref (char *key, char *value, char **globalVar, bool savePref)
 	/* Values can be concatenated */
 	char tmpValue[256];
 
-	safe_snprintf(__FILE__, __LINE__, tmpValue, sizeof(tmpValue), 
+	safe_snprintf(__FILE__, __LINE__, tmpValue, sizeof(tmpValue),
 		      "%s,%s", *globalVar, value);
 	storePrefsValue (key, tmpValue);
 	free(*globalVar);
@@ -953,15 +981,15 @@ void processIntPref (char *key, char *value, int *globalVar, bool savePref)
 {
   char buf[512];
 
-  if ((key == NULL) || (value == NULL)) return;
+  if((key == NULL) || (value == NULL)) return;
 
-  if (savePref) {
-    safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf),
-		   "%d", atoi (value));
+  *globalVar = atoi(value);
+
+  if(savePref) {
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),
+		  "%d", *globalVar);
     storePrefsValue (key, buf);
   }
-
-  *globalVar = atoi (value);
 }
 
 /* ******************************** */
@@ -973,7 +1001,7 @@ void processBoolPref (char *key, bool value, bool *globalVar, bool savePref)
   if (key == NULL) return;
 
   if (savePref) {
-    safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf),
+    safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf),
 		   "%d", value);
     storePrefsValue (key, buf);
   }
@@ -999,32 +1027,33 @@ bool processNtopPref (char *key, char *value, bool savePref, UserPref *pref) {
 
   if(value == NULL) value = ""; /* Safer */
 
-  if (strcmp(key, NTOP_PREF_DEVICES) == 0) {
-    if ((pref->devices != NULL) &&
-	(strcmp (pref->devices, value))) {
+  /* traceEvent(CONST_TRACE_ERROR, "==>> processNtopPref [%s][%s]", key, value); */
+  
+  if(strcmp(key, NTOP_PREF_DEVICES) == 0) {
+    if((pref->devices != NULL) && (strcmp(pref->devices, value))) {
       startCap = TRUE;
     }
 
     if((pref->devices == NULL) || (strstr(pref->devices, value) == NULL))
       processStrPref (NTOP_PREF_DEVICES, value, &pref->devices, savePref);
-  } else if(strcmp (key, NTOP_PREF_CAPFILE) == 0) {
-    if (((value != NULL) &&
-	 (((pref->rFileName != NULL) && (strcmp (pref->rFileName, value)))))
+  } else if(strcmp(key, NTOP_PREF_CAPFILE) == 0) {
+    if(((value != NULL) &&
+	 (((pref->rFileName != NULL) && (strcmp(pref->rFileName, value)))))
 	|| ((value != NULL) && ((pref->rFileName == NULL)))) {
       startCap = TRUE;
     }
     processStrPref (NTOP_PREF_CAPFILE, value, &pref->rFileName, savePref);
-  } else if(strcmp (key, NTOP_PREF_FILTER) == 0) {
+  } else if(strcmp(key, NTOP_PREF_FILTER) == 0) {
     processStrPref (NTOP_PREF_FILTER, value, &pref->currentFilterExpression, savePref);
-  } else if(strcmp (key, NTOP_PREF_SAMPLING) == 0) {
+  } else if(strcmp(key, NTOP_PREF_SAMPLING) == 0) {
     int sampleRate;
     processIntPref (NTOP_PREF_SAMPLING, value, &sampleRate, savePref);
     pref->samplingRate = (u_short)sampleRate;
-  } else if(strcmp (key, NTOP_PREF_WEBPORT) == 0) {
-    if (value != NULL) {
+  } else if(strcmp(key, NTOP_PREF_WEBPORT) == 0) {
+    if(value != NULL) {
       stringSanityCheck(value, "-w | --http-server");
       if(!isdigit(*value)) {
-	traceEvent (CONST_TRACE_ERROR, "flag -w expects a numeric argument.\n");
+	traceEvent(CONST_TRACE_ERROR, "flag -w expects a numeric argument.\n");
 	return(startCap);
       }
 
@@ -1032,7 +1061,7 @@ bool processNtopPref (char *key, char *value, bool savePref, UserPref *pref) {
       if((pref->webAddr = strchr(value,':'))) {
 	/* DS: Search for : to find xxx.xxx.xxx.xxx:port */
 	/* This code is to be able to bind to a particular interface */
-	if (savePref) {
+	if(savePref) {
 	  storePrefsValue (key, value);
 	}
 	*pref->webAddr = '\0';
@@ -1042,20 +1071,19 @@ bool processNtopPref (char *key, char *value, bool savePref, UserPref *pref) {
       else {
 	processIntPref (NTOP_PREF_WEBPORT, value, &pref->webPort, savePref);
       }
-    }
-    else {
-      safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf), "%d",
+    } else {
+      safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
 		     DEFAULT_NTOP_WEB_PORT);
       value = buf;
       processIntPref (NTOP_PREF_WEBPORT, value, &pref->webPort, savePref);
     }
   }
 #ifdef HAVE_OPENSSL
-  else if (strcmp (key, NTOP_PREF_SSLPORT) == 0) {
-    if (value != NULL) {
+  else if(strcmp(key, NTOP_PREF_SSLPORT) == 0) {
+    if(value != NULL) {
       stringSanityCheck(value, "-W | --https-server");
       if(!isdigit(*value)) {
-	traceEvent (CONST_TRACE_ERROR, "flag -W expects a numeric argument.\n");
+	traceEvent(CONST_TRACE_ERROR, "flag -W expects a numeric argument.\n");
 	return(startCap);
       }
 
@@ -1064,7 +1092,7 @@ bool processNtopPref (char *key, char *value, bool savePref, UserPref *pref) {
       if((pref->sslAddr = strchr(tmpStr,':'))) {
 	/* DS: Search for : to find xxx.xxx.xxx.xxx:port */
 	/* This code is to be able to bind to a particular interface */
-	if (savePref) {
+	if(savePref) {
 	  storePrefsValue (key, value);
 	}
 	*pref->sslAddr = '\0';
@@ -1075,166 +1103,181 @@ bool processNtopPref (char *key, char *value, bool savePref, UserPref *pref) {
       }
       free(tmpStr);
     }
-    if (value == NULL) {
-      safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf), "%d",
+    if(value == NULL) {
+      safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
 		     DEFAULT_NTOP_WEB_PORT);
       value = buf;
       processIntPref (NTOP_PREF_SSLPORT, value, &pref->sslPort, savePref);
     }
   }
 #endif
-  else if (strcmp (key, NTOP_PREF_EN_SESSION) == 0) {
+  else if(strcmp(key, NTOP_PREF_EN_SESSION) == 0) {
     processBoolPref (NTOP_PREF_EN_SESSION, value2bool(value),
 		     &pref->enableSessionHandling, savePref);
-  } else if(strcmp (key, NTOP_PREF_EN_PROTO_DECODE) == 0) {
+  } else if(strcmp(key, NTOP_PREF_EN_PROTO_DECODE) == 0) {
     processBoolPref (NTOP_PREF_EN_PROTO_DECODE, value2bool(value),
 		     &pref->enablePacketDecoding, savePref);
-  } else if(strcmp (key, NTOP_PREF_FLOWSPECS) == 0) {
+  } else if(strcmp(key, NTOP_PREF_FLOWSPECS) == 0) {
     processStrPref (NTOP_PREF_FLOWSPECS, value, &pref->flowSpecs, savePref);
-  } else if(strcmp (key, NTOP_PREF_LOCALADDR) == 0) {
+  } else if(strcmp(key, NTOP_PREF_LOCALADDR) == 0) {
     processStrPref (NTOP_PREF_LOCALADDR, value, &pref->localAddresses,
 		    savePref);
-  } else if(strcmp (key, NTOP_PREF_STICKY_HOSTS) == 0) {
+  } else if(strcmp(key, NTOP_PREF_STICKY_HOSTS) == 0) {
     processBoolPref (NTOP_PREF_STICKY_HOSTS, value2bool(value), &pref->stickyHosts,
 		     savePref);
-  } else if(strcmp (key, NTOP_PREF_TRACK_LOCAL) == 0) {
+  } else if(strcmp(key, NTOP_PREF_TRACK_LOCAL) == 0) {
     processBoolPref (NTOP_PREF_TRACK_LOCAL, value2bool(value),
 		     &pref->trackOnlyLocalHosts, savePref);
-  } else if(strcmp (key, NTOP_PREF_NO_PROMISC) == 0) {
+  } else if(strcmp(key, NTOP_PREF_NO_PROMISC) == 0) {
     processBoolPref (NTOP_PREF_NO_PROMISC, value2bool(value),
 		     &pref->disablePromiscuousMode, savePref);
-  } else if(strcmp (key, NTOP_PREF_DAEMON) == 0) {
+  } else if(strcmp(key, NTOP_PREF_DAEMON) == 0) {
     processBoolPref (NTOP_PREF_DAEMON, value2bool(value), &pref->daemonMode,
 		     savePref);
-  } else if(strcmp (key, NTOP_PREF_REFRESH_RATE) == 0) {
-    if (value == NULL) {
-      safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf), "%d",
+  } else if(strcmp(key, NTOP_PREF_REFRESH_RATE) == 0) {
+    if(value == NULL) {
+      safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
 		     DEFAULT_NTOP_AUTOREFRESH_INTERVAL);
       value = buf;
     }
     processIntPref (NTOP_PREF_REFRESH_RATE, value, &pref->refreshRate,
 		    savePref);
-  } else if(strcmp (key, NTOP_PREF_MAXLINES) == 0) {
-    if (value == NULL) {
-      safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf), "%d",
+  } else if(strcmp(key, NTOP_PREF_MAXLINES) == 0) {
+    if(value == NULL) {
+      safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
 		     CONST_NUM_TABLE_ROWS_PER_PAGE);
       value = buf;
     }
     processIntPref (NTOP_PREF_MAXLINES, value, &pref->maxNumLines,
 		    savePref);
-  } else if(strcmp (key, NTOP_PREF_PRINT_FCORIP) == 0) {
+  } else if(strcmp(key, NTOP_PREF_PRINT_FCORIP) == 0) {
     tmpInt = atoi (value);
-    if (tmpInt == NTOP_PREF_VALUE_PRINT_IPONLY) {
+    if(tmpInt == NTOP_PREF_VALUE_PRINT_IPONLY) {
       pref->printIpOnly = TRUE, pref->printFcOnly = FALSE;
-    }
-    else if (tmpInt == NTOP_PREF_VALUE_PRINT_FCONLY) {
+    } else if(tmpInt == NTOP_PREF_VALUE_PRINT_FCONLY) {
       pref->printIpOnly = FALSE, pref->printFcOnly = TRUE;
-    }
-    else {
+    } else {
       pref->printIpOnly = FALSE, pref->printFcOnly = FALSE;
     }
 
     processIntPref (NTOP_PREF_PRINT_FCORIP, value, &tmpInt, savePref);
-  } else if(strcmp (key, NTOP_PREF_NO_INVLUN) == 0) {
+  } else if(strcmp(key, NTOP_PREF_NO_INVLUN) == 0) {
     processBoolPref (NTOP_PREF_NO_INVLUN, value2bool(value),
 		     &pref->noInvalidLunDisplay, savePref);
-  } else if(strcmp (key, NTOP_PREF_W3C) == 0) {
+  } else if(strcmp(key, NTOP_PREF_W3C) == 0) {
     processBoolPref (NTOP_PREF_W3C, value2bool(value), &pref->w3c, savePref);
-  } else if(strcmp (key, NTOP_PREF_IPV4V6) == 0) {
+  } else if(strcmp(key, NTOP_PREF_IPV4V6) == 0) {
     processIntPref (NTOP_PREF_IPV4V6, value, &pref->ipv4or6, savePref);
-  } else if(strcmp (key, NTOP_PREF_DOMAINNAME) == 0) {
+  } else if(strcmp(key, NTOP_PREF_DOMAINNAME) == 0) {
     processStrPref (NTOP_PREF_DOMAINNAME, value, &tmpStr,
 		    savePref);
-    if (tmpStr != NULL) {
-      strncpy (pref->domainName, tmpStr, sizeof (pref->domainName));
+    if(tmpStr != NULL) {
+      strncpy (pref->domainName, tmpStr, sizeof(pref->domainName));
       free (tmpStr);      /* alloc'd in processStrPref() */
     }
-  } else if(strcmp (key, NTOP_PREF_NUMERIC_IP) == 0) {
+  } else if(strcmp(key, NTOP_PREF_NUMERIC_IP) == 0) {
     processBoolPref (NTOP_PREF_NUMERIC_IP, value2bool(value), &pref->numericFlag,
 		     savePref);
-  } else if(strcmp (key, NTOP_PREF_PROTOSPECS) == 0) {
+  } else if(strcmp(key, NTOP_PREF_PROTOSPECS) == 0) {
     processStrPref (NTOP_PREF_PROTOSPECS, value, &pref->protoSpecs,
 		    savePref);
-  } else if(strcmp (key, NTOP_PREF_P3PCP) == 0) {
+  } else if(strcmp(key, NTOP_PREF_P3PCP) == 0) {
     processStrPref (NTOP_PREF_P3PCP, value, &pref->P3Pcp, savePref);
-  } else if(strcmp (key, NTOP_PREF_P3PURI) == 0) {
+  } else if(strcmp(key, NTOP_PREF_P3PURI) == 0) {
     processStrPref (NTOP_PREF_P3PURI, value, &pref->P3Puri, savePref);
-  } else if(strcmp (key, NTOP_PREF_MAPPERURL) == 0) {
-    processStrPref (NTOP_PREF_MAPPERURL, value, &pref->mapperURL, savePref);
-  } else if(strcmp (key, NTOP_PREF_WWN_MAP) == 0) {
+  } else if(strcmp(key, NTOP_PREF_WWN_MAP) == 0) {
     processStrPref (NTOP_PREF_WWN_MAP, value, &pref->fcNSCacheFile,
 		    savePref);
-  } else if(strcmp (key, NTOP_PREF_MAXHASH) == 0) {
-    if (value == NULL) {
-      safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf), "%d",
+  } else if(strcmp(key, NTOP_PREF_MAXHASH) == 0) {
+    if(value == NULL) {
+      safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
 		     -1);
       value = buf;
     }
     processIntPref(NTOP_PREF_MAXHASH, value,
 		   (int*)&pref->maxNumHashEntries, savePref);
-  } else if(strcmp (key, NTOP_PREF_MAXSESSIONS) == 0) {
-    if (value == NULL) {
-      safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf), "%d",
+  } else if(strcmp(key, NTOP_PREF_SQL_DB_CONFIG) == 0) {
+    processStrPref (NTOP_PREF_SQL_DB_CONFIG, value, &tmpStr, savePref);
+    if(tmpStr != NULL) {
+      strncpy(pref->sqlDbConfig, tmpStr, sizeof(pref->sqlDbConfig));
+      free(tmpStr);      /* alloc'd in processStrPref() */
+    }
+  } else if(strcmp(key, NTOP_PREF_SQL_REC_LIFETIME) == 0) {
+    if(value == NULL) {
+      safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), 
+		    "%d",
+		    -1);
+      value = buf;
+    }
+    processIntPref(NTOP_PREF_SQL_REC_LIFETIME, value,
+		   (int*)&pref->sqlRecDaysLifetime, savePref);
+  } else if(strcmp(key, NTOP_PREF_SAVE_REC_INTO_DB) == 0) {
+    processBoolPref(NTOP_PREF_SAVE_REC_INTO_DB, value2bool(value),
+		    &pref->saveRecordsIntoDb, savePref);
+    myGlobals.runningPref.saveRecordsIntoDb = pref->saveRecordsIntoDb; /* activate immediately */
+  } else if(strcmp(key, NTOP_PREF_SAVE_SESSIONS_INTO_DB) == 0) {
+    processBoolPref(NTOP_PREF_SAVE_SESSIONS_INTO_DB, value2bool(value),
+		    &pref->saveSessionsIntoDb, savePref);   
+    myGlobals.runningPref.saveSessionsIntoDb = pref->saveSessionsIntoDb; /* activate immediately */
+  } else if(strcmp(key, NTOP_PREF_MAXSESSIONS) == 0) {
+    if(value == NULL) {
+      safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
 		     -1);
       value = buf;
     }
     processIntPref (NTOP_PREF_MAXSESSIONS, value,
 		    (int*)&pref->maxNumSessions, savePref);
-  } else if(strcmp (key, NTOP_PREF_MERGEIF) == 0) {
+  } else if(strcmp(key, NTOP_PREF_MERGEIF) == 0) {
     processBoolPref (NTOP_PREF_MERGEIF, value2bool(value),
 		     &pref->mergeInterfaces, savePref);
-  } else if(strcmp (key, NTOP_PREF_NO_ISESS_PURGE) == 0) {
-    processBoolPref (NTOP_PREF_NO_ISESS_PURGE, value2bool(value),
+  } else if(strcmp(key, NTOP_PREF_MERGEIF) == 0) {
+    processBoolPref (NTOP_PREF_MERGEIF, value2bool(value),
+		     &pref->mergeInterfaces, savePref);
+  } else if(strcmp(key, NTOP_PREF_ENABLE_L7PROTO) == 0) {
+    processBoolPref (NTOP_PREF_ENABLE_L7PROTO, value2bool(value),
+		     &pref->enableL7, savePref);
+  } else if(strcmp(key, NTOP_PREF_NO_ISESS_PURGE) == 0) {
+    processBoolPref(NTOP_PREF_NO_ISESS_PURGE, value2bool(value),
 		     &pref->disableInstantSessionPurge, savePref);
-  }
-#if !defined(WIN32) && defined(HAVE_PCAP_SETNONBLOCK)
-  else if (strcmp (key, NTOP_PREF_NOBLOCK) == 0) {
-    processBoolPref (NTOP_PREF_NOBLOCK, value2bool(value),
-		     &pref->setNonBlocking, savePref);
-  }
-#endif
-  else if (strcmp (key, NTOP_PREF_NO_STOPCAP) == 0) {
-    processBoolPref (NTOP_PREF_NO_STOPCAP, value2bool(value),
-		     &pref->disableStopcap, savePref);
-  } else if(strcmp (key, NTOP_PREF_NO_TRUST_MAC) == 0) {
-    processBoolPref (NTOP_PREF_NO_TRUST_MAC, value2bool(value),
+  } else if(strcmp(key, NTOP_PREF_NO_TRUST_MAC) == 0) {
+    processBoolPref(NTOP_PREF_NO_TRUST_MAC, value2bool(value),
 		     &pref->dontTrustMACaddr, savePref);
-  } else if(strcmp (key, NTOP_PREF_PCAP_LOGBASE) == 0) {
+  } else if(strcmp(key, NTOP_PREF_PCAP_LOGBASE) == 0) {
     processStrPref (NTOP_PREF_PCAP_LOGBASE, value,
 		    &pref->pcapLogBasePath, savePref);
   }
 #ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-  else if (strcmp (key, NTOP_PREF_USE_SSLWATCH) == 0) {
-    processBoolPref (NTOP_PREF_USE_SSLWATCH, value2bool(value),
+  else if(strcmp(key, NTOP_PREF_USE_SSLWATCH) == 0) {
+    processBoolPref(NTOP_PREF_USE_SSLWATCH, value2bool(value),
 		     &pref->useSSLwatchdog, savePref);
   }
 #endif
-  else if (strcmp (key, NTOP_PREF_DBG_MODE) == 0) {
-    processBoolPref (NTOP_PREF_DBG_MODE, value2bool(value), &pref->debugMode,
+  else if(strcmp(key, NTOP_PREF_DBG_MODE) == 0) {
+    processBoolPref(NTOP_PREF_DBG_MODE, value2bool(value), &pref->debugMode,
 		     savePref);
-  } else if(strcmp (key, NTOP_PREF_TRACE_LVL) == 0) {
-    if (value == NULL) {
-      safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf), "%d",
+  } else if(strcmp(key, NTOP_PREF_TRACE_LVL) == 0) {
+    if(value == NULL) {
+      safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
 		     DEFAULT_TRACE_LEVEL);
       value = buf;
     }
     processIntPref (NTOP_PREF_TRACE_LVL, value, &pref->traceLevel,
 		    savePref);
-  } else if(strcmp (key, NTOP_PREF_DUMP_OTHER) == 0) {
-    processBoolPref (NTOP_PREF_DUMP_OTHER, value2bool(value),
+  } else if(strcmp(key, NTOP_PREF_DUMP_OTHER) == 0) {
+    processBoolPref(NTOP_PREF_DUMP_OTHER, value2bool(value),
 		     &pref->enableOtherPacketDump, savePref);
-  } else if(strcmp (key, NTOP_PREF_DUMP_SUSP) == 0) {
-    processBoolPref (NTOP_PREF_DUMP_SUSP, value2bool(value),
+  } else if(strcmp(key, NTOP_PREF_DUMP_SUSP) == 0) {
+    processBoolPref(NTOP_PREF_DUMP_SUSP, value2bool(value),
 		     &pref->enableSuspiciousPacketDump, savePref);
-  } else if(strcmp (key, NTOP_PREF_ACCESS_LOG) == 0) {
+  } else if(strcmp(key, NTOP_PREF_ACCESS_LOG) == 0) {
     processStrPref (NTOP_PREF_ACCESS_LOG, value,
 		    &pref->accessLogFile,
 		    savePref);
   }
 #ifndef WIN32
-  else if (strcmp (key, NTOP_PREF_USE_SYSLOG) == 0) {
-    if (value == NULL) {
-      safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf), "%d",
+  else if(strcmp(key, NTOP_PREF_USE_SYSLOG) == 0) {
+    if(value == NULL) {
+      safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
 		     DEFAULT_NTOP_SYSLOG);
       value = buf;
     }
@@ -1242,20 +1285,20 @@ bool processNtopPref (char *key, char *value, bool savePref, UserPref *pref) {
 		    &pref->useSyslog, savePref);
   }
 #endif
-  else if (strcmp (key, NTOP_PREF_PCAP_LOG) == 0) {
+  else if(strcmp(key, NTOP_PREF_PCAP_LOG) == 0) {
     processStrPref (NTOP_PREF_PCAP_LOG, value, &pref->pcapLog, savePref);
-  } else if(strcmp (key, NTOP_PREF_NO_MUTEX_EXTRA) == 0) {
-    processBoolPref (NTOP_PREF_NO_MUTEX_EXTRA, value2bool(value),
+  } else if(strcmp(key, NTOP_PREF_NO_MUTEX_EXTRA) == 0) {
+    processBoolPref(NTOP_PREF_NO_MUTEX_EXTRA, value2bool(value),
 		     &pref->disableMutexExtraInfo, savePref);
   }
 #ifdef MAKE_WITH_SCHED_YIELD
-  else if (strcmp (key, NTOP_PREF_NO_SCHEDYLD) == 0) {
-    processBoolPref (NTOP_PREF_NO_SCHEDYLD, value2bool(value),
+  else if(strcmp(key, NTOP_PREF_NO_SCHEDYLD) == 0) {
+    processBoolPref(NTOP_PREF_NO_SCHEDYLD, value2bool(value),
 		     &pref->disableSchedYield, savePref);
   }
 #endif
-  else if (strncmp (key, "ntop.", strlen ("ntop.")) == 0) {
-    traceEvent (CONST_TRACE_WARNING, "Unknown preference: %s, value = %s\n",
+  else if(strncmp (key, "ntop.", strlen ("ntop.")) == 0) {
+    traceEvent(CONST_TRACE_WARNING, "Unknown preference: %s, value = %s\n",
 		key, (value == NULL) ? "(null)" : value);
   }
 
@@ -1285,8 +1328,12 @@ void initUserPrefs(UserPref *pref) {
   pref->refreshRate = DEFAULT_NTOP_AUTOREFRESH_INTERVAL;
   pref->disablePromiscuousMode = DEFAULT_NTOP_DISABLE_PROMISCUOUS;
   pref->traceLevel = DEFAULT_TRACE_LEVEL;
-  pref->maxNumHashEntries = DEFAULT_NTOP_MAX_HASH_ENTRIES;
-  pref->maxNumSessions    = DEFAULT_NTOP_MAX_NUM_SESSIONS;
+  pref->maxNumHashEntries  = DEFAULT_NTOP_MAX_HASH_ENTRIES;
+  pref->maxNumSessions     = DEFAULT_NTOP_MAX_NUM_SESSIONS;
+  strncpy(pref->sqlDbConfig, DEFAULT_NTOP_SQL_DB_CONFIG, sizeof(pref->sqlDbConfig));
+  pref->sqlRecDaysLifetime = DEFAULT_NTOP_SQL_REC_DAYS_LIFETIME;
+  pref->saveRecordsIntoDb  = DEFAULT_NTOP_SAVE_REC_INTO_DB;
+  pref->saveSessionsIntoDb = DEFAULT_NTOP_SAVE_SESSIONS_INTO_DB;
   pref->webAddr = DEFAULT_NTOP_WEB_ADDR;
   pref->webPort = DEFAULT_NTOP_WEB_PORT;
   pref->ipv4or6 = DEFAULT_NTOP_FAMILY;
@@ -1324,16 +1371,13 @@ void initUserPrefs(UserPref *pref) {
    pref->P3Pcp  = DEFAULT_NTOP_P3PCP;
    pref->P3Puri = DEFAULT_NTOP_P3PURI;
 
-#if !defined(WIN32) && defined(HAVE_PCAP_SETNONBLOCK)
-   pref->setNonBlocking = DEFAULT_NTOP_SETNONBLOCK;
-#endif
    pref->disableStopcap = DEFAULT_NTOP_DISABLE_STOPCAP;
    pref->disableInstantSessionPurge = DEFAULT_NTOP_DISABLE_IS_PURGE;
    pref->printIpOnly = DEFAULT_NTOP_PRINTIPONLY;
    pref->printFcOnly = DEFAULT_NTOP_PRINTFCONLY;
-   pref->noInvalidLunDisplay = DEFAULT_NTOP_NO_INVLUN_DISPLAY;
+   pref->noInvalidLunDisplay   = DEFAULT_NTOP_NO_INVLUN_DISPLAY;
    pref->disableMutexExtraInfo = DEFAULT_NTOP_DISABLE_MUTEXINFO;
-   pref->skipVersionCheck = DEFAULT_NTOP_SKIP_VERSION_CHECK;
+   pref->skipVersionCheck      = DEFAULT_NTOP_SKIP_VERSION_CHECK;
 }
 
 /* *******************************/
