@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -168,14 +168,17 @@ void handleBootp(HostTraffic *srcHost,
 		strncpy(realDstHost->hostNumIpAddress,
 			_intoa(realDstHost->hostIp4Address, buf, sizeof(buf)),
 			sizeof(realDstHost->hostNumIpAddress));
-		if(myGlobals.runningPref.numericFlag == 0) ipaddr2str(realDstHost->hostIpAddress, 1);
+		if(myGlobals.runningPref.numericFlag == 0) {
+			realDstHost->hostIpAddress.hostFamily = AF_INET;
+			ipaddr2str(realDstHost->hostIpAddress, 1);
+		}
                 if (realDstHost->dnsDomainValue != NULL) free(realDstHost->dnsDomainValue);
 		realDstHost->dnsDomainValue = NULL;
                 if (realDstHost->dnsTLDValue != NULL) free(realDstHost->dnsTLDValue);
 		realDstHost->dnsTLDValue = NULL;
 		if (realDstHost->ip2ccValue != NULL) free(realDstHost->ip2ccValue);
 		realDstHost->ip2ccValue = NULL;
-		if(isBroadcastAddress(&realDstHost->hostIpAddress))
+		if(isBroadcastAddress(&realDstHost->hostIpAddress, NULL, NULL))
 		  FD_SET(FLAG_BROADCAST_HOST, &realDstHost->flags);
 		else
 		  FD_CLR(FLAG_BROADCAST_HOST, &realDstHost->flags);
@@ -266,7 +269,11 @@ void handleBootp(HostTraffic *srcHost,
 			/* Fix courtesy of Christoph Zens <chris@topfen.homeip.net> */
 			if(len >= MAX_LEN_SYM_HOST_NAME) len = MAX_LEN_SYM_HOST_NAME-1;
 			tmpName[len] = '\0';
-			for(i=0; i<strlen(tmpName); i++) if(isupper(tmpName[i])) tolower(tmpName[i]);
+			for(i=0; i<strlen(tmpName); i++) {
+			  if(isupper(tmpName[i]))
+			    tmpName[i] = tolower(tmpName[i]);
+			}
+
                         setResolvedName(realDstHost, tmpName, FLAG_HOST_SYM_ADDR_TYPE_NAME);
 			fillDomainName(realDstHost);
 		      }
