@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -538,7 +538,7 @@ void updateThpt(int fullUpdate) {
 /* Check if a host can be potentially added the host matrix */
 int isMatrixHost(HostTraffic *host, int actualDeviceId) {
   if((host->hostIpAddress.hostFamily == AF_INET) /* Only IPv4 addresses are used in the matrix */
-     && (deviceLocalAddress(&host->hostIpAddress, actualDeviceId) || multicastHost(host))
+     && (deviceLocalAddress(&host->hostIpAddress, actualDeviceId, NULL, NULL) || multicastHost(host))
      && (!broadcastHost(host)))
     return(1);
   else
@@ -727,3 +727,61 @@ int isInitialFtpData(char* packetData) {
   else
     return(0);
 }
+
+/* ********************************** */
+
+char* findHostCommunity(u_int32_t host_ip, char *buf, u_short buf_len) {
+  datum key, nextkey;
+  int len = strlen(COMMUNITY_PREFIX);
+
+
+  key = gdbm_firstkey(myGlobals.prefsFile);
+  while (key.dptr) {
+    char val[256], localAddresses[1024], *communityName;
+    u_int32_t localNetworks[MAX_NUM_NETWORKS][4]; /* [0]=network, [1]=mask, [2]=broadcast, [3]=mask_v6 */
+    u_short numLocalNetworks = 0, i;
+    
+    if((fetchPrefsValue(key.dptr, val, sizeof(val)) == 0)
+       && (!strncmp(key.dptr, COMMUNITY_PREFIX, len))) {
+      localAddresses[0] = '\0';
+      communityName = (char*)&key.dptr[len];
+
+      handleAddressLists(val, localNetworks, &numLocalNetworks,
+			 localAddresses, sizeof(localAddresses),
+			 CONST_HANDLEADDRESSLISTS_COMMUNITY);
+
+      // traceEvent(CONST_TRACE_WARNING, "--> Community %s has %d entries", communityName, numLocalNetworks);
+      for(i=0; i<numLocalNetworks; i++) {
+	if((host_ip & localNetworks[i][1]) == localNetworks[i][0]) {
+	  //traceEvent(CONST_TRACE_WARNING, "--> Found community %s [%d]", communityName, numLocalNetworks);
+	  snprintf(buf, buf_len, "%s", communityName);
+	  return(buf);
+	}
+      }
+    }
+
+    nextkey = gdbm_nextkey(myGlobals.prefsFile, key);
+    free (key.dptr);
+    key = nextkey;
+  }
+
+  return(NULL);
+}
+
+/* ********************************** */
+
+void setHostCommunity(HostTraffic *el) {
+  char *community, buf[64];
+
+  if((el == NULL) || (el->hostIpAddress.hostFamily != AF_INET))
+    return; /* Only IPv4 is supported */
+  else if(el->community != NULL)
+    return; /* Already set */
+
+  community = findHostCommunity(el->hostIpAddress.addr._hostIp4Address.s_addr,
+				buf, sizeof(buf));
+
+  if(community)
+    el->community = strdup(community);
+}
+
