@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2007 Luca Deri <deri@ntop.org>
  *
  * 			    http://www.ntop.org/
  *
@@ -556,9 +556,10 @@ void createVendorTable(struct stat *dbStat) {
   } /* for macInputFiles */
 
   if (!myGlobals.runningPref.printFcOnly) {
-      traceEvent(CONST_TRACE_INFO, "Fingeprint: Loading signature file.");
+      traceEvent(CONST_TRACE_INFO, "Fingerprint: Loading signature file");
       
-      fd = checkForInputFile(NULL, NULL, CONST_OSFINGERPRINT_FILE, NULL, &compressedFormat);
+      fd = checkForInputFile("Fingerprint", "Fingerprint file...",
+			     CONST_OSFINGERPRINT_FILE, NULL, &compressedFormat);
 
       if(fd != NULL) {
           char line[384], lineKey[8];
@@ -581,7 +582,7 @@ void createVendorTable(struct stat *dbStat) {
                   traceEvent(CONST_TRACE_ERROR, "While adding %s=%s.", lineKey, line);       
           }
           
-          traceEvent(CONST_TRACE_INFO, "Fingeprint: ...loaded %d records", numEntries);
+          traceEvent(CONST_TRACE_INFO, "Fingerprint: ...loaded %d records", numEntries);
       } else
           traceEvent(CONST_TRACE_NOISY, "Unable to find fingeprint signature file.");
   }
